/* File: gsave.c

	SHARP MZ-2000/2200/80B/B2 Emulator "emz2000"
		Output Graphics Files Modules

	Copyright (C) 1996-2019 FUKUI, Toshio.
	This file is part of the emz2000 software.
	See copyright notice in the COPYING file.
*/

#include "config.h"

#include <stdio.h>
#ifdef HAVE_STRING_H
#include <string.h>
#else
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif
#endif /* HAVE_STRING_H */
#ifdef HAVE_MALLOC_H
#include <malloc.h>
#endif
#include <unistd.h>

#ifdef HAVE_LIBGIF
#include <gif_lib.h>
#endif
#ifdef HAVE_LIBUNGIF
#include <gif_lib.h>
#endif

#ifndef HAVE_STRING_H
#define memset(X,Y,Z) bzero((X),(Z))
#define memcpy(X,Y,Z) bcopy((Y),(X),(Z))
#define strchr index
#define strrchr rindex
#endif

#include "common.h"
#include "gsave.h"
#include "bmpstruct.h"

/* gif write only parameters */
static int _interlace_flag = TRUE;

#if defined(HAVE_LIBGIF) || defined(HAVE_LIBUNGIF)
static int InterlacedOffset[] = { 0, 4, 2, 1 };
static int InterlacedJumps[] = { 8, 8, 4, 2 };
#endif

int gsave_writefile_bmp( unsigned int attr, const char *filename,
	int sx, int sy, const unsigned char *p )
{
	int j, k, x, y, ct;
	/* int index, tflag; */
	int bitmap_color, col_size, col_bit;
	unsigned int t;
	unsigned char r, g, b; /* , a; */
	unsigned char *buf, *bitdata, *cp;
	/* unsigned short cs; */
	FILE *fp;
	BITMAPFILEHEADER *filehead;
	BITMAPINFOHEADER *fileinfo;
	RGBQUAD *rgbcol;
	int planes = 3;
	int bits = 8;

	if (0 /* have palette */) {
#if 0
		if (planes != 1) {
			if (!(attr & GSAVE_ATTR_NOMSG))
				ERRMSG("gsave_writefile_bmp() can't save the bitmap file. (palette and planes != 1)\n");
			return FALSE;
		}
		if (p->u.i.pi->u.pi.bits == 1) {
			bitmap_color = 1;
			col_size = 2;
			col_bit  = 1;
		} else if (p->u.i.pi->u.pi.bits == 4) {
			bitmap_color = 4;
			col_size = 16;
			col_bit  = 4;
		} else if (p->u.i.pi->u.pi.bits == 8) {
			bitmap_color = 8;
			col_size = 256;
			col_bit  = 8;
		} else {
			if (!(attr & GSAVE_ATTR_NOMSG))
				ERRMSG("gsave_writefile_bmp() can't save the bitmap file. (palette and unknown bits)\n");
			return FALSE;
		}
		/* palette_image = ... */
#endif
	} else {
		if (planes != 3) {
			if (!(attr & GSAVE_ATTR_NOMSG))
				ERRMSG("gsave_writefile_bmp() can't save the bitmap file. (planes != 3)\n");
			return FALSE;
		}
		if (bits == 8) {
			bitmap_color = 24;
			col_size = 0;
			col_bit  = 24;
		} else if (bits == 5) {
			bitmap_color = 15;
			col_size = 0;
			col_bit  = 15;
		} else {
			if (!(attr & GSAVE_ATTR_NOMSG))
				ERRMSG("gsave_writefile_bmp() can't save the bitmap file. (nopalette and unknown bits)\n");
			return FALSE;
		}
	}
	fp = fopen(filename, "wb");
	if (!fp) {
		if (!(attr & GSAVE_ATTR_NOMSG))
			ERRMSG("gsave_writefile_bmp() can't write open the file \"%s\"\n", filename);
		return FALSE;
	}

	/* make bitmap memory */
	x = sx;
	y = sy;
	buf = MALLOC( sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) +
		col_size*sizeof(RGBQUAD) + ((col_bit+7)/8)*(x + 10)*y );
	if (!buf) {
		if (!(attr & GSAVE_ATTR_NOMSG))
			ERRMSG("gsave_writefile_bmp() can't allocate memory. (bitmap %d*%d)\n", x, y);
		fclose( fp );
		unlink( filename );
		return FALSE;
	}
	memset( buf, 0, sizeof(BITMAPFILEHEADER)+sizeof(BITMAPINFOHEADER)+
		col_size*sizeof(RGBQUAD) + ((col_bit+7)/8)*(x + 10)*y );
	filehead = (BITMAPFILEHEADER *)buf;
	fileinfo = (BITMAPINFOHEADER *)(buf + sizeof(BITMAPFILEHEADER));
	rgbcol = (RGBQUAD *)(((unsigned char *)fileinfo) + sizeof(BITMAPINFOHEADER) );
	bitdata = (unsigned char *)(((unsigned char *)rgbcol) + col_size * sizeof(RGBQUAD) );
	cp = bitdata;

	/* make bitmap info */
	fileinfo -> biSize = sizeof(BITMAPINFOHEADER);
	fileinfo -> biWidth  = x;
	fileinfo -> biHeight = y;
	fileinfo -> biPlanes = 1;
	if (col_bit == 15) {
		fileinfo -> biBitCount = 16;
		fileinfo -> biCompression = BI_RGB;
	} else {
		fileinfo -> biBitCount = col_bit;
		fileinfo -> biCompression = BI_RGB;
	}
	fileinfo -> biClrUsed = 0;

	/* make RGB info */
	if (bitmap_color == 1 || bitmap_color == 4 || bitmap_color == 8) {
		for (j = 0; j < col_size; j++) {
			r = 0;
			g = 0;
			b = 0;
			rgbcol[j].rgbRed   = r;
			rgbcol[j].rgbGreen = g;
			rgbcol[j].rgbBlue  = b;
			rgbcol[j].rgbReserved = 0;
		}
	}

	/* make bit table */
	ct = 0;
	switch (bitmap_color) {
#if 0
	case 1 :
		for (j = y - 1; j >= 0 ; --j) {
			tflag = TRUE;
			for (k = 0; k < x; k++) {
				if (k % 8 == 0) {
					*cp = 0;
					ct++;
				}
				if (index = (k,j) < 0) {
					/* error */
					FREE(buf);
					fclose(fp);
					return FALSE;
				}
				*cp = *cp | ((index & 0x01U) << (7-k % 8));
				if (k % 8 == 7) {
					cp++;
					tflag = TRUE;
				} else
					tflag = FALSE;
			}
			if (!tflag)
				cp++;
			while (((unsigned long)cp - (unsigned long)bitdata) & 0x03) {
				cp++;
				ct++;
			}
		}
		break;
	case 4 :
		for (j = y - 1; j >= 0 ; --j) {
			tflag = TRUE;
			for (k = 0; k < x; k++) {
				if (index = (k,j) < 0) {
					/* error */
					FREE(buf);
					fclose(fp);
					return FALSE;
				}
				if (k % 2 == 0) {
					*cp = (index << 4) & 0xf0;
					ct++;
					tflag = FALSE;
				} else {
					*cp = *cp | (index & 0x0fU);
					cp++;
					tflag = TRUE;
				}
			}
			if (!tflag)
				cp++;
			while (((unsigned long)cp-(unsigned long)bitdata) & 0x03) {
				cp++;
				ct++;
			}
		}
		break;
	case 8 :
		for (j = y - 1; j >= 0 ; --j) {
			for (k = 0; k < x; k++) {
				if (index = (k,j) < 0) {
					/* error */
					FREE(buf);
					fclose(fp);
					return FALSE;
				}
				*cp++ = (unsigned char)index;
				ct++;
			}
			while (((unsigned long)cp-(unsigned long)bitdata) & 0x03) {
				cp++;
				ct++;
			}
		}
		break;
	case 15 :
		for (j = y - 1; j >= 0 ; --j) {
			for (k = 0; k < x; k++) {
				if (!_getrgba8( p, k, j, &r, &g, &b, &a )) {
					FREE(buf);
					fclose(fp);
					return FALSE;
				}
				cs =  (((unsigned short)b << 8) >> 11) & 0x001fU;
				cs |= (((unsigned short)g << 8) >>  6) & 0x03e0U;
				cs |= (((unsigned short)r << 8) >>  1) & 0x7c00U;
				*cp++ = cs & 0xff;
				*cp++ = (cs >> 8) & 0xff;
				ct += 2;
			}
			while (((unsigned long)cp-(unsigned long)bitdata) & 0x03) {
				cp++;
				ct++;
			}
		}
		break;
#endif
	case 24 :
		for (j = y - 1; j >= 0; --j) {
			for (k = 0; k < x; k++) {
				b = p[k * 4     + j * x * 4];
				g = p[k * 4 + 1 + j * x * 4];
				r = p[k * 4 + 2 + j * x * 4];
				*cp++ = b;
				*cp++ = g;
				*cp++ = r;
				ct += 3;
			}
			while (((unsigned long)cp - (unsigned long)bitdata) & 0x03) {
				cp++;
				ct++;
			}
		}
		break;
	default :
		if (!(attr & GSAVE_ATTR_NOMSG))
			ERRMSG("gsave_writefile_bmp() unknown bitmap type.\n");
		FREE(buf);
		fclose(fp);
		return FALSE;
	}

	/* make bitmap file header */
	filehead -> bfType[0] = 'B';
	filehead -> bfType[1] = 'M';

	t = sizeof(BITMAPFILEHEADER)+sizeof(BITMAPINFOHEADER )+ col_size*sizeof(RGBQUAD) + ct;
	filehead -> bfSize_l = t % 65536;
	filehead -> bfSize_u = t / 65536;
	t = (unsigned int)((unsigned long)bitdata - (unsigned long)buf);
	filehead -> bfOffBits_l = t % 65536;
	filehead -> bfOffBits_u = t / 65536;

	/* write to */
	if (fwrite( buf, sizeof(BITMAPFILEHEADER)+sizeof(BITMAPINFOHEADER)+
			col_size*sizeof(RGBQUAD)+ct, 1, fp ) < 1) {
		if (!(attr & GSAVE_ATTR_NOMSG))
			ERRMSG("gsave_writefile_bmp() write I/O error.\n");
		FREE(buf);
		fclose(fp);
		return FALSE;
	}
	FREE(buf);
	fclose(fp);
	return TRUE;
}

int gsave_writefile_gif( unsigned int attr, const char *filename,
	int sx, int sy,	const unsigned char *p )
{
#if defined(HAVE_LIBGIF) || defined(HAVE_LIBUNGIF)
	int i, j, width, height, col_size, fullcolor;
#if defined(GIFLIB_MAJOR) && GIFLIB_MAJOR >= 5
	int giferror;
#endif
	unsigned char r, g, b; /* , a; */
	GifFileType *GifFile;
	GifPixelType *OutputBuffer;
	ColorMapObject *ColorMap;
#if 0
	int planes = 3;
	int bits = 8;
#endif

#if defined(GIFLIB_MAJOR) && GIFLIB_MAJOR >= 5
	if (!(GifFile = EGifOpenFileName( (char *)filename, FALSE, &giferror )))
#else
	if (!(GifFile = EGifOpenFileName( (char *)filename, FALSE )))
#endif
	{
		if (!(attr & GSAVE_ATTR_NOMSG))
			ERRMSG("gsave_writefile_gif() can't write open the file \"%s\"\n", filename);
		return FALSE;
	}
	width  = sx;
	height = sy;
	if (0 /* have palette */) {
#if 0
		fullcolor = FALSE;
		if (planes != 1) {
			if (!(attr & GSAVE_ATTR_NOMSG))
				ERRMSG("gsave_writefile_gif() can't save the gif file. (palette and planes != 1)\n");
#if defined(GIFLIB_MAJOR) && GIFLIB_MAJOR >= 5
			EGifCloseFile( GifFile, &giferror );
#else
			EGifCloseFile( GifFile );
#endif
			return FALSE;
		}
		if (p -> u.i.pi -> u.pi.bits == 1)
			col_size = 2;
		else if (p -> u.i.pi -> u.pi.bits == 4)
			col_size = 16;
		else if (p -> u.i.pi -> u.pi.bits == 8)
			col_size = 256;
		else {
			if (!(attr & GSAVE_ATTR_NOMSG))
				ERRMSG("gsave_writefile_gif() can't save the gif file. (palette and unknown bits)\n");
#if defined(GIFLIB_MAJOR) && GIFLIB_MAJOR >= 5
			EGifCloseFile( GifFile, &giferror );
#else
			EGifCloseFile( GifFile );
#endif
			return FALSE;
		}
#if defined(GIFLIB_MAJOR) && GIFLIB_MAJOR >= 5
		ColorMap = GifMakeMapObject( col_size, NULL );
#else
		ColorMap = MakeMapObject( col_size, NULL );
#endif
		if (!ColorMap) {
			if (!(attr & GSAVE_ATTR_NOMSG))
				ERRMSG("gsave_writefile_gif() colormap memory allocation failed\n");
#if defined(GIFLIB_MAJOR) && GIFLIB_MAJOR >= 5
			EGifCloseFile( GifFile, &giferror );
#else
			EGifCloseFile( GifFile );
#endif
			return FALSE;
		}
		/* palette_image = ... */
		for (i = 0; i < col_size; i++) {
			if (!get_rgba8( palette_image, i, 0, &r, &g, &b, &a )) {
#if defined(GIFLIB_MAJOR) && GIFLIB_MAJOR >= 5
				GifFreeMapObject( ColorMap );
				EGifCloseFile( GifFile, &giferror );
#else
				FreeMapObject( ColorMap );
				EGifCloseFile( GifFile );
#endif
				return FALSE;
			}
			ColorMap -> Colors[i].Red   = r;
			ColorMap -> Colors[i].Green = g;
			ColorMap -> Colors[i].Blue  = b;
		}
		OutputBuffer = MALLOC( width * sizeof(GifPixelType) );
		if (!OutputBuffer) {
			if (!(attr & GSAVE_ATTR_NOMSG))
				ERRMSG("gsave_writefile_gif() memory allocation failed\n");
#if defined(GIFLIB_MAJOR) && GIFLIB_MAJOR >= 5
			GifFreeMapObject( ColorMap );
			EGifCloseFile( GifFile, &giferror );
#else
			FreeMapObject( ColorMap );
			EGifCloseFile( GifFile );
#endif
			return FALSE;
		}
#endif
	} else {
		GifPixelType *RedBuffer, *GreenBuffer, *BlueBuffer;

		fullcolor = TRUE;
		col_size = 256;
#if defined(GIFLIB_MAJOR) && GIFLIB_MAJOR >= 5
		ColorMap = GifMakeMapObject( col_size, NULL );
#else
		ColorMap = MakeMapObject( col_size, NULL );
#endif
		if (!ColorMap) {
			if (!(attr & GSAVE_ATTR_NOMSG))
				ERRMSG("gsave_writefile_gif() colormap memory allocation failed\n");
#if defined(GIFLIB_MAJOR) && GIFLIB_MAJOR >= 5
			EGifCloseFile( GifFile, &giferror );
#else
			EGifCloseFile( GifFile );
#endif
			return FALSE;
		}
		RedBuffer = MALLOC( width * height * sizeof( GifByteType ) );
		GreenBuffer = MALLOC( width * height * sizeof( GifByteType ) );
		BlueBuffer = MALLOC( width * height * sizeof( GifByteType ) );
		OutputBuffer = MALLOC( width * height * sizeof( GifByteType ) );
		if (!RedBuffer || !GreenBuffer || !BlueBuffer || !OutputBuffer) {
			if (!(attr & GSAVE_ATTR_NOMSG))
				ERRMSG("gsave_writefile_gif() memory allocation failed\n");
#if defined(GIFLIB_MAJOR) && GIFLIB_MAJOR >= 5
			GifFreeMapObject( ColorMap );
			EGifCloseFile( GifFile, &giferror );
#else
			FreeMapObject( ColorMap );
			EGifCloseFile( GifFile );
#endif
			FREE( OutputBuffer );
			FREE( BlueBuffer );
			FREE( GreenBuffer );
			FREE( RedBuffer );
			return FALSE;
		}
		for (i = 0; i < height; i++) {
			for (j = 0; j < width; j++) {
				b = p[j * 4     + i * width * 4];
				g = p[j * 4 + 1 + i * width * 4];
				r = p[j * 4 + 2 + i * width * 4];
				RedBuffer[i * width + j]   = r;
				GreenBuffer[i * width + j] = g;
				BlueBuffer[i * width + j]  = b;
			}
		}
#if defined(GIFLIB_MAJOR) && GIFLIB_MAJOR >= 5
		if (GifQuantizeBuffer( width, height, &col_size,
				RedBuffer, GreenBuffer, BlueBuffer,
				OutputBuffer, ColorMap -> Colors ) == GIF_ERROR)
#else
		if (QuantizeBuffer( width, height, &col_size,
				RedBuffer, GreenBuffer, BlueBuffer,
				OutputBuffer, ColorMap -> Colors ) == GIF_ERROR)
#endif
		{
			if (!(attr & GSAVE_ATTR_NOMSG))
				ERRMSG("gsave_writefile_gif() quantize failed\n");
#if defined(GIFLIB_MAJOR) && GIFLIB_MAJOR >= 5
			GifFreeMapObject( ColorMap );
			EGifCloseFile( GifFile, &giferror );
#else
			FreeMapObject( ColorMap );
			EGifCloseFile( GifFile );
#endif
			FREE( OutputBuffer );
			FREE( BlueBuffer );
			FREE( GreenBuffer );
			FREE( RedBuffer );
			return FALSE;
		}
		FREE( BlueBuffer );
		FREE( GreenBuffer );
		FREE( RedBuffer );
	}
	if (EGifPutScreenDesc( GifFile,	width, height, col_size, 0, ColorMap ) == GIF_ERROR) {
#if defined(GIFLIB_MAJOR) && GIFLIB_MAJOR >= 5
		if (!(attr & GSAVE_ATTR_NOMSG))
			ERRMSG( "%s\n", GifErrorString( giferror ) );
		GifFreeMapObject( ColorMap );
		EGifCloseFile( GifFile, &giferror );
#else
		if (!(attr & GSAVE_ATTR_NOMSG))
			PrintGifError();
		FreeMapObject( ColorMap );
		EGifCloseFile( GifFile );
#endif
		FREE( OutputBuffer );
		return FALSE;
	}
	if (EGifPutImageDesc( GifFile, 0, 0, width, height, _interlace_flag, NULL ) == GIF_ERROR) {
#if defined(GIFLIB_MAJOR) && GIFLIB_MAJOR >= 5
		if (!(attr & GSAVE_ATTR_NOMSG))
			ERRMSG( "%s\n", GifErrorString( giferror ) );
		GifFreeMapObject( ColorMap );
		EGifCloseFile( GifFile, &giferror );
#else
		if (!(attr & GSAVE_ATTR_NOMSG))
			PrintGifError();
		FreeMapObject( ColorMap );
		EGifCloseFile( GifFile );
#endif
		FREE( OutputBuffer );
		return FALSE;
	}
	if (_interlace_flag) {
#if 0
		int k;
#endif
		for (i = 0; i < 4; i++) {
			int r = GIF_ERROR;

			for (j = InterlacedOffset[i]; j < height; j += InterlacedJumps[i]) {
				if (!fullcolor) {
#if 0
					for (k = 0; k < width; k++) {
						int index;

						if (!get_index( p, k, j, &index )) {
#if defined(GIFLIB_MAJOR) && GIFLIB_MAJOR >= 5
							GifFreeMapObject( ColorMap );
							EGifCloseFile( GifFile, &giferror );
#else
							FreeMapObject( ColorMap );
							EGifCloseFile( GifFile );
#endif
							FREE( OutputBuffer );
							return FALSE;
						}
						OutputBuffer[k] = index;
					}
					r = EGifPutLine( GifFile, OutputBuffer, width );
#endif
				} else
					r = EGifPutLine( GifFile, &OutputBuffer[j * width], width );
			}
			if (r == GIF_ERROR) {
#if defined(GIFLIB_MAJOR) && GIFLIB_MAJOR >= 5
				if (!(attr & GSAVE_ATTR_NOMSG))
					ERRMSG( "%s\n", GifErrorString( giferror ) );
				GifFreeMapObject( ColorMap );
				EGifCloseFile( GifFile, &giferror );
#else
				if (!(attr & GSAVE_ATTR_NOMSG))
					PrintGifError();
				FreeMapObject( ColorMap );
				EGifCloseFile( GifFile );
#endif
				FREE( OutputBuffer );
				return FALSE;
			}
		}
	} else {
		for (i = 0; i < height; i++) {
			int r = GIF_ERROR;

			if (!fullcolor) {
#if 0
				for (j = 0; j < width; j++) {
					int index;

					if (!get_index( p, j, i, &index )) {
#if defined(GIFLIB_MAJOR) && GIFLIB_MAJOR >= 5
						GifFreeMapObject( ColorMap );
						EGifCloseFile( GifFile, &giferror );
#else
						FreeMapObject( ColorMap );
						EGifCloseFile( GifFile );
#endif
						FREE( OutputBuffer );
						return FALSE;
					}
					OutputBuffer[j] = index;
				}
				r = EGifPutLine( GifFile, OutputBuffer, width );
#endif
			} else
				r = EGifPutLine( GifFile, &OutputBuffer[i * width], width );
			if (r == GIF_ERROR) {
#if defined(GIFLIB_MAJOR) && GIFLIB_MAJOR >= 5
				if (!(attr & GSAVE_ATTR_NOMSG))
					ERRMSG( "%s\n", GifErrorString( giferror ) );
				GifFreeMapObject( ColorMap );
				EGifCloseFile( GifFile, &giferror );
#else
				if (!(attr & GSAVE_ATTR_NOMSG))
					PrintGifError();
				FreeMapObject( ColorMap );
				EGifCloseFile( GifFile );
#endif
				FREE( OutputBuffer );
				return FALSE;
			}
		}
	}
#if defined(GIFLIB_MAJOR) && GIFLIB_MAJOR >= 5
	GifFreeMapObject( ColorMap );
#else
	FreeMapObject( ColorMap );
#endif
	FREE( OutputBuffer );
#if defined(GIFLIB_MAJOR) && GIFLIB_MAJOR >= 5
	if (EGifCloseFile( GifFile, &giferror ) == GIF_ERROR)
#else
	if (EGifCloseFile( GifFile ) == GIF_ERROR)
#endif
	{
#if defined(GIFLIB_MAJOR) && GIFLIB_MAJOR >= 5
	if (!(attr & GSAVE_ATTR_NOMSG))
		ERRMSG( "%s\n", GifErrorString( giferror ) );
#else
	if (!(attr & GSAVE_ATTR_NOMSG))
		PrintGifError();
#endif
		return FALSE;
	}
	return TRUE;
#else
	if (!(attr & GSAVE_ATTR_NOMSG))
		ERRMSG("gsave_writefile_gif() is disabled. (giflib is not supported)\n");
	return FALSE;
#endif
}

int gsave_writefile( unsigned int attr, const char *filename,
	int sx, int sy,	const unsigned char *p )
{
	char *cp;

	cp = strrchr( filename, '.' );
	if (cp) {
#if defined(HAVE_LIBGIF) || defined(HAVE_LIBUNGIF)
		if (!strcasecmp( cp+1 , "gif" ))
			return gsave_writefile_gif( attr, filename, sx, sy, p );
#endif
		if (!strcasecmp( cp+1 , "bmp" ))
			return gsave_writefile_bmp( attr, filename, sx, sy, p );
	}
	return gsave_writefile_bmp( attr, filename, sx, sy, p );
}

int gsave_reset_param_gif( void )
{
	_interlace_flag = FALSE;
	return TRUE;
}

int gsave_set_param_gif( int interlace )
{
	int t = _interlace_flag;

	if (interlace >= 0)
		_interlace_flag = interlace ? TRUE : FALSE;
	return t;
}

int gsave_init( void )
{
	gsave_reset_param_gif();
	return 0;
}

/*
	Local Variables:
	mode:c++
	c-set-style:"k&r"
	c-basic-offset:8
	tab-width:8
	End:
*/
