/* File: keyboard.c

	SHARP MZ-2000/2200/80B/B2 Emulator "emz2000"
		Keyboard Subroutines

	Extend the emz2000 keyboard.c by FUKUI, Toshio.
	This file is part of the emz2000 software.
	See copyright notice in the COPYING file.
*/

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <gtk/gtk.h>

#include "common.h"
#include "mz2000.h"

static volatile unsigned char keyint_select = 0xffU;
static void (*keyint_func)( u8 KeyBitMask ) = NULL;

typedef struct keys {
	int keyval;
	u8 MZKeyVal;
} KEYS;

static struct keys keysyms[] = {
	{GDK_KEY_F1, 0},		/* F1 */
	{GDK_KEY_F2, 1},		/* F2 */
	{GDK_KEY_F3, 2},		/* F3 */
	{GDK_KEY_F4, 3},		/* F4 */
	{GDK_KEY_F5, 4},		/* F5 */
	{GDK_KEY_F6, 5},		/* F6 */
	{GDK_KEY_F7, 6},		/* F7 */
	{GDK_KEY_F8, 7},		/* F8 */

	{GDK_KEY_F9, 8},		/* F9 */
	{GDK_KEY_F10, 9},		/* F10 */
	{GDK_KEY_F11, 9},		/* F10 */
	{GDK_KEY_F12, 9},		/* F10 */
	{GDK_KEY_KP_8, 10},		/* 8 (tenkey) */
	{GDK_KEY_KP_9, 11},		/* 9 (tenkey) */
	{GDK_KEY_KP_Multiply, 12},	/* 00 (tenkey) */
	{GDK_KEY_KP_Divide, 12},	/* 00 (tenkey) */
	{GDK_KEY_KP_Decimal, 13},	/* . (tenkey) */
	{GDK_KEY_KP_Add, 14},	/* + (tenkey) */
	{GDK_KEY_KP_Subtract, 15},	/* - (tenkey) */

	{GDK_KEY_KP_0, 16},		/* 0 (tenkey) */
	{GDK_KEY_KP_1, 17},		/* 1 (tenkey) */
	{GDK_KEY_KP_2, 18},		/* 2 (tenkey) */
	{GDK_KEY_KP_3, 19},		/* 3 (tenkey) */
	{GDK_KEY_KP_4, 20},		/* 4 (tenkey) */
	{GDK_KEY_KP_5, 21},		/* 5 (tenkey) */
	{GDK_KEY_KP_6, 22},		/* 6 (tenkey) */
	{GDK_KEY_KP_7, 23},		/* 7 (tenkey) */

	{GDK_KEY_Muhenkan, 24},	/* TAB */
	{GDK_KEY_End, 24},		/* CLR/HOME */
	{GDK_KEY_space, 25},	/* SPACE */
	{GDK_KEY_Henkan, 25},	/* SPACE */
	{GDK_KEY_Hiragana_Katakana, 25}, /* SPACE */
	{GDK_KEY_Return, 26},	/* CR */
	{GDK_KEY_KP_Enter, 26},	/* CR */
	{GDK_KEY_Up, 27},		/* UP */
	{GDK_KEY_Page_Up, 27},	/* UP */
	{GDK_KEY_Down, 28},		/* DOWN */
	{GDK_KEY_Page_Down, 28},	/* DOWN */
	{GDK_KEY_Left, 29},		/* LEFT */
	{GDK_KEY_Right, 30},	/* RIGHT */
	{GDK_KEY_Break, 31},	/* BREAK */
	{GDK_KEY_Pause, 31},	/* BREAK */
	{GDK_KEY_BackSpace, 31},	/* BREAK */

	{'/', 32},		/* / */
	{'?', 32},		/* / */
	{'A', 33},		/* A */
	{'a', 33},		/* A */
	{'B', 34},		/* B */
	{'b', 34},		/* B */
	{'C', 35},		/* C */
	{'c', 35},		/* C */
	{'D', 36},		/* D */
	{'d', 36},		/* D */
	{'E', 37},		/* E */
	{'e', 37},		/* E */
	{'F', 38},		/* F */
	{'f', 38},		/* F */
	{'G', 39},		/* G */
	{'g', 39},		/* G */

	{'H', 40},		/* H */
	{'h', 40},		/* H */
	{'I', 41},		/* I */
	{'i', 41},		/* I */
	{'J', 42},		/* J */
	{'j', 42},		/* J */
	{'K', 43},		/* K */
	{'k', 43},		/* K */
	{'L', 44},		/* L */
	{'l', 44},		/* L */
	{'M', 45},		/* M */
	{'m', 45},		/* M */
	{'N', 46},		/* N */
	{'n', 46},		/* N */
	{'O', 47},		/* O */
	{'o', 47},		/* O */
 
	{'P', 48},		/* P */
	{'p', 48},		/* P */
	{'Q', 49},		/* Q */
	{'q', 49},		/* Q */
	{'R', 50},		/* R */
	{'r', 50},		/* R */
	{'S', 51},		/* S */
	{'s', 51},		/* S */
	{'T', 52},		/* T */
	{'t', 52},		/* T */
	{'U', 53},		/* U */
	{'u', 53},		/* U */
	{'V', 54},		/* V */
	{'v', 54},		/* V */
	{'W', 55},		/* W */
	{'w', 55},		/* W */

	{'X', 56},		/* X */
	{'x', 56},		/* X */
	{'Y', 57},		/* Y */
	{'y', 57},		/* Y */
	{'Z', 58},		/* Z */
	{'z', 58},		/* Z */
	{'^', 59},		/* ^ */
	{'~', 59},		/* ^ */
	{'\\', 60},		/* \ */
	{'|', 60},		/* \ */
	{GDK_KEY_F35, 61},		/* ? */
	{'_', 61},		/* ? */
	{'.', 62},		/* . */
	{'>', 62},		/* . */
	{',', 63},		/* , */
	{'<', 63},		/* , */

	{'0', 64},		/* 0 */
	{'~', 64},		/* 0 */
	{'1', 65},		/* 1 */
	{'!', 65},		/* 1 */
	{'2', 66},		/* 2 */
	{'"', 66},		/* 2 */
	{'3', 67},		/* 3 */
	{'#', 67},		/* 3 */
	{'4', 68},		/* 4 */
	{'$', 68},		/* 4 */
	{'5', 69},		/* 5 */
	{'%', 69},		/* 5 */
	{'6', 70},		/* 6 */
	{'&', 70},		/* 6 */
	{'7', 71},		/* 7 */
	{'\'', 71},		/* 7 */

	{'8', 72},		/* 8 */
	{'(', 72},		/* 8 */
	{'9', 73},		/* 9 */
	{')', 73},		/* 9 */
	{':', 74},		/* : */
	{'*', 74},		/* : */
	{';', 75},		/* ; */
	{'+', 75},		/* ; */
	{'-', 76},		/* - */
	{'=', 76},		/* - */
	{'@', 77},		/* @ */
	{'`', 77},		/* @ */
	{'[', 78},		/* [ */
	{'{', 78},		/* [ */
	{0, 79},

	{']', 80},		/* ] */
	{'}', 80},		/* ] */
	{0, 81},
	{GDK_KEY_Home, 82},		/* CLR/HOME */
	{GDK_KEY_Insert, 83},	/* INST/DEL */
	{GDK_KEY_Delete, 83},	/* INST/DEL */
	{0, 84},
	{0, 85},
	{0, 86},
	{0, 87},

	{GDK_KEY_Tab, 88},		/* GRPH */
	{GDK_KEY_Caps_Lock, 89},	/* SFT LOCK */
	{GDK_KEY_Control_L, 89},	/* SFT LOCK */
	{GDK_KEY_Shift_L, 90},	/* SHIFT */
	{GDK_KEY_Shift_R, 90},	/* SHIFT */
	{GDK_KEY_Escape, 91},	/* KANA */
	{GDK_KEY_Zenkaku_Hankaku, 91},   /* KANA */
	{0, 92},
	{0, 93},
	{0, 94},
	{0, 95},
	{-1, 255},
};

static struct keys keysyms_curkeynormal[] = {
	{GDK_KEY_Up, 27},		/* UP */
	{GDK_KEY_Down, 28},		/* DOWN */
	{GDK_KEY_Left, 29},		/* LEFT */
	{GDK_KEY_Right, 30},	/* RIGHT */
	{-1, 255},
};

static struct keys keysyms_curkey2468[] = {
	{GDK_KEY_Down, 18},		/* 2 (tenkey) */
	{GDK_KEY_Left, 20},		/* 4 (tenkey) */
	{GDK_KEY_Right, 22},	/* 6 (tenkey) */
	{GDK_KEY_Up, 10},		/* 8 (tenkey) */
	{-1, 255},
};

static struct keys keysyms_curkey0135[] = {
	{GDK_KEY_Down, 12},		/* 00 (tenkey) */
	{GDK_KEY_Left, 17},		/* 1 (tenkey) */
	{GDK_KEY_Right, 19},		/* 3 (tenkey) */
	{GDK_KEY_Up, 21},		/* 5 (tenkey) */
	{-1, 255},
};

static struct keys keysyms_tenkey0_0[] = {
	{GDK_KEY_KP_0, 16},		/* 0 (tenkey) */
	{-1, 255}
};

static struct keys keysyms_tenkey0_00[] = {
	{GDK_KEY_KP_0, 12},		/* 00 (tenkey) */
	{-1, 255}
};

static struct keys keysyms_zxtabspcnormal[] = {
	{GDK_KEY_Muhenkan, 24},	/* TAB */
	{GDK_KEY_space, 25},	/* SPACE */
	{GDK_KEY_Henkan, 25},	/* SPACE */
	{GDK_KEY_Hiragana_Katakana, 25}, /* SPACE */
	{'X', 56},		/* X */
	{'x', 56},		/* X */
	{'Z', 58},		/* Z */
	{'z', 58},		/* Z */
	{-1, 255}
};

static struct keys keysyms_zxtabspcswap[] = {
	{GDK_KEY_Muhenkan, 58},	/* TAB */
	{GDK_KEY_space, 56},	/* SPACE */
	{GDK_KEY_Henkan, 56},	/* SPACE */
	{GDK_KEY_Hiragana_Katakana, 56}, /* SPACE */
	{'X', 25},		/* X */
	{'x', 25},		/* X */
	{'Z', 24},		/* Z */
	{'z', 24},		/* Z */
	{-1, 255}
};

static struct keys keysyms_grphtabnormal[] = {
	{GDK_KEY_Muhenkan, 24},	/* TAB */
	{GDK_KEY_Tab, 88},		/* GRPH */
	{-1, 255}
};

static struct keys keysyms_grphtabswap[] = {
	{GDK_KEY_Muhenkan, 88},	/* GRPH */
	{GDK_KEY_Tab, 24},		/* TAB */
	{-1, 255}
};

static struct keys keysyms_tenkeys123000normal[] = {
	{GDK_KEY_KP_1, 17},		/* 1 (tenkey) */
	{GDK_KEY_KP_2, 18},		/* 2 (tenkey) */
	{GDK_KEY_KP_3, 19},		/* 3 (tenkey) */
	{-1, 255}
};

static struct keys keysyms_tenkeys123000swap[] = {
	{GDK_KEY_KP_1, 16},		/* 0  (tenkey) */
	{GDK_KEY_KP_2, 12},		/* 00 (tenkey) */
	{GDK_KEY_KP_3, 13},		/* .  (tenkey) */
	{-1, 255}
};

static int _keyboard_convert( int keyval, int keyraw )
{
	switch (keyraw) {
	case 0x5a :
		return GDK_KEY_KP_0;
	case 0x5b :
		return GDK_KEY_KP_Decimal;
	case 0x57 :
		return GDK_KEY_KP_1;
	case 0x58 :
		return GDK_KEY_KP_2;
	case 0x59 :
		return GDK_KEY_KP_3;
	case 0x53 :
		return GDK_KEY_KP_4;
	case 0x54 :
		return GDK_KEY_KP_5;
	case 0x55 :
		return GDK_KEY_KP_6;
	case 0x4f :
		return GDK_KEY_KP_7;
	case 0x50 :
		return GDK_KEY_KP_8;
	case 0x51 :
		return GDK_KEY_KP_9;
	case 0x6a :
		return GDK_KEY_KP_Divide;
	case 0x3f :
		return GDK_KEY_KP_Multiply;
	case 0x52 :
		return GDK_KEY_KP_Subtract;
	case 0x56 :
		return GDK_KEY_KP_Add;
	case 0xd3 :
		return GDK_KEY_F35;	/* \_ */
	default :
		return keyval;
	}
}

int keyboard_press( MZ2000 *mz2000, int keyval, int keyraw )
{
	int mzkeycode, strobe, keybit;
	unsigned char bk_keydata, keydata;

	/* printf( "keyval-pre=%04x\n", keyval ); */
	keyval = _keyboard_convert( keyval, keyraw );
	if (keyval < 0x0100)
		mzkeycode = mz2000->keyboard->kcode1[keyval];
	else if (keyval > 0xff00 && keyval <= 0xffff)
		mzkeycode = mz2000->keyboard->kcode2[keyval & 0xff];
	else
		mzkeycode = 0xff;
	/* printf( "press code=%d keyval=%04x(raw=%02x)\n", mzkeycode, keyval, keyraw ); */
	if (!(mzkeycode == 0xff)) {
		strobe = mzkeycode >> 3;
		keybit = mzkeycode & 0x07;
		bk_keydata = mz2000->keyboard->key_bitdata[strobe];
		mz2000->keyboard->key_bitdata[strobe] &= ~(1 << keybit);
		keydata = mz2000->keyboard->key_bitdata[strobe];
	} else {
		strobe = -1;
		bk_keydata = keydata = 0;
	}
	if (keyint_select == strobe && bk_keydata != keydata) {
		if (keyint_select != 0xffU && keyint_func)
			keyint_func( bk_keydata ^ keydata );
	}
	return 1;
}

int keyboard_release( MZ2000 *mz2000, int keyval, int keyraw )
{
	int mzkeycode, strobe, keybit;
	unsigned char bk_keydata, keydata;

	keyval = _keyboard_convert( keyval, keyraw );
	if (keyval < 0x0100)
		mzkeycode = mz2000->keyboard->kcode1[keyval];
	else if (keyval > 0xff00 && keyval <= 0xffff)
		mzkeycode = mz2000->keyboard->kcode2[keyval & 0xff];
	else
		mzkeycode = 0xff;
	/* printf( "release code=%d keyval=%04x(raw=%02x)\n", mzkeycode, keyval, keyraw ); */
	if (!(mzkeycode == 0xff)) {
		strobe = mzkeycode >> 3;
		keybit = mzkeycode & 0x07;
		bk_keydata = mz2000->keyboard->key_bitdata[strobe];
		mz2000->keyboard->key_bitdata[strobe] |= (1 << keybit);
		keydata = mz2000->keyboard->key_bitdata[strobe];
	} else {
		strobe = -1;
		bk_keydata = keydata = 0;
	}
	return 1;
}

void keyboard_setinterrupt( unsigned char select, void *intfunc )
{
	keyint_select = 0xffU;
	keyint_func = intfunc;
	keyint_select = select;
	return;
}

void keyboard_setdirect( MZ2000 *mz2000, int press_flag, int strobe, int bit )
{
	if (press_flag)
		mz2000->keyboard->key_bitdata[strobe] &= ~(1U << bit);
	else
		mz2000->keyboard->key_bitdata[strobe] |= (1U << bit);
	return;
}

int keyboard_getbitdata( MZ2000 *mz2000, int strobe )
{
	return mz2000->keyboard->key_bitdata[strobe];
}

int keyboard_getallbitdata( MZ2000 *mz2000 )
{
	int i, bitdata = 0xff;

	for (i = 0; i < 12; i++)
		bitdata &= mz2000->keyboard->key_bitdata[i];
	return bitdata;
}

int keyboard_breakkey_released( MZ2000 *mz2000 )
{
	return mz2000->keyboard->key_bitdata[3] & 0x80 ? TRUE : FALSE;
}

int keyboard_releaseall( MZ2000 *mz2000 )
{
	int i;

	for (i = 0; i < 16; i++)
		mz2000->keyboard->key_bitdata[i] = 0xff;
	return TRUE;
}

static int set_keymap( MZ2000 *mz2000, struct keys *keydata )
{
	struct keys *curp;

	curp = keydata;
	while (curp -> keyval >= 0) {
		if (curp -> keyval) {
			int n;

			n = curp -> keyval;
			if (n < 0x0100)
				mz2000->keyboard->kcode1[n] = curp->MZKeyVal;
			else if (n > 0xff00 && n <= 0xffff)
				mz2000->keyboard->kcode2[(unsigned int)n & 0xff] = curp->MZKeyVal;
		}
		curp++;
	}
	return 0;
}

int keyboard_setkeymode( unsigned char keymode )
{
       if (keymode & 0x01U) {
		/* cursor keys = 2468 */
		set_keymap( mz2000_global, keysyms_curkey2468 );
	} else if (keymode & 0x02U) {
		/* cursor keys = [00]135 */
		set_keymap( mz2000_global, keysyms_curkey0135 );
	} else {
		/* cursor keys = normal */
		set_keymap( mz2000_global, keysyms_curkeynormal );
	}
	if (keymode & 0x04U) {
		/* 0 = [00] */
		set_keymap( mz2000_global, keysyms_tenkey0_00 );
	} else
		set_keymap( mz2000_global, keysyms_tenkey0_0 );
	if (keymode & 0x08U) {
		/* ZX <-> TAB/SPACE */
		set_keymap( mz2000_global, keysyms_zxtabspcswap );
	} else
		set_keymap( mz2000_global, keysyms_zxtabspcnormal );
	if (keymode & 0x10U) {
		/* GRPH <-> TAB */
		set_keymap( mz2000_global, keysyms_grphtabswap );
	} else
		set_keymap( mz2000_global, keysyms_grphtabnormal );
	if (keymode & 0x20U) {
		/* 1 2 3 = 0 [00] . */
		set_keymap( mz2000_global, keysyms_tenkeys123000swap );
	} else {
		/* 1 2 3 = normal */
		set_keymap( mz2000_global, keysyms_tenkeys123000normal );
	}
	return TRUE;
}

void keyboard_exit( MZ2000 *mz2000 )
{
	if (mz2000 -> keyboard) {
		FREE( (void *)(mz2000 -> keyboard) );
		mz2000 -> keyboard = NULL;
	}
	return;
}

int keyboard_init( MZ2000 *mz2000 )
{
	int i;

	if ((mz2000->keyboard = (MZKeyboard *)MALLOC(sizeof(MZKeyboard))) == NULL) {
		fprintf(stderr, "No enough memory\n");
		return 0;
	}
	for (i = 0; i < 256; i++) {
		mz2000->keyboard->kcode1[i] = 0xff;
		mz2000->keyboard->kcode2[i] = 0xff;
	}
	set_keymap( mz2000, keysyms );
	keyint_select = 0xffU;
	keyint_func = NULL;
	keyboard_releaseall(mz2000);
	return 1;
}

/*
	Local Variables:
	mode:c++
	c-set-style:"k&r"
	c-basic-offset:8
	tab-width:8
	End:
*/
