/* File: mz2000.h

	SHARP MZ-2000/2200/80B/B2 Emulator "EmuZ-2000"
		System Definitions Headders

	Combine the emz2000 mz2000.h, EmuZ-2000 emuz2000.h,
		and extend by FUKUI, Toshio and TSUNEOKA, Shinji.
	This file is part of the emz2000 software.
	See copyright notice in the COPYING file.
*/

#ifndef __MZ2000_H_INCLUDED
#define __MZ2000_H_INCLUDED

#include "z80.h"

#include <gtk/gtk.h>

#define MZ2000_REFRESHRATE		10	/* ms (16 is bad) */
#define MZ2000_DISPLAY_WIDTH		640
#define MZ2000_DISPLAY_HEIGHT		(200 * 2)

#define TIMER8253_USE_INTERNALTIME
#define TIMER8253_COUNTER_MAXNUM	3

#pragma pack(push, 4)

typedef unsigned int u32;

typedef struct mzfont {
	unsigned char chr40[256 * 2][128];
	unsigned char chr80[256 * 2][64];
} MZFont;

typedef struct mzscreen {
	GtkWidget *screen_widget;
	u8 scrbuf[MZ2000_DISPLAY_WIDTH * MZ2000_DISPLAY_HEIGHT];
	u8 scrbufd[MZ2000_DISPLAY_WIDTH * MZ2000_DISPLAY_HEIGHT * 4];
	u8 F4;
	u8 F5;
	u8 F6;
	char isgreen;
	char issize40;
	int blank, reverse;
	MZFont *font;
	guint32 pixel[8];
	guint32 colors[16];
	void *_gc_oldreserved;
	void *_pixmap_oldreserved;
	int _depth_oldreserved;
	unsigned char nfr, nfg, nfb;
	unsigned char nbr, nbg, nbb;
	unsigned char rfr, rfg, rfb;
	unsigned char rbr, rbg, rbb;
} MZScreen;

typedef struct mzkeyboard {
	int kcode1[256];
	int kcode2[256];
	u8 key_bitdata[16];
} MZKeyboard;

typedef struct mzioinfo {
	u8 port_d8;
	u8 port_d9;
	u8 port_da;
	u8 port_db;
	u8 port_dc;
	u8 port_dd;

	u8 port_e0;
	u8 port_e1;
	u8 port_e2;
	u8 port_e3;
	u8 port_e4;
	u8 port_e5;
	u8 port_e6;
	u8 port_e7;
	u8 port_e8;
	u8 port_e9;
	u8 port_ea;
	u8 port_eb;
	u8 port_f4;
	u8 port_f5;
	u8 port_f6;
	u8 port_f7;
	int strobe_low;
} MZIOInfo;

typedef struct mz2000 {
	u8 *ipl;
	u8 *memory;
	u8 *vram;
	u8 *gram1, *gram2, *gram3;
	u8 *iplrom2000, *iplrom80b;
	MZIOInfo *ioinfo;
	int inipl, vramon, selchr, gramno, blank;
	int z80MaxClock;
	int nowZ80Clock;
	int howipl;		/* 0: no need, 1: internal function, 2: z80 program, etc. */
	volatile int needReset;	/* Reset switch */
	volatile int needIpl;	/* IPL switch */
	volatile int playStop;	/* temporary stop flag */

	volatile int cpu_peripherals_start; /* Peripherals Initialize Info. */
		/* bit 0 ... 8255 */
		/* bit 1 ... 8253 */
		/* bit 2 ... Z80 PIO A (Keyboard, Memory Mapping, CRTC) */
		/* bit 3 ... Z80 PIO B (Keyboard) */
		/* bit 4 ... Reserved for Z80 PIO A (16bit board) */
		/* bit 5 ... Reserved for Z80 PIO B (16bit board) */

	/* 8253 (timer) */
	int timer8253_start;
	z80clk timer8253_starttime[TIMER8253_COUNTER_MAXNUM];
	z80clk timer8253_currenttime[TIMER8253_COUNTER_MAXNUM];
	int timer8253_rwmode[TIMER8253_COUNTER_MAXNUM];
	u16 timer8253_count[TIMER8253_COUNTER_MAXNUM];
	u16 timer8253_fast_mode;

	/* Z80 PIO (keyboard) */
	int z80pio_modecontrol_state;
	u8 intVect;
	u8 bitMask;
	volatile unsigned char keyint_select;
	void (*keyint_func)( u8 KeyBitMask );

	int cpu_peripherals_end;

	/* Architecture and Pause Mode */
	volatile int mzmode;	/* FALSE=2000 / TRUE=80B */
	volatile int pause;	/* Pause On/Off */

	/* Sound */
	char soundmode;		/* Sound On/Off */
	char enable_soundtape;
	char enable_soundfdd;
	char sound_bit;
	int sound_freq;
	int sound_playbuffer_ms;
	int sound_delaytime_ms;

	/* Other Hardware */
	u8 *iplmon;
	char screen_highreso;	/* Screen High Resolution Mode */
	unsigned char sound_volume1;	/* Sound Vol. MZ Pulse Sound */
	unsigned char sound_volume2;	/* Sound Vol. Effect Waves */
	char enable_soundtminterpolations; /* Sound Time Interpolations */
	int font_select;	/* Reserved (Windows only) */
	int font_english;	/* Font English Mode */
	int vblankspecial;	/* V-Blank Special Mode */
	int dropautoipl;	/* Drop Auto IPL On/Off (FD) */
	int forcereadonly;	/* Force Read Only On/Off */
	char enable_initreset;  /* Auto Initialize I/O on Reset */
        char iplmemclr;         /* IPL reset memory init On/Off */
        char iplmemzclr;        /* Init memory is zero clear */
	char fullscreen_bpp;    /* Reserved (Windows only) */
	int language_info;	/* Laguage mode, 0 ... Auto (Reserved) */
	int aeroglass;		/* Reserved (Windows only) */
	int dropautoipltape;	/* Drop Auto IPL On/Off (TAPE) */
	char droptofd2;		/* Drop to FD2 */
	char fd_exec_fast;	/* In FD, fast running mode */
	char fd_ignore_crc;	/* In FD, ignore read crc error */
	char fd_erase_sct17crc;	/* In FD, erase sector 17 crc to 16 sectors */
	int consoler_cblank;	/* Console R Color Blank */
	char screen_window;	/* Screen Window (UNIX only) */
	char fullscreen_resolution; /* Reserved (Windows only) */
	char language;		/* Language Mode */
	char currentlanguage;	/* Current Language */
	u8 keyboard_mode;	/* Keyboard Mode */

	/* Green Display Color */
	unsigned char greencolor_nfr;
	unsigned char greencolor_nfg;
	unsigned char greencolor_nfb;
	unsigned char greencolor_nbr;
	unsigned char greencolor_nbg;
	unsigned char greencolor_nbb;
	unsigned char greencolor_rfr;
	unsigned char greencolor_rfg;
	unsigned char greencolor_rfb;
	unsigned char greencolor_rbr;
	unsigned char greencolor_rbg;
	unsigned char greencolor_rbb;

	/* Memory Wait and Frame Clock Adjust */
	int frameclock_alpha;
	int memory_waittvramclock_r;
	int memory_waittvramclock_w;
	int memory_waitgramclock_r;
	int memory_waitgramclock_w;
	int memory_waitgram80bclock_r;
	int memory_waitgram80bclock_w;

	/* Machine State (.mze) */
	int sfilename_havewrite;
	char sfilename[PATH_MAX];

	/* Cassette Tape Deck */
	char tfilename[PATH_MAX];
	int tapemode_reverse;	/* Reverse */
	char tapeff_ignore;	/* Ignore FF */
	char tapewrite_interpolation;	/* Write Interpolation On/Off */
	char tape_reserved1;
	char tape_reserved2;
	int tapecounter;	/* Counter Mode */
	int tapectspeed_2000;	/* Counter Speed 2000 */
	int tapectspeed_80B;	/* Counter Speed 80B */
	int tapectspeed_2500;	/* Counter Speed 2500 */
	int tapespeaker;	/* Speaker On/Off */
	unsigned int tapePitch;
	unsigned int tapeThreshold;
	unsigned int writeThreshold;
	unsigned int amplitudeRange;
	int tapeLoad;
	int tapeFF;
	int tapeEnd;

	/* Floppy Disk Drive */
	int enable_fddif;	/* Enable I/F (MZ-1E05/MZ-8FBI) */

	/* Quick Disk Drive MZ-1E18+MZ-1F11 */
	char qfilename[PATH_MAX];

	/* Main Window */
	MZScreen *screen;
	guint statusbarid;
	GtkWidget *statusbar;
	/* Screen */
	int displayMode;	/* 0=Green Monitor, 1=Color Monitor */
	/* Keyboard */
	volatile MZKeyboard *keyboard;

	/* 16bit Board MZ-1M01/MZ-1R08(Kanji-ROM) */
	/* (not implemented currently) */
	int enable_16bit;	/* Enable 16bit Board (reserved) */
} MZ2000;

#pragma pack(pop)

/* main.c */
typedef enum {
	UILANG_AUTOMATIC = -1,
	UILANG_ENGLISH = 0,
	UILANG_JAPANESE,
	UILANG_MAXNUM
} UILANG_TYPE;
typedef enum {
	TAPE_COUNTERMODE_MZ2080 = 0,
	TAPE_COUNTERMODE_MZ2500,
	TAPE_COUNTERMODE_KBYTES,
	TAPE_COUNTERMODE_BYTES,
	TAPE_COUNTERMODE_MAXNUM
} TAPE_COUNTERMODE_TYPE;
#define TAPE_COUNTERMODE_DEFAULT TAPE_COUNTERMODE_MZ2080
extern MZ2000 *mz2000_global;
extern char app_currentpath[];
extern char app_defaultinifile[];
extern void setplaystop( MZ2000 *mz, int flag );
extern int reset_mz2000_sub( MZ2000 *mz2000 );
extern int reset_mz2000( MZ2000 *mz2000 );
extern int iplreset_mz( MZ2000 *mz2000 );
extern int iplreset_mz2000( MZ2000 *mz2000 );
extern int iplreset_mz80b( MZ2000 *mz2000 );

/* mz2io.c */
#define CPUPERIPHERALS_RESET		0x00U
#define CPUPERIPHERALS_INIT		0x0fU
#define CPUPERIPHERALS_INIT_8255	0x01U
#define CPUPERIPHERALS_INIT_8253	0x02U
#define CPUPERIPHERALS_INIT_Z80PIOA	0x04U
#define CPUPERIPHERALS_INIT_Z80PIOB	0x08U
#define CPUPERIPHERALS_INIT_Z80PIO16A	0x10U
#define CPUPERIPHERALS_INIT_Z80PIO16B	0x20U
extern void mz2io_keyintfunc( u8 KeyBitMask );
extern void reinit_io( MZ2000 *mz );
extern void refresh_io( MZ2000 *mz );
extern void io_exit( MZ2000 *mz );
extern int io_init( MZ2000 *mz );

/* mz2mem.c */
#define MZ_IPLROM_SIZE		( 2 * 1024)	/* IPL Main memory size */
#define MZ_MAINMEMORY_SIZE	(64 * 1024)	/* Main memory size */
#define MZ_TEXTVRAM_SIZE	( 4 * 1024)	/* Text V-RAM size */
#define MZ_GRAMPAGE_SIZE	(16 * 1024)	/* G-RAM page size */
#define	GRAM_MAX		0x3fff		/* Graphic area memory size */
#define	GRAM_CLIP		0x3e80		/* Graphic area scan clip */
#define	TEXT_MAX		0xfff		/* Text area memory size */
#define	TEXT_CLIP		0x800		/* Text area scan clip */
#define	GRAM_MAX_80B		0x1fff		/* Graphic area memory size */
#define	GRAM_CLIP_80B		0x1f40		/* Graphic area scan clip */
extern int vram_on( MZ2000 *mz );
extern int gram_on( MZ2000 *mz );
extern int v_gram_off( MZ2000 *mz );
extern int v_gram_on80B_D0( MZ2000 *mz );
extern int v_gram_on80B_50( MZ2000 *mz );
extern int v_gram_off80B( MZ2000 *mz );
extern int memory_clear( MZ2000 *mz, int mode ); /* 0 = all / 1 = main / 2 = T/G-RAM */
extern int memory_ipl( MZ2000 *mz );
extern int memory_normal( MZ2000 *mz );
extern void memory_exit( MZ2000 *arg );
extern int memory_init( MZ2000 *arg );

/* screen.c */
#define GUI_REFRESH_RATE	MZ2000_REFRESHRATE
extern int screen_clear( MZ2000 *mz );
extern int screen_putmsg( MZ2000 *mz, int adr, const char *s );
extern int screen_putmsgn( MZ2000 *mz, int adr, const char *s, int n );
extern int screen_redraw_region(MZ2000 *mz);
extern int screen_redraw(MZ2000 *mz, int x, int y, int width, int height);
extern int screen_region_add_textvram( MZ2000 *mz, int addr );
extern int screen_region_add_gram2000( MZ2000 *mz, int page, int addr );
extern int screen_region_add_gram80B( MZ2000 *mz, int page, int addr );
extern int screen_region_add_all( MZ2000 *mz );
extern int screen_portF4(MZ2000 *mz, unsigned char val);
extern int screen_portF5(MZ2000 *mz, unsigned char val);
extern int screen_portF6(MZ2000 *mz, unsigned char val);
extern int screen_setreverse( MZ2000 *mz, int val );
extern int screen_setblank( MZ2000 *mz, unsigned char val );
extern int screen_setchrsize( MZ2000 *mz, int chr80_flag );
extern int screen_setmonitor( MZ2000 *mz, int color_flag );
extern int screen_sethighreso( MZ2000 *mz, int highreso_flag );
extern int screen_setfontlang( MZ2000 *mz, int english_flag );
extern int screen_setgreencolor( MZ2000 *mz,
	unsigned char nfr, unsigned char nfg, unsigned char nfb,
	unsigned char nbr, unsigned char nbg, unsigned char nbb,
	unsigned char rfr, unsigned char rfg, unsigned char rfb,
	unsigned char rbr, unsigned char rbg, unsigned char rbb );
extern int screen_initattr2( MZ2000 *mz );
extern int screen_initattr( MZ2000 *mz );
extern int screen_initarch( MZ2000 *mz );
extern void screen_exit( MZ2000 *mz );
extern int screen_init( MZ2000 *mz, const char *fontfile );

/* keyboard.c */
extern int keyboard_press( MZ2000 *mz2000, int keyval, int keyraw );
extern int keyboard_release( MZ2000 *mz2000, int keyval, int keyraw );
extern void keyboard_setinterrupt( unsigned char select, void *intfunc );
extern void keyboard_setdirect( MZ2000 *mz2000, int press_flag, int strobe, int bit );
extern int keyboard_getbitdata( MZ2000 *mz2000, int strobe );
extern int keyboard_getallbitdata( MZ2000 *mz2000 );
extern int keyboard_breakkey_released( MZ2000 *mz2000 );
extern int keyboard_releaseall( MZ2000 *mz2000 );
extern int keyboard_setkeymode( unsigned char keymode );
extern void keyboard_exit( MZ2000 *mz2000 );
extern int keyboard_init( MZ2000 *mz2000 );

/* mz2tape.c */
#define TAPE_IPLMSG_MAXNUM	16
typedef enum {
	MZ2TAPE_RUNMODE_EJECT = -1,
	MZ2TAPE_RUNMODE_STOP = 0,
	MZ2TAPE_RUNMODE_PLAY,
	MZ2TAPE_RUNMODE_RECORD,
	MZ2TAPE_RUNMODE_FF,
	MZ2TAPE_RUNMODE_REW,
	MZ2TAPE_RUNMODE_MAXNUM
} MZ2TAPE_RUNMODE;
typedef enum {
	MZ2TAPE_FILEMODE_ERRNDLG = -2,
	MZ2TAPE_FILEMODE_ERROR = -1,
	MZ2TAPE_FILEMODE_MZT = 0,
	MZ2TAPE_FILEMODE_MZF,
	MZ2TAPE_FILEMODE_MTW,
	MZ2TAPE_FILEMODE_MTI,
	MZ2TAPE_FILEMODE_WAV,
	MZ2TAPE_FILEMODE_DAT,
	MZ2TAPE_FILEMODE_HEX,
	MZ2TAPE_FILEMODE_MAXNUM
} MZ2TAPE_FILEMODE;
extern int mz2tape_ready( void );
extern int mz2tape_getstatus( unsigned long *pos, unsigned long *total,
	unsigned long *offset, int *tapeend,
	unsigned long *tapefreq, unsigned long *deffreq );
extern int mz2tape_getwritable( void );
extern int mz2tape_setwritable( int enable );
extern int mz2tape_getkinh( void );
extern int mz2tape_setkinh( int enable );
extern int mz2tape_getrev( void );
extern int mz2tape_setrev( int enable );
extern int mz2tape_eject( MZ2000 *mz, int sound_flag );
extern int mz2tape_stop2( MZ2000 *mz );
extern int mz2tape_stop( MZ2000 *mz );
extern int mz2tape_rew( MZ2000 *mz );
extern int mz2tape_ff( MZ2000 *mz );
extern int mz2tape_play( MZ2000 *mz );
extern int mz2tape_record( MZ2000 *mz );
extern int mz2tape_head( MZ2000 *mz );
extern int mz2tape_updatestatus( MZ2000 *mz, unsigned char *port );
extern int mz2tape_updatewrite( MZ2000 *mz, int flag );
extern int mz2tape_updatetimer( void );
extern int mz2tape_exec( MZ2000 *mz, const char *filename, int readonly );
extern void tape_exit( MZ2000 *mz );
extern int tape_init( MZ2000 *mz );

/* disk.c */
#define	MZ_DISK_DRIVE_MAX	4
#define DISK_IPLMSG_MAXNUM	10
typedef enum {
	DISK_STATUS_NONE = 0,
	DISK_STATUS_INSERT,
	DISK_STATUS_WCHANGED,
	DISK_STATUS_MAXNUM
} DISK_STATUS_TYPE;
typedef enum {
	DISK_EJECT_NORMAL = 0,
	DISK_EJECT_CHECKONLY,
	DISK_EJECT_FORCE,
	DISK_EJECT_MAXNUM,
} DISK_EJECT_TYPE;
extern char disk_iplmsg[DISK_IPLMSG_MAXNUM + 1];
extern int disk_updatetimer( void );
extern const char *disk_getdefpath( MZ2000 *mz, int drive );
extern int disk_getexist( MZ2000 *mz, int drive );
extern int disk_getwritable( MZ2000 *mz, int drive );
extern int disk_getaccess( MZ2000 *mz, int drive );
extern int disk_getmodified( MZ2000 *mz, int drive );
extern char *disk_getfnptr( MZ2000 *mz, int drive );
extern int disk_writeback( MZ2000 *mz, int drive );
extern int disk_eject( MZ2000 *mz, int drive, DISK_EJECT_TYPE mode );
extern int disk_change( MZ2000 *mz, int drive, const char *filename, int d88flag, int nomsg, int readonly );
extern int disk_exec( MZ2000 *mz, int drive, const char *filename, int d88flag, int readonly );
extern int disk_setparam( int exec_fast, int ignore_crc, int erase_sct17crc );
extern void disk_reset( MZ2000 *mz );
extern void disk_exit( MZ2000 *mz );
extern int disk_init( MZ2000 *mz );
/* I/O */
extern u8 disk_port_in_d8( u16 port );
extern int disk_port_out_d8( u16 port, u8 value );
extern u8 disk_port_in_d9( u16 port );
extern int disk_port_out_d9( u16 port, u8 value );
extern u8 disk_port_in_da( u16 port );
extern int disk_port_out_da( u16 port, u8 value );
extern u8 disk_port_in_db( u16 port );
extern int disk_port_out_db( u16 port, u8 value );
extern u8 disk_port_in_dc( u16 port );
extern int disk_port_out_dc( u16 port, u8 value );
extern u8 disk_port_in_dd( u16 port );
extern int disk_port_out_dd( u16 port, u8 value );

/* sound.c */
typedef enum {
	SOUND_WAV_FDMOTOR = 0,
	SOUND_WAV_FDMOTOR_ST,
	SOUND_WAV_FDEJECT,
	SOUND_WAV_FDSEEK,
	SOUND_WAV_FDSEEK1,
	SOUND_WAV_2000EJECT,
	SOUND_WAV_2000PLAY,
	SOUND_WAV_2000STOP,
	SOUND_WAV_2000FF,
	SOUND_WAV_2000REW,
	SOUND_WAV_2000FFST,
	SOUND_WAV_2000APSS,
	SOUND_WAV_80BEJECT,
	SOUND_WAV_80BPLAY,
	SOUND_WAV_80BSTOP,
	SOUND_WAV_80BFF,
	SOUND_WAV_80BREW,
	SOUND_WAV_MAXNUM
} SOUND_WAVLIST;
extern void sound_reset( MZ2000 *mz );
extern void sound_setstate( MZ2000 *mz, int onoff );
extern int sound_playwav_file( MZ2000 *mz, SOUND_WAVLIST no );
extern int sound_enablechk( void );
extern int sound_updateparams( MZ2000 *mz );
extern void sound_exit( void );
extern int sound_init( MZ2000 *mz );

/* state.c */
extern int save_state( MZ2000 *mz );
extern int load_state( MZ2000 *mz );

/* copypaste.c */
typedef enum {
	PASTE_MONITOR_NORMAL = 0,
	PASTE_MONITOR_PUPMON,
	PASTE_MONITOR_TS,
	PASTE_MONITOR_HUBASIC,
	PASTE_MONITOR_DBBASIC,
	PASTE_MONITOR_MAXNUM
} PASTE_MONITOR_MODE;
extern int paste_mode;
extern int paste_wait;
extern int paste_waitcr;
extern int copypaste_havepaste( void );
extern int copypaste_pastestart( void );
extern int copypaste_pasteend( int force_flag );
extern void copypaste_pastetimer( MZ2000 *mz );
extern int copypaste_runningpaste( void );
extern int copypaste_copy( void );
extern void copypaste_exit( void );
extern int copypaste_init( void );

#endif /* __MZ2000_H_INCLUDED */

/*
	Local Variables:
	mode:c++
	c-set-style:"k&r"
	c-basic-offset:8
	tab-width:8
	End:
*/
