/* File: screen.c

	SHARP MZ-2000/2200/80B/B2 Emulator "emz2000"
		CRTC with Display Modules

	Extend the emz2000 screen.c by FUKUI, Toshio.
	This file is part of the emz2000 software.
	See copyright notice in the COPYING file.
*/

#include "config.h"

#include <stdio.h>
#include <memory.h>
#include <stdlib.h>
#include <ctype.h>
#include <gtk/gtk.h>

#include "common.h"
#include "mz2000.h"
#include "uigtk.h"

static int screen_init_flag = FALSE;
static unsigned char *screen_mem_p = NULL;

int screen_clear( MZ2000 *mz )
{
	int i;

	for (i = 0; i < TEXT_CLIP; i++)
		mz -> vram[i] = 0;
	return TRUE;
}

int screen_putmsg( MZ2000 *mz, int adr, const char *s )
{
	int i;

	for (i = 0; i < strlen(s); i++) {
		if (!s[i] || s[i] == 0x0d)
			break;
		mz -> vram[adr + i] = s[i];
	}
	return TRUE;
}

int screen_putmsgn( MZ2000 *mz, int adr, const char *s, int n )
{
	int i;

	for (i = 0; i < n; i++) {
		if (!s[i] || s[i] == 0x0d)
			break;
		mz -> vram[adr + i] = s[i];
	}
	return TRUE;
}

static int screen_setcolor( MZ2000 *mz )
{
	int i, c;
	MZScreen *scr = mz->screen;

	if (scr->isgreen || (mz -> mzmode)) {
		int t, gc = scr->F6 & 0x07U;
		int gp = scr -> F6 & 0x08U;

		if (scr -> isgreen)
			t = 4;
		else
			t = 7;
		for (i = 0; i < 16; i++) {
			if ((i >= 8) || (!gp && (gc & i)))
				c = t;
			else
				c = 0;
			if (scr -> reverse)
				c = c ? 0 : t;
			if (scr -> blank)
				c = 0;
			scr->colors[i] = scr->pixel[c];
		}
	} else {
		scr->colors[0] = scr->pixel[scr->F4];
		for (i = 1; i < 16; i++) {
			int go = (i & scr -> F6) & 0x07U;
			int pg = scr -> F5 & 8;

			if (i >= 8) {
				if (i >= 9) {
					if (go)
						c = pg ? go : (scr -> F5 & 0x07U);
					else
						c = (scr -> F5 & 0x07U);
				} else
					c = (scr -> F5 & 0x07U);
			} else {
				if (!go)
					c = scr -> F4;
				else
					c = go;
			}
			if (scr -> blank)
				c = 0;
			if (scr -> reverse && mz -> consoler_cblank)
				c = 0;
			scr->colors[i] = scr->pixel[c];
		}
	}
	return TRUE;
}

static int screen_ram2buf( MZ2000 *mz, int x, int y, int width, int height )
{
	int i, j, k;
	int chx1, chy1, chx2, chy2;
	int grx1, gry1, grx2, gry2;
	unsigned char tb, tr, tg;
	guint32 cc;
	MZScreen *scr = mz->screen;

	/* Text V-RAM */
	if (scr->issize40) {
		chx1 = x / 16;
		chy1 = y / 16;
		chx2 = (x + width + 15) / 16;
		chy2 = (y + height + 15) / 16;

		for (i = chy1; i < chy2; i++) {
			for (j = chx1; j < chx2; j++) {
				int c = mz->vram[i*40+j];

				if (mz -> font_english)
					c += 256;
				for (k = 0; k < 8; k++) {
					memcpy(&(scr->scrbuf[i*MZ2000_DISPLAY_WIDTH*16 + k*MZ2000_DISPLAY_WIDTH*2 + j*16]),
						&(scr->font->chr40[c][k*16]), 16);
					memcpy(&(scr->scrbuf[i*MZ2000_DISPLAY_WIDTH*16 + k*MZ2000_DISPLAY_WIDTH*2 + j*16 + MZ2000_DISPLAY_WIDTH]),
						&(scr->font->chr40[c][k*16]), 16);
				}
			}
		}
	} else {
		chx1 = x / 8;
		chy1 = y / 16;
		chx2 = (x + width + 7) / 8;
		chy2 = (y + height + 15) / 16;

		for (i = chy1; i < chy2; i++) {
			for (j = chx1; j < chx2; j++) {
				int c = mz->vram[i*80+j];

				if (mz -> font_english)
					c += 256;
				for (k = 0; k < 8; k++) {
					memcpy(&(scr->scrbuf[i*MZ2000_DISPLAY_WIDTH*16 + k*MZ2000_DISPLAY_WIDTH*2 + j*8]),
						&(scr->font->chr80[c][k*8]), 8);
					memcpy(&(scr->scrbuf[i*MZ2000_DISPLAY_WIDTH*16 + k*MZ2000_DISPLAY_WIDTH*2 + j*8 + MZ2000_DISPLAY_WIDTH]),
						&(scr->font->chr80[c][k*8]), 8);
				}
			}
		}
	}

	/* G-RAM */
	grx1 = x / 8;
	if ((mz -> mzmode) && (grx1 & 1))			/* 80B 320x200 */
		--grx1;
	gry1 = y / 2;
	grx2 = (x + width + 7) / 8;
	gry2 = (y + height + 1) / 2;
	for (i = gry1; i < gry2; i++) {
		for (j = grx1; j < grx2; j++) {
			if ((mz -> mzmode) && (j & 1))	/* 80B 320x200 */
				continue;
			for (k = 0; k < 8; k++) {
				u8 c;
				if (!(mz -> mzmode)) {	/* 2000 640x200 3 Pages */
					c =  ((mz->gram1[i*80+j] >> k) & 1);
					c |= ((mz->gram2[i*80+j] >> k) & 1) ? 2 : 0;
					c |= ((mz->gram3[i*80+j] >> k) & 1) ? 4 : 0;
					scr->scrbuf[i*MZ2000_DISPLAY_WIDTH*2 + j*8 + k] |= c;
					scr->scrbuf[i*MZ2000_DISPLAY_WIDTH*2 + j*8 + k
						+ MZ2000_DISPLAY_WIDTH] |= c;
				} else {			/* 80B 320x200 2 Pages */
					c =  ((mz->gram1[i*40+j/2] >> k) & 1);
					c |= ((mz->gram2[i*40+j/2] >> k) & 1) ? 2 : 0;
					scr->scrbuf[i*MZ2000_DISPLAY_WIDTH*2 + j*8 + k*2  ] |= c;
					scr->scrbuf[i*MZ2000_DISPLAY_WIDTH*2 + j*8 + k*2+1] |= c;
					scr->scrbuf[i*MZ2000_DISPLAY_WIDTH*2 + j*8 + k*2
						+ MZ2000_DISPLAY_WIDTH] |= c;
					scr->scrbuf[i*MZ2000_DISPLAY_WIDTH*2 + j*8 + k*2+1
						+ MZ2000_DISPLAY_WIDTH] |= c;
				}
			}
		}
	}

	/* Make bitmap buffer */
	for (j = y; j < y+height; j++) {
		if (!(mz -> screen_highreso) && (j % 2)) {
			for (i = x; i < x+width; i++) {
				scr->scrbufd[i * 4     + j * MZ2000_DISPLAY_WIDTH * 4] = 0;
				scr->scrbufd[i * 4 + 1 + j * MZ2000_DISPLAY_WIDTH * 4] = 0;
				scr->scrbufd[i * 4 + 2 + j * MZ2000_DISPLAY_WIDTH * 4] = 0;
				scr->scrbufd[i * 4 + 3 + j * MZ2000_DISPLAY_WIDTH * 4] = 0; 
			}
			continue;
		}
		if (!mz -> screen -> isgreen) {
			for (i = x; i < x+width; i++) {
				cc = scr -> colors[scr->scrbuf[j * MZ2000_DISPLAY_WIDTH + i]];
				tb = cc;
				tg = cc >> 8;
				tr = cc >> 16;
				scr->scrbufd[i * 4     + j * MZ2000_DISPLAY_WIDTH * 4] = tb;
				scr->scrbufd[i * 4 + 1 + j * MZ2000_DISPLAY_WIDTH * 4] = tg;
				scr->scrbufd[i * 4 + 2 + j * MZ2000_DISPLAY_WIDTH * 4] = tr;
				scr->scrbufd[i * 4 + 3 + j * MZ2000_DISPLAY_WIDTH * 4] = 0; 
			}
		} else {
			unsigned char fr, fg, fb;
			unsigned char br, bg, bb;

			if (!mz -> screen -> reverse) {
				fr = mz -> screen -> nfr;
				fg = mz -> screen -> nfg;
				fb = mz -> screen -> nfb;
				br = mz -> screen -> nbr;
				bg = mz -> screen -> nbg;
				bb = mz -> screen -> nbb;
			} else {
				br = mz -> screen -> rfr;
				bg = mz -> screen -> rfg;
				bb = mz -> screen -> rfb;
				fr = mz -> screen -> rbr;
				fg = mz -> screen -> rbg;
				fb = mz -> screen -> rbb;
			}
			for (i = x; i < x+width; i++) {
				cc = scr -> colors[scr->scrbuf[j * MZ2000_DISPLAY_WIDTH + i]];
				if (cc) {
					scr->scrbufd[i * 4     + j * MZ2000_DISPLAY_WIDTH * 4] = fb;
					scr->scrbufd[i * 4 + 1 + j * MZ2000_DISPLAY_WIDTH * 4] = fg;
					scr->scrbufd[i * 4 + 2 + j * MZ2000_DISPLAY_WIDTH * 4] = fr;
					scr->scrbufd[i * 4 + 3 + j * MZ2000_DISPLAY_WIDTH * 4] = 0;
				} else {
					scr->scrbufd[i * 4     + j * MZ2000_DISPLAY_WIDTH * 4] = bb;
					scr->scrbufd[i * 4 + 1 + j * MZ2000_DISPLAY_WIDTH * 4] = bg;
					scr->scrbufd[i * 4 + 2 + j * MZ2000_DISPLAY_WIDTH * 4] = br;
					scr->scrbufd[i * 4 + 3 + j * MZ2000_DISPLAY_WIDTH * 4] = 0;
				}
			}
		}
	}
	return TRUE;
}

int screen_redraw( MZ2000 *mz, int x, int y, int width, int height )
{
	int i, j;
	unsigned char c1, c2, c3, *tp2;
	unsigned char *tp, *p;
	MZScreen *scr = mz -> screen;
	cairo_t *cr;
	GdkPixbuf *pbuf, *pbuf2;
	GtkWidget *widget = scr -> screen_widget;
	GdkWindow *drawable = gtk_widget_get_window( widget );

	if (!screen_mem_p)
		screen_mem_p = MALLOC( MZ2000_DISPLAY_WIDTH * MZ2000_DISPLAY_HEIGHT * 3 );
	p = screen_mem_p;
	if (!p)
		return 0;
	tp = p;
	tp2 = scr -> scrbufd;
	for (j = 0; j < MZ2000_DISPLAY_HEIGHT; j++) {
		for (i = 0; i < MZ2000_DISPLAY_WIDTH; i++) {
//			c3 =  scr->scrbufd[i * 4     + j * MZ2000_DISPLAY_WIDTH * 4];
//			c2 =  scr->scrbufd[i * 4 + 1 + j * MZ2000_DISPLAY_WIDTH * 4];
//			c1 =  scr->scrbufd[i * 4 + 2 + j * MZ2000_DISPLAY_WIDTH * 4];
			c3 = *tp2++;
			c2 = *tp2++;
			c1 = *tp2++;
			tp2++;
			*tp++ =  c1;
			*tp++ =  c2;
			*tp++ =  c3;
		}
	}
	pbuf = gdk_pixbuf_new_from_data( p,
		GDK_COLORSPACE_RGB,
		FALSE,	/* has_alpha */
		8,	/* bit per sample */
		MZ2000_DISPLAY_WIDTH,
		MZ2000_DISPLAY_HEIGHT,
		MZ2000_DISPLAY_WIDTH * 3,
		NULL, NULL );
	pbuf2 = gdk_pixbuf_scale_simple( pbuf,
		uigtk_winx,
		uigtk_winy,
		GDK_INTERP_NEAREST );
	cr = gdk_cairo_create( drawable );
	gdk_cairo_set_source_pixbuf( cr, pbuf2, 0.0, 0.0 );
	cairo_paint( cr );
	cairo_destroy( cr );
	g_object_unref( pbuf2 );
	g_object_unref( pbuf );
	return TRUE;
}

int screen_redraw_region( MZ2000 *mz )
{
	screen_redraw( mz, 0, 0, MZ2000_DISPLAY_WIDTH, MZ2000_DISPLAY_HEIGHT );
	return TRUE;
}

int screen_region_add_area( MZ2000 *mz, int x, int y, int width, int height )
{
	screen_ram2buf( mz, x, y, width, height );
	return TRUE;
}

int screen_region_add_textvram( MZ2000 *mz, int addr )
{
	if (mz->screen->issize40) {
		if (addr < 40 * 25)
			screen_region_add_area( mz, (addr % 40)*16, addr/40*16, 16, 16 );
	} else {
		if (addr < 80 * 25)
			screen_region_add_area( mz, (addr % 80)*8, addr/80*16, 8, 16 );
	}
	return TRUE;
}

int screen_region_add_gram2000( MZ2000 *mz, int page, int addr )
{
	screen_region_add_area( mz, (addr % 80)*8, addr/80*2, 8, 2 );
	return TRUE;
}

int screen_region_add_gram80B( MZ2000 *mz, int page, int addr )
{
	screen_region_add_area( mz, (addr % 40)*16, addr/40*2, 16, 2 );
	return TRUE;
}

int screen_region_add_all( MZ2000 *mz )
{
	screen_region_add_area( mz, 0, 0,
		MZ2000_DISPLAY_WIDTH, MZ2000_DISPLAY_HEIGHT );
	return TRUE;
}

int screen_portF4( MZ2000 *mz, unsigned char val )
{
	if (mz->screen->F4 == (val & 7))
		return TRUE;
	mz->screen->F4 = val & 7;
	screen_setcolor(mz);
	return screen_region_add_area(mz, 0, 0, MZ2000_DISPLAY_WIDTH, MZ2000_DISPLAY_HEIGHT);
}

int screen_portF5( MZ2000 *mz, unsigned char val )
{
	if (mz->screen->F5 == (val & 15))
		return TRUE;
	mz->screen->F5 = val & 15;
	screen_setcolor(mz);
	return screen_region_add_area(mz, 0, 0, MZ2000_DISPLAY_WIDTH, MZ2000_DISPLAY_HEIGHT);
}

int screen_portF6( MZ2000 *mz, unsigned char val )
{
	if (mz->screen->F6 == (val & 15))
		return TRUE;
	mz->screen->F6 = val & 15;
	screen_setcolor(mz);
	return screen_region_add_area(mz, 0, 0, MZ2000_DISPLAY_WIDTH, MZ2000_DISPLAY_HEIGHT);
}

int screen_setreverse( MZ2000 *mz, int val )
{
	if (val >= 0)
		mz -> screen -> reverse = val;
	screen_setcolor(mz);
	return screen_region_add_area(mz, 0, 0, MZ2000_DISPLAY_WIDTH, MZ2000_DISPLAY_HEIGHT);
}

int screen_setblank( MZ2000 *mz, unsigned char val )
{
	if (mz -> screen -> blank == val)
		return TRUE;
	mz -> screen -> blank = val;
	screen_setcolor(mz);
	return screen_region_add_area(mz, 0, 0, MZ2000_DISPLAY_WIDTH, MZ2000_DISPLAY_HEIGHT);
}

static int screen_size40( MZ2000 *mz )
{
	if (mz->screen->issize40)
		return TRUE;
	mz -> screen -> issize40 = TRUE;
	mz -> ioinfo -> port_e8 &= ~0x20U;
	return screen_region_add_area(mz, 0, 0, MZ2000_DISPLAY_WIDTH, MZ2000_DISPLAY_HEIGHT);
}

static int screen_size80( MZ2000 *mz )
{
	if (!(mz->screen->issize40))
		return TRUE;
	mz -> screen -> issize40 = FALSE;
	mz -> ioinfo -> port_e8 |= 0x20U;
	return screen_region_add_area(mz, 0, 0, MZ2000_DISPLAY_WIDTH, MZ2000_DISPLAY_HEIGHT);
}

int screen_setchrsize( MZ2000 *mz, int chr80_flag )
{
	if (chr80_flag)
		return screen_size80( mz );
	else
		return screen_size40( mz );
}

static int screen_color( MZ2000 *mz )
{
	if (!(mz->screen->isgreen))
		return TRUE;
	mz->screen->isgreen = FALSE;
	screen_setcolor( mz );
	screen_region_add_area(mz, 0, 0, MZ2000_DISPLAY_WIDTH, MZ2000_DISPLAY_HEIGHT);
	return TRUE;
}

static int screen_green( MZ2000 *mz )
{
	if (mz->screen->isgreen)
		return TRUE;
	mz->screen->isgreen = TRUE;
	screen_setcolor( mz );
	screen_region_add_area(mz, 0, 0, MZ2000_DISPLAY_WIDTH, MZ2000_DISPLAY_HEIGHT);
	return TRUE;
}

int screen_setmonitor( MZ2000 *mz, int color_flag )
{
	if (color_flag)
		return screen_color( mz );
	else
		return screen_green( mz );
}

int screen_sethighreso( MZ2000 *mz, int highreso_flag )
{
	mz -> screen_highreso = highreso_flag;
	if (mz -> screen -> isgreen) {
		mz -> screen -> isgreen = FALSE;
		screen_green( mz );
	} else {
		mz -> screen -> isgreen = TRUE;
		screen_color( mz );
	}
	return TRUE;
}

int screen_setfontlang( MZ2000 *mz, int english_flag )
{
	mz -> font_english = english_flag;
	if (mz -> screen -> isgreen) {
		mz -> screen -> isgreen = FALSE;
		screen_green( mz );
	} else {
		mz -> screen -> isgreen = TRUE;
		screen_color( mz );
	}
	return TRUE;
}

int screen_setgreencolor( MZ2000 *mz,
	unsigned char nfr, unsigned char nfg, unsigned char nfb,
	unsigned char nbr, unsigned char nbg, unsigned char nbb,
	unsigned char rfr, unsigned char rfg, unsigned char rfb,
	unsigned char rbr, unsigned char rbg, unsigned char rbb )
{
	mz -> screen -> nfr = nfr;
	mz -> screen -> nfg = nfg;
	mz -> screen -> nfb = nfb;
	mz -> screen -> nbr = nbr;
	mz -> screen -> nbg = nbg;
	mz -> screen -> nbb = nbb;
	mz -> screen -> rfr = rfr;
	mz -> screen -> rfg = rfg;
	mz -> screen -> rfb = rfb;
	mz -> screen -> rbr = rbr;
	mz -> screen -> rbg = rbg;
	mz -> screen -> rbb = rbb;
	return TRUE;
}

int screen_initattr2( MZ2000 *mz )
{
	if (!screen_init_flag)
		return FALSE;
	mz->screen->F4 = 0;
	mz->screen->F5 = 7;
	mz->screen->F6 = 0;
	mz->screen->blank = FALSE;
	mz->screen->reverse = FALSE;
	mz->screen->issize40 = TRUE;
	screen_setcolor( mz );
	mz->screen->issize40 = FALSE;
	screen_size40( mz );
	return TRUE;
}

int screen_initattr( MZ2000 *mz )
{
	screen_initattr2( mz );
	screen_clear( mz );
	return TRUE;
}

int screen_initarch( MZ2000 *mz )
{
	return TRUE;
}

static MZFont *fontrom_read( const char *fontfile )
{
	int i, j, k, t;
	char tpath[PATH_MAX];
	FILE *fp, *fp2;
	MZFont *font;

	if (!(font = (MZFont *)MALLOC( sizeof(MZFont) ))) {
		fprintf( stderr, "No enough memory\n" );
		return NULL;
	}
	memset( font, 0, sizeof(MZFont) );
	if (!fontfile)
		fontfile = FONTROMFILE;
	if (!(fp = fopen( fontfile, "r" ))) {
#ifdef EMZ2000_LIBDIR
		strncpy( tpath, EMZ2000_LIBDIR "/", PATH_MAX );
		tpath[PATH_MAX - 1] = '\0';
		strncat( tpath, fontfile, PATH_MAX - strlen(tpath) - 1 );
		fp = fopen( tpath, "r");
#endif
	}
	fontfile = FONTEROMFILE;
	if (!(fp2 = fopen( fontfile, "r" ))) {
#ifdef EMZ2000_LIBDIR
		strncpy( tpath, EMZ2000_LIBDIR "/", PATH_MAX );
		tpath[PATH_MAX - 1] = '\0';
		strncat( tpath, fontfile, PATH_MAX - strlen(tpath) - 1 );
		fp2 = fopen( tpath, "r");
#endif
	}
	if (!fp && !fp2) {
		FREE( font );
		if (fp)
			fclose( fp );
		if (fp2)
			fclose( fp2 );
		return NULL;
	}
	for (i = 0; i < 256; i++) {
		for (j = 0; j < 8; j++) {
			int c1, c2;

			if (fp) {
				c1 = getc(fp);
				if (fp2)
					c2 = getc( fp2 );
				else
					c2 = c1;
			} else {
				c2 = getc( fp2 );
				if (fp)
					c1 = getc( fp );
				else
					c1 = c2;
			}
			if (c1 == EOF) {
				fprintf( stderr, "Font file format error (%d,%d)\n", i, j );
				FREE( font );
				if (fp)
					fclose( fp );
				if (fp2)
					fclose( fp2 );
				return NULL;
			}
			for (k = 0; k < 8; k++) {
				t = (c1 >> (7-k)) & 1 ? 8 : 0;
				font->chr40[i][j*16+k*2]   = t;
				font->chr40[i][j*16+k*2+1] = t;
				font->chr80[i][j*8+k]      = t;
			}
			for (k = 0; k < 8; k++) {
				t = (c2 >> (7-k)) & 1 ? 8 : 0;
				font->chr40[i+256][j*16+k*2]   = t;
				font->chr40[i+256][j*16+k*2+1] = t;
				font->chr80[i+256][j*8+k]      = t;
			}
		}
	}
	if (fp)
		fclose( fp );
	if (fp2)
		fclose( fp2 );
	return font;
}

static MZFont *font_read( const char *fontfile )
{
	int i, j, k;
	FILE *fp;
	MZFont *font;

	if (!(font = (MZFont *)MALLOC( sizeof(MZFont) ))) {
		fprintf( stderr, "No enough memory\n" );
		return NULL;
	}
	memset( font, 0, sizeof(MZFont) );
	if (!fontfile)
		fontfile = FONTFILE;
	if (!(fp = fopen( fontfile, "r" ))) {
#ifdef EMZ2000_LIBDIR
		char tpath[PATH_MAX];

		strncpy( tpath, EMZ2000_LIBDIR "/", PATH_MAX );
		tpath[PATH_MAX - 1] = '\0';
		strncat( tpath, fontfile, PATH_MAX - strlen(tpath) - 1 );
		if (!(fp = fopen( tpath, "r"))) {
#endif
			fprintf( stderr, "Can't find font file: %s\n", tpath );
			FREE( font );
			return NULL;
#ifdef EMZ2000_LIBDIR
		}
#endif
	}
	for (i = 0; i < 256; i++) {
		for (j = 0; j < 8; j++) {
			for (k = 0; k < 8; k++) {
				while (1) {
					int c = getc(fp);

					if (isspace(c))
						continue;
					else if (c == EOF) {
						fprintf( stderr, "Font file format error (%d,%d)\n", i, j );
						FREE( font );
						fclose( fp );
						return NULL;
					} else if (c == '#') {
						while ((c != '\n')&&(c != EOF)) c = getc(fp);
					} else if (c == 'X') {
						font->chr40[i][j*16+k*2]   = 8;
						font->chr40[i][j*16+k*2+1] = 8;
						font->chr80[i][j*8+k]      = 8;
						font->chr40[i+256][j*16+k*2]   = 8;
						font->chr40[i+256][j*16+k*2+1] = 8;
						font->chr80[i+256][j*8+k]      = 8;
						break;
					} else if (c == '.') {
						font->chr40[i][j*16+k*2]   = 0;
						font->chr40[i][j*16+k*2+1] = 0;
						font->chr80[i][j*8+k]      = 0;
						font->chr40[i+256][j*16+k*2]   = 0;
						font->chr40[i+256][j*16+k*2+1] = 0;
						font->chr80[i+256][j*8+k]      = 0;
						break;
					} else {
						fprintf( stderr, "Font file unexpected character: %c\n", c );
						FREE( font );
						fclose( fp );
						return NULL;
					}
				}
			}
		}
	}
	fclose( fp );
	return font;
}

void screen_exit( MZ2000 *mz )
{
	MZScreen *screen;

	uigtk_exit( mz );
	screen = mz -> screen;
	if (screen -> font) {
		FREE( screen -> font );
		screen -> font = NULL;
	}
	if (screen_mem_p) {
		FREE( screen_mem_p );
		screen_mem_p = NULL;
	}
	FREE( screen );
	mz->screen = NULL;
	screen_init_flag = FALSE;
	return;
}

int screen_init( MZ2000 *mz, const char *fontfile )
{
	int i;
	MZScreen *screen;
	GtkWidget *drawingarea;

	if (!(screen = (MZScreen *)CALLOC(sizeof(MZScreen),1))) {
		fprintf(stderr, "No enough memory\n");
		return FALSE;
	}
	mz->screen = screen;
	screen_mem_p = NULL;

	/* font read */
	if (!(screen->font = fontrom_read( fontfile ))) {
		if (!(screen->font = font_read( fontfile ))) {
			FREE(screen);
			return FALSE;
		}
		/* fprintf( stderr, "font: pattern file.\n" ); */
	} else {
		/* fprintf( stderr, "font: rom binary image.\n" ); */
	}

	/* gtk and pixmap */
	if (!uigtk_init( mz )) {
		FREE(screen);
		return FALSE;
	}
	drawingarea = screen -> screen_widget;

	/* output registers */
	screen->F4 = 0;
	screen->F5 = 7;
	screen->F6 = 0;
	screen->blank = FALSE;
	screen->reverse = FALSE;

	/* screen default */
	screen->issize40 = TRUE;
	screen->isgreen = TRUE;

	/* Colors */
	for (i = 0; i < 8; i++) {
		guint32 p = 0;
		if (i & 0x01) p |= 0xff;
		if (i & 0x02) p |= 0xff0000;
		if (i & 0x04) p |= 0xff00;
		screen->pixel[i] = p;
	}
	screen_setcolor(mz);
	screen_init_flag = TRUE;
	return TRUE;
}

/*
	Local Variables:
	mode:c++
	c-set-style:"k&r"
	c-basic-offset:8
	tab-width:8
	End:
*/
