/* File: tape.h

	SHARP MZ-2000/2200/80B/B2 Emulator "emz2000"
		emz2000 Style Tape Module Headers

	Modify the emz2000 tape.h by FUKUI, Toshio.
	This file is part of the emz2000 software.
	See copyright notice in the COPYING file.
*/

#ifndef __TAPE_H_INCLUDED
#define __TAPE_H_INCLUDED

#include <stdio.h>
#include "z80.h"

typedef struct tbuffer {
	unsigned char *buffer;
	int bufsize;
	int curoffset;
} TBUFFER;

struct st_skblank {
	z80clk restclock;
};

struct st_skrlbody {
	z80clk restclock;
};

struct st_skrepeat {
	z80clk restclock;
	int restrep;
	int replong;
};

struct st_skmzdata {
	z80clk restclock;
	int restndata;
	int curdata;
	int curbit;
	int islong;
	int checksum;
};

struct st_wrrlbody {
	int prevhl;
	int interval;
};

struct st_wrrepeat {
	z80clk restclock;
	int searchhi;
	int restrep;
	int replong;
};

struct st_wrmzdata {
	z80clk restclock;
	int size;
	int send;
	int searchhi;
	int curdata;
	int curbit;
	int datasize;
	int checksum;
	int nbits;
};

typedef struct tape TAPE;

struct tape {
	FILE *fp;
	TBUFFER *bp;
	int flags;
	int hl;  /* high  / low  */
	int state;
	z80clk clock;
	int (*skip_func)(TAPE *tp);
	int (*write_func)(TAPE *tp);
	union {
		struct st_skblank skblank;
		struct st_skrlbody skrlbody;
		struct st_skrepeat skrepeat;
		struct st_skmzdata skmzdata;
		struct st_wrrlbody wrrlbody;
		struct st_wrrepeat wrrepeat;
		struct st_wrmzdata wrmzdata;
	} u;
	int datasize;
	int shortclk;
	int longclk;
	int checkclk;
	int datablk_offset;
	int mainblk_offset;
	int nfail;
	int with_checksum;
};

#define TAPE_RUNLENGTH 1
#define TAPE_FILE2000B 2
#define VALIDREADFORMATS 2  /* the number of read formats */
#define TAPE_DATA2000B (VALIDREADFORMATS+1)
#define TAPE_INFO2000B (VALIDREADFORMATS+2)
#define TAPE_BOTH2000B (VALIDREADFORMATS+3)
#define TAPE_NONE2000B (VALIDREADFORMATS+4)
#define VALIDWRITEFORMATS (VALIDREADFORMATS+4) /* the number of write formats */

extern TAPE *tropen(const char *name);
extern TAPE *tropen_mem(unsigned char *buffer, int bufsize);
extern TAPE *twopen(const char *name,int format);
extern TAPE *twopen_mem(unsigned char *buffer, int bufsize, int format);
extern int tclose(TAPE *tp);
extern int trewind(TAPE *tp);

extern int tskip(TAPE *tp, z80clk clock);
extern int tscan(TAPE *tp);

extern int twrite(TAPE *tp, int bit, z80clk clock);

extern int tcopy(TAPE *intp, TAPE *outtp, int step);
extern int tcheck_tape(TAPE *intp);

#endif /* __TAPE_H_INCLUDED */

/*
	Local Variables:
	mode:c++
	c-set-style:"k&r"
	c-basic-offset:8
	tab-width:8
	End:
*/
