/* File: tuchar.h

	Code Converter and Other Functions for Unicode/UCS2or4
		(require libiconv)

	Copyright (C) 2000-2019 FUKUI, Toshio.
	This file is part of the emz2000 software.
	See copyright notice in the COPYING file.
*/

#ifndef __TUCHAR_H_INCLUDED
#define __TUCHAR_H_INCLUDED

#if defined(_LANGUAGE_C_PLUS_PLUS) || defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif
#if 0
}
#endif

#define TUCHAR_USE_UCS4	/* select UCS2/UCS4 select */

#ifdef TUCHAR_USE_UCS4
#define TUCHAR_ERR	(0xffffffffU)
#define TUCHAR_WSPACE	(0x00003000U)
#define TUCHAR_WBYTE	(0x00000100U)

typedef unsigned int TUCHAR;
#else
#define TUCHAR_ERR	(0xffffU)
#define TUCHAR_WSPACE	(0x3000U)
#define TUCHAR_WBYTE	(0x0100U)

typedef unsigned short TUCHAR;
#endif
#define TUCHAR_NULL	((TUCHAR)'\0')
#define TUCHAR_LF	((TUCHAR)'\n')
#define TUCHAR_CR	((TUCHAR)'\r')
#define TUCHAR_TAB	((TUCHAR)'\t')
#define TUCHAR_SPACE	((TUCHAR)' ')

#define TUCHAR_RESULT_NOERROR	0

typedef enum {
	TUCHAR_TYPELIST_HALFSPACE = 0,
	TUCHAR_TYPELIST_HALFCHARS,
	TUCHAR_TYPELIST_WIDESPACE,
	TUCHAR_TYPELIST_WIDECHARS,
	TUCHAR_TYPELIST_MAXNUM
} TUCHAR_TYPELIST;

extern int tuchar_isdigit( TUCHAR ch );
extern int tuchar_isalpha( TUCHAR ch );
extern int tuchar_isalnum( TUCHAR ch );
extern int tuchar_islower( TUCHAR ch );
extern int tuchar_isupper( TUCHAR ch );
extern int tuchar_isgraph( TUCHAR ch );
extern TUCHAR tuchar_tolower( TUCHAR ch );
extern TUCHAR tuchar_toupper( TUCHAR ch );
extern TUCHAR_TYPELIST tuchar_getchartype( TUCHAR ch );
extern int tuchar_getbytes( TUCHAR ch );
extern int tuchar_getbytes_utf8( TUCHAR ch );
extern int tuchar_strlen( const TUCHAR *s );
extern TUCHAR *tuchar_strcpy( TUCHAR *s, const char *cp );
extern TUCHAR *tuchar_strcat( TUCHAR *s, const char *cp );
extern TUCHAR *tuchar_loc2tu( const char *loc_p, size_t loc_len );
extern TUCHAR *tuchar_file2tu( const char *loc_p, size_t loc_len );
extern TUCHAR *tuchar_utf2tu( const char *loc_p, size_t loc_len );
extern char *tuchar_tu2loc( const TUCHAR *tuchar_p, size_t tuchar_len );
extern char *tuchar_tu2file( const TUCHAR *tuchar_p, size_t tuchar_len );
extern char *tuchar_tu2utf( const TUCHAR *tuchar_p, size_t tuchar_len );
extern int tuchar_fputs( const TUCHAR *tuchar_p, size_t tuchar_len, FILE *fp );
extern int tuchar_setcodingname( const char *name, int use_external_codename_buf );
extern int tuchar_setfilecodingname( const char *name, int use_external_codename_buf );
extern const char *tuchar_getcodingname( void );
extern const char *tuchar_getfilecodingname( void );
extern int tuchar_init( const char *locstr );
extern void tuchar_shutdown( void );

#if 0
{
#endif
#if defined(_LANGUAGE_C_PLUS_PLUS) || defined(__cplusplus) || defined(c_plusplus)
}
#endif

#endif /* TUCHAR_H_INCLUDED */

/*
	Local Variables:
	mode:c++
	c-set-style:"k&r"
	c-basic-offset:8
	tab-width:8
	End:
*/
