/* File: uigtk.c

	SHARP MZ-2000/2200/80B/B2 Emulator "emz2000"
		libgtk modules for main routines UI

	Copyright (C) 2002-2019 FUKUI, Toshio.
	This file is part of the emz2000 software.
	See copyright notice in the COPYING file.
*/

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <unistd.h>
#include <gtk/gtk.h>

#include "common.h"
#include "mz2000.h"
#include "uigtk.h"
#include "plugin.h"

/* window information */
GtkWidget	*uigtk_window = NULL;
GtkWidget	*uigtk_vbox = NULL;
GtkActionGroup	*uigtk_actions = NULL;
gint uigtk_winx = MZ2000_DISPLAY_WIDTH;
gint uigtk_winy = MZ2000_DISPLAY_HEIGHT;
gint uigtk_winadd_x = -1;
gint uigtk_winadd_y = -1;

/* local information */
gint uigtk_timeout_refscid = -1;
volatile int uigtk_refresh_screen_flag = FALSE;
MZ2000 *uigtk_mz = NULL;
int redraw_end_flag = FALSE;

/* initialized flags */
gint uigtk_enable_messagebox_flag = FALSE;

static gint uigtk_key_press( GtkWidget *widget, GdkEventKey *event, MZ2000 *mz )
{
	if (!(event->state & GDK_MOD1_MASK)) {
		keyboard_press( mz, event->keyval, event->hardware_keycode );
		g_signal_stop_emission_by_name( widget, "key-press-event" );
	} else {
		int flag = FALSE;

		switch (event -> keyval) {
		case GDK_KEY_F1 :	/* pause */
			setplaystop( uigtk_mz, TRUE );
			uigtk_mz -> pause = !uigtk_mz -> pause;
			setplaystop( uigtk_mz, FALSE );
			flag = TRUE;
			break;
		case GDK_KEY_F2 :	/* none (debug information) */
			flag = TRUE;
			break;
		case GDK_KEY_F3 :	/* change sound mode */
			uigtk_mz -> soundmode = !uigtk_mz -> soundmode;
			sound_reset( uigtk_mz );
			flag = TRUE;
			break;
		case GDK_KEY_F4 :	/* change display mode */
			if (!(uigtk_mz -> displayMode)) {
				screen_setmonitor( mz, TRUE );
				uigtk_mz -> displayMode = TRUE;
			} else {
				screen_setmonitor( mz, FALSE );
				uigtk_mz -> displayMode = FALSE;
			}
			flag = TRUE;
			break;
		case GDK_KEY_F8 :	/* state save */
			uigtk_menu_savestate();
			flag = TRUE;
			break;
		case GDK_KEY_F9 :	/* clock down */
		case ';' :
			uigtk_mz -> nowZ80Clock = uigtk_mz -> nowZ80Clock - 1000;
			if (uigtk_mz -> nowZ80Clock < 1000)
				uigtk_mz -> nowZ80Clock = 1000;
			setspeed_z80( uigtk_mz -> nowZ80Clock );
			flag = TRUE;
			break;
		case GDK_KEY_F10 :	/* clock up */
		case ':' :
			uigtk_mz -> nowZ80Clock = uigtk_mz -> nowZ80Clock + 1000;
			if (uigtk_mz -> nowZ80Clock > uigtk_mz -> z80MaxClock)
				uigtk_mz -> nowZ80Clock = uigtk_mz -> z80MaxClock;
			setspeed_z80( uigtk_mz -> nowZ80Clock );
			flag = TRUE;
			break;
		case ']' :	/* clock default */
			uigtk_mz -> nowZ80Clock = DEFAULT_SPEED_KHZ;
			setspeed_z80( uigtk_mz -> nowZ80Clock );
			flag = TRUE;
			break;
		case GDK_KEY_F11 :	/* reset */
			mz -> needReset = TRUE;
			stop_z80();
			setplaystop( mz, FALSE );
			flag = TRUE;
			break;
		case GDK_KEY_F12 :	/* IPL reset */
			mz -> needIpl = TRUE;
			stop_z80();
			setplaystop( mz, FALSE );
			flag = TRUE;
			break;
		case ',' :
			if (!mz2tape_getkinh())
				mz2tape_rew( uigtk_mz );
			flag = TRUE;
			break;
		case '.' :
			if (!mz2tape_getkinh())
				mz2tape_ff( uigtk_mz );
			flag = TRUE;
			break;
		case '/' :
			if (!mz2tape_getkinh())
				mz2tape_stop( uigtk_mz );
			flag = TRUE;
			break;
		case '\\' :
			if (!mz2tape_getkinh())
				mz2tape_eject( uigtk_mz, TRUE );
			flag = TRUE;
			break;
		default :
			break;
		}
		if (flag)
			g_signal_stop_emission_by_name( widget, "key-press-event" );
	}
	return FALSE;
}

static gint uigtk_key_release( GtkWidget *widget, GdkEventKey *event, MZ2000 *mz )
{
	if (!(event->state & GDK_MOD1_MASK)) {
		keyboard_release( mz, event->keyval, event->hardware_keycode );
		g_signal_stop_emission_by_name( widget, "key-release-event" );
	}
	return FALSE;
}

static gint uigtk_win_configure_event( GtkWidget *widget, GdkEventConfigure *event, MZ2000 *mz )
{
	gint sx, sy;
	GtkAllocation allocation;

	gtk_widget_get_allocation( widget, &allocation );
	sx = allocation.width;
	sy = allocation.height;
	//printf( "main_resize:%d %d\n", sx, sy );
	return FALSE;
}

static gint uigtk_windowstate( GtkWidget *widget, GdkEventWindowState *event, MZ2000 *mz )
{
	uigtk_mz -> screen_window = event -> new_window_state;
	return TRUE;
}

static gint uigtk_close( GtkWidget *widget, GdkEventAny *event, MZ2000 *mz )
{
	uigtk_menu_quit();
	return TRUE;
}

static gint uigtk_destroy( GtkWidget *widget, GdkEventAny *event, MZ2000 *mz )
{
	mz = uigtk_mz;
	if (uigtk_timeout_refscid >= 0) {
		uigtk_refresh_screen_flag = FALSE;
		uigtk_timeout_refscid = -1;
		usleep( GUI_REFRESH_RATE * 3 * 1000 );
	}
	if (mz -> statusbar) {
		gtk_statusbar_pop( GTK_STATUSBAR(mz -> statusbar), mz->statusbarid );
		gtk_container_remove( GTK_CONTAINER(uigtk_vbox), GTK_WIDGET(mz->statusbar) );
		mz -> statusbar = NULL;
	}
	if (mz -> screen -> screen_widget) {
		gtk_container_remove( GTK_CONTAINER(uigtk_vbox), GTK_WIDGET(mz->screen->screen_widget) );
		mz -> screen -> screen_widget = NULL;
	}
	if (uigtk_vbox) {
		gtk_container_remove( GTK_CONTAINER(uigtk_window), uigtk_vbox );
		uigtk_vbox = NULL;
	}
	if (uigtk_window) {
		//gtk_window_position( GTK_WINDOW(uigtk_window), GTK_WIN_POS_NONE );
		//gdk_window_get_root_origin( gtk_widget_get_window( GTK_WIDGET(uigtk_window) ), &uigtk_posx, &uigtk_posy );
		gtk_widget_hide( GTK_WIDGET(uigtk_window) );
		gtk_widget_destroy( uigtk_window );
		uigtk_window = NULL;
	}
	gtk_main_quit();
	return TRUE;
}

void uigtk_dropfiles( GtkWidget *widget, GdkDragContext *context,
	gint x, gint y,
	GtkSelectionData *data,
	guint info, guint time_, MZ2000 *mz )
{
	int i, r, data_ct;
	char *p;
	gchar **urilist;
	gchar **filelist;
	gchar *hostname;
	GError *error = NULL;

	if (!(gtk_selection_data_get_length(data)) || gtk_selection_data_get_format(data) != 8) {
		uigtk_messagebox( "Drop Data Error - 0", "Error",
			UIGTK_BUTTON_OKONLY | UIGTK_ICON_ERROR );
		gtk_drag_finish( context, FALSE, FALSE, time_ );
		return;
	}
	if (info != 0) {
		uigtk_messagebox( "Drop Data Error - 1", "Error",
			UIGTK_BUTTON_OKONLY | UIGTK_ICON_ERROR );
		gtk_drag_finish( context, FALSE, FALSE, time_ );
		return;
	}
	urilist = gtk_selection_data_get_uris( data );
	if (!urilist) {
		uigtk_messagebox( "Drop Data Error - 2", "Error",
			UIGTK_BUTTON_OKONLY | UIGTK_ICON_ERROR );
		gtk_drag_finish( context, FALSE, FALSE, time_ );
		return;
	}
	for (data_ct = 0; urilist[data_ct]; data_ct++)
		;
	data_ct++;
	filelist = g_new0( gchar *, data_ct );
	for (i = 0; urilist[i]; i++) {
		filelist[i] = g_filename_from_uri( urilist[i], &hostname, &error );
		if (!filelist[i]) {
			uigtk_messagebox( "Drop Data Error - 3", "Error",
				UIGTK_BUTTON_OKONLY | UIGTK_ICON_ERROR );
			g_clear_error( &error );
			g_free( hostname );
			g_strfreev( filelist );
			g_strfreev( urilist );
			gtk_drag_finish( context, FALSE, FALSE, time_ );
		}
		g_free( hostname );
	}
	g_strfreev( urilist );

	setplaystop( mz, TRUE );
	for (i = 0; filelist[i]; i++) {
		p = strrchr( filelist[i], '.' );
		if (!p) {
			uigtk_messagebox( "No extention filename", "Error",
				UIGTK_BUTTON_OKONLY | UIGTK_ICON_ERROR );
		} else {
			p++;
			if (!strcasecmp( p, "d88" ) || !strcasecmp( p, "d20" ) || !strcasecmp( p, "d77" ) || !strcasecmp( p, "dsk" )) {
				disk_change( mz, mz -> droptofd2 ? 1 : 0, filelist[i], TRUE, FALSE, mz -> forcereadonly );
				if (mz -> dropautoipl && !(mz -> droptofd2)) {
					mz -> needIpl = TRUE;
					stop_z80();
				}
			} else if (!strcasecmp( p, "2d" )) {
				disk_change( mz, mz -> droptofd2 ? 1 : 0, filelist[i], FALSE, FALSE, mz -> forcereadonly );
				if (mz -> dropautoipl && !(mz -> droptofd2)) {
					mz -> needIpl = TRUE;
					stop_z80();
				}
			} else if (!strcasecmp( p, "q20" )) {
				strncpy( mz -> qfilename, filelist[i], PATH_MAX );
				mz -> qfilename[PATH_MAX - 1] = '\0';
				plugin_device_change( PLUGIN_DEVICETYPE_QD, 0, filelist[i], 0, FALSE, mz -> forcereadonly );
			} else if (!strcasecmp( p, "mze" )) {
				strncpy( mz -> sfilename, filelist[i], PATH_MAX );
				mz -> sfilename[PATH_MAX - 1] = '\0';
				load_state( mz );
			} else {
				strncpy( mz -> tfilename, filelist[i], PATH_MAX );
				mz -> tfilename[PATH_MAX - 1] = '\0';
				r = mz2tape_exec( mz, filelist[i], mz -> forcereadonly );
				if (r <= MZ2TAPE_FILEMODE_ERRNDLG) {
					uigtk_messagebox( "Unknown type filename", "Error",
						UIGTK_BUTTON_OKONLY | UIGTK_ICON_ERROR );
				}
				if (r == MZ2TAPE_FILEMODE_MTI || r == MZ2TAPE_FILEMODE_WAV) {
					if (mz -> dropautoipltape && (!(mz -> enable_fddif) || !(mz -> howipl))) {
						mz -> needIpl = TRUE;
						stop_z80();
					}
				}
			}
		}
	}
	g_strfreev(filelist);
	gtk_drag_finish( context, TRUE, FALSE, time_ );
	setplaystop( mz, FALSE );
	return;
}

static gint uigtk_screen_configure_event( GtkWidget *widget, GdkEventConfigure *event, MZ2000 *mz )
{
	GtkAllocation allocation;

	gtk_widget_get_allocation( widget, &allocation );
	uigtk_winx = allocation.width;
	uigtk_winy = allocation.height;
	//printf( "resize-drawarea:%d %d\n", uigtk_winx, uigtk_winy );
	return FALSE;
}

static GtkWidget *uigtk_screen( GtkWidget *main_window, MZ2000 *mz )
{
	GtkWidget *screen_widget;

	screen_widget = GTK_WIDGET( gtk_drawing_area_new() );
	gtk_widget_add_events( screen_widget, GDK_EXPOSURE_MASK
			  | GDK_ENTER_NOTIFY_MASK
			  | GDK_LEAVE_NOTIFY_MASK
			  | GDK_KEY_PRESS_MASK
			  | GDK_KEY_RELEASE_MASK);
	g_signal_connect( main_window, "key-press-event",
			G_CALLBACK(uigtk_key_press), mz );
	g_signal_connect( main_window, "key-release-event",
			G_CALLBACK(uigtk_key_release), mz );
	return screen_widget;
}

static gboolean uigtk_refresh_screen( gpointer user_data )
{
	int i, tape_runmode, tapeend;
	unsigned long pos, total, offset, tfreq, dfreq;
	char c, c2, c3, c4, c5, c6;
	char buf[LNMAX], buf2[LNMAX], buf3[LNMAX];
	MZ2000 *mz = (MZ2000 *)user_data;

	gtk_statusbar_pop( GTK_STATUSBAR(mz -> statusbar), mz->statusbarid );
	if (!(mz -> mzmode))
		snprintf( buf, LNMAX, "MZ-2000/2200 " );
	else
		snprintf( buf, LNMAX, "MZ-80B/B2 " );
	if (!(mz -> pause)) {
		if (!getspeed_z80())
			snprintf( buf2, LNMAX, "NO WAIT        " );
		else
			snprintf( buf2, LNMAX, "%dMHz        ", getspeed_z80() / 1000 );
	} else
		snprintf( buf2, LNMAX, "PAUSE        " );
	strncat( buf, buf2, LNMAX - strlen(buf) - 1 );
	for (i = 0; i < MZ_DISK_DRIVE_MAX; i++) {
		if (disk_getwritable( mz, i ))
			c2 = ' ';
		else
			c2 = 'r';
		switch (disk_getexist( mz, i )) {
		case DISK_STATUS_INSERT :
			c = '+';
			if (disk_getaccess( mz, i ))
				c = '*';
			break;
		case DISK_STATUS_WCHANGED :
			c = 'W';
			if (disk_getaccess( mz, i ))
				c = '*';
			break;
		default :
			c = '-';
			c2 = ' ';
			if (disk_getaccess( mz, i ))
				c = '*';
			break;
		}
		snprintf( buf2, LNMAX, "FD%d:%c%c ", i+1, c, c2 );
		strncat( buf, buf2, LNMAX - strlen(buf) - 1 );
	}
	if (mz2tape_getwritable())
		c2 = ' ';
	else
		c2 = 'r';
	tape_runmode = mz2tape_getstatus( &pos, &total, &offset, &tapeend,
		&tfreq, &dfreq );
	switch (tape_runmode) {
	case MZ2TAPE_RUNMODE_STOP :
		c = '+';
		break;
	case MZ2TAPE_RUNMODE_PLAY :
		c = '*';
		break;
	case MZ2TAPE_RUNMODE_RECORD :
		c = 'W';
		break;
	case MZ2TAPE_RUNMODE_FF :
		c = '>';
		break;
	case MZ2TAPE_RUNMODE_REW :
		c = '<';
		break;
	default :
		c = '-';
		c2 = ' ';
		break;
	}
	if (tape_runmode >= 0) {
		switch (mz -> tapecounter) {
		case TAPE_COUNTERMODE_MZ2080 :
			pos = (unsigned long)(((unsigned long long)(pos - offset))
				* (unsigned long long)dfreq * 10 / (unsigned long long)tfreq);
			total = (unsigned long)(((unsigned long long)(total - offset))
				* (unsigned long long)dfreq * 10  / (unsigned long long)tfreq);
			if (!(mz -> mzmode)) {
				pos   = pos / mz -> tapectspeed_2000;
				total = total / mz -> tapectspeed_2000;
			} else {
				pos   = pos / mz -> tapectspeed_80B;
				total = total / mz -> tapectspeed_80B;
			}
			break;
		case TAPE_COUNTERMODE_MZ2500 :
			pos = (unsigned long)(((unsigned long long)(pos - offset))
				* (unsigned long long)dfreq * 10 / (unsigned long long)tfreq);
			total = (unsigned long)(((unsigned long long)(total - offset))
				* (unsigned long long)dfreq * 10  / (unsigned long long)tfreq);
			pos   = pos   / mz -> tapectspeed_2500;
			total = total / mz -> tapectspeed_2500;
			break;
		case TAPE_COUNTERMODE_KBYTES :
			pos   = (pos   + 500) / 1000;
			total = (total + 500) / 1000;
			break;
		default : /* TAPE_COUNTERMODE_BYTES */
			break;
		}
		switch (mz -> tapecounter) {
		case TAPE_COUNTERMODE_MZ2080 :
		case TAPE_COUNTERMODE_MZ2500 :
			if (tapeend)
				snprintf( buf3, LNMAX, "(%03lu.%01lu-END/%03lu.%01lu) ",
					pos / 10, pos % 10, total / 10, total % 10 );
			else
				snprintf( buf3, LNMAX, "(%03lu.%01lu/%03lu.%01lu) ",
					pos / 10, pos % 10, total / 10, total % 10 );
			break;
		case TAPE_COUNTERMODE_KBYTES :
		default :
			if (tapeend)
				snprintf( buf3, LNMAX, "(%03lu-END/%03lu) ", pos, total );
			else
				snprintf( buf3, LNMAX, "(%03lu/%03lu) ", pos, total );
			break;
		}
	} else {
		snprintf( buf3, LNMAX, "(-/-) " );
	}
	if (c2 != ' ')
		snprintf( buf2, LNMAX, "CAS:%c%c", c, c2 );
	else
		snprintf( buf2, LNMAX, "CAS:%c", c );
	strncat( buf, buf2, LNMAX - strlen(buf) - 1 );
	strncat( buf, buf3, LNMAX - strlen(buf) - 1 );

	if (plugin_device_getinterface( PLUGIN_DEVICETYPE_QD )) {
		if (plugin_device_getwritable( PLUGIN_DEVICETYPE_QD, 0 ))
			c2 = ' ';
		else
			c2 = 'r';
		switch (plugin_device_getexist( PLUGIN_DEVICETYPE_QD, 0 )) {
		case DISK_STATUS_INSERT :
		case DISK_STATUS_WCHANGED :
			c = '+';
			if (plugin_device_getaccess( PLUGIN_DEVICETYPE_QD, 0 ))
				c = '*';
			break;
		default :
			c = '-';
			c2 = ' ';
			break;
		}
		snprintf( buf2, LNMAX, " QD:%c%c ", c, c2 );
		strncat( buf, buf2, LNMAX - strlen(buf) - 1 );
	}
	if (mz -> mzmode) {
		if (mz->ioinfo->port_e0 & 0x20U)
			c = 'S';
		else
			c = '-';
		if (mz->ioinfo->port_e0 & 0x40U)
			c2 = 'G';
		else
			c2 = '-';
		if (mz->ioinfo->port_e0 & 0x80U)
			c3 = 'K';
		else
			c3 = '-';
		snprintf( buf2, LNMAX, " LED:%c%c%c ", c, c2, c3 );
		strncat( buf, buf2, LNMAX - strlen(buf) - 1 );
	}

	if (mz -> keyboard_mode) {
		if (mz -> keyboard_mode & 0x01U)
			c = '1';
		else
			c = '-';
		if (mz -> keyboard_mode & 0x02U)
			c2 = '2';
		else
			c2 = '-';
		if (mz -> keyboard_mode & 0x04U)
			c3 = '3';
		else
			c3 = '-';
		if (mz -> keyboard_mode & 0x08U)
			c4 = '4';
		else
			c4 = '-';
		if (mz -> keyboard_mode & 0x10U)
			c5 = '5';
		else
			c5 = '-';
		if (mz -> keyboard_mode & 0x20U)
			c6 = '6';
		else
			c6 = '-';
		snprintf( buf2, LNMAX, " KBD:%c%c%c%c%c%c ", c, c2, c3, c4, c5, c6 );
		strncat( buf, buf2, LNMAX - strlen(buf) - 1 );
	}

	gtk_statusbar_push( GTK_STATUSBAR(mz -> statusbar), mz->statusbarid, buf );
	screen_redraw_region( mz );
	uigtk_menu_updateall();

	/* resize */
	if (!redraw_end_flag) {
		int sx, sy;
		GtkAllocation allocation;

		gtk_widget_get_allocation( GTK_WIDGET(uigtk_window), &allocation );
		sx = allocation.width;
		sy = allocation.height;
		//printf("window size=%d %d\n", sx, sy);
		if ((uigtk_winadd_x < 0 || uigtk_winadd_y < 0) && sx > 1 && sy > 1) {
			if (uigtk_winy < 200) {
				uigtk_winadd_x = 0;
				uigtk_winadd_y = sy;
				//printf("add_y %d\n", uigtk_winadd_y);
			}
		}
		if (uigtk_winadd_x >= 0 && uigtk_winadd_y >=0) {
			if (uigtk_winx > 1 && uigtk_winy > 1)
				redraw_end_flag = TRUE;
			else {
				gtk_window_resize( GTK_WINDOW(uigtk_window),
					MZ2000_DISPLAY_WIDTH  + uigtk_winadd_x,
					MZ2000_DISPLAY_HEIGHT + uigtk_winadd_y );
			}
		}
	}
	return uigtk_refresh_screen_flag;
}

int uigtk_exit( MZ2000 *mz )
{
	return 0;
}

static GtkTargetEntry dropdata_list[] = {
	{ "text/uri-list",  0, 0 },
};

int uigtk_init( MZ2000 *mz )
{
	int r;
	GtkWidget *vbox;
#ifdef EMZ2000_LIBDIR
	GError *error = NULL;
	char tpath[PATH_MAX];
#endif

	uigtk_mz = mz;
	redraw_end_flag = FALSE;

	/* root window */
	uigtk_window = gtk_window_new( GTK_WINDOW_TOPLEVEL );
	gtk_widget_set_name( uigtk_window, APPWINDOWNAME );
	gtk_window_set_title( GTK_WINDOW(uigtk_window),
		"MZ-2000/2200/80B/B2 Emulator \"emz2000\" TF-Edition" );
#ifdef EMZ2000_LIBDIR
	strncpy( tpath, EMZ2000_LIBDIR "/", PATH_MAX );
	tpath[PATH_MAX - 1] = '\0';
	strncat( tpath, ICONFILENAME, PATH_MAX - strlen(tpath) - 1 );
	r = gtk_window_set_default_icon_from_file( tpath, &error );
#else
	r = FALSE;
#endif
	if (!r)
		r = gtk_window_set_default_icon_from_file( ICONFILENAME, NULL );
 	gtk_window_set_resizable( GTK_WINDOW(uigtk_window), FALSE );
	gtk_container_set_border_width( GTK_CONTAINER(uigtk_window), 0 );
	g_signal_connect( uigtk_window, "configure-event",
		G_CALLBACK(uigtk_win_configure_event), mz );
	g_signal_connect( uigtk_window, "delete-event",
		G_CALLBACK(uigtk_close), mz );
	g_signal_connect( uigtk_window, "destroy",
		G_CALLBACK(uigtk_destroy), mz );
	g_signal_connect( uigtk_window, "window-state-event",
		G_CALLBACK(uigtk_windowstate), mz );
	g_signal_connect( uigtk_window, "drag-data-received",
		G_CALLBACK(uigtk_dropfiles), mz );
	gtk_widget_show( uigtk_window );
	gtk_drag_dest_set( GTK_WIDGET(uigtk_window), GTK_DEST_DEFAULT_ALL,
		dropdata_list, sizeof(dropdata_list) / sizeof(dropdata_list[0]),
		GDK_ACTION_DEFAULT |
		GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK | GDK_ACTION_ASK  );

	/* vbox */
	vbox = gtk_box_new( GTK_ORIENTATION_VERTICAL, 0 );
	uigtk_vbox = vbox;
	gtk_container_add( GTK_CONTAINER(uigtk_window), vbox );
	gtk_widget_show( GTK_WIDGET(vbox) );

	/* menu and toolbar */
	uigtk_create_menu( vbox );

	/* drawingarea */
	mz -> screen -> screen_widget = uigtk_screen( uigtk_window, mz );
	gtk_box_pack_start( GTK_BOX(vbox), mz -> screen -> screen_widget, TRUE, TRUE, 0 );
	gtk_widget_show( mz -> screen -> screen_widget );
	g_signal_connect( mz -> screen -> screen_widget, "configure-event",
		G_CALLBACK(uigtk_screen_configure_event), mz );

	/* status bar */
	mz -> statusbar = gtk_statusbar_new();
	mz -> statusbarid = gtk_statusbar_get_context_id( GTK_STATUSBAR( mz -> statusbar ), "Status Display" );
	gtk_statusbar_push( GTK_STATUSBAR(mz -> statusbar), mz->statusbarid, "" );
	gtk_box_pack_end( GTK_BOX(vbox), mz -> statusbar, FALSE, TRUE, 0 );
	gtk_widget_show( mz -> statusbar );

	/* refresh timer */
	uigtk_refresh_screen_flag = TRUE;
	uigtk_timeout_refscid = g_timeout_add( GUI_REFRESH_RATE, uigtk_refresh_screen, mz );

	/* draw all */
	gtk_widget_show_all( uigtk_window );
	uigtk_menu_updateall();

	/* allow resize. current window size is gtk 3.16 <= (1,1), 3.18 >= (200,200) */
	gtk_window_set_resizable( GTK_WINDOW(uigtk_window), TRUE );
	gtk_window_resize( GTK_WINDOW(uigtk_window), 1, 1 );

	return TRUE;
}

int uigtk_enable_messagebox( void )
{
	uigtk_enable_messagebox_flag = TRUE;
	return 0;
}

/*
	Local Variables:
	mode:c++
	c-set-style:"k&r"
	c-basic-offset:8
	tab-width:8
	End:
*/
