/* File: uigtk_clockdlg.c

	SHARP MZ-2000/2200/80B/B2 Emulator "emz2000"
		libgtk modules for a hardware speed settings dialog

	Copyright (C) 2003-2019 FUKUI, Toshio.
	This file is part of the emz2000 software.
	See copyright notice in the COPYING file.
*/

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <gtk/gtk.h>

#include "common.h"
#include "mz2000.h"
#include "uigtk.h"

static MZ2000 *mz;
static GtkAdjustment *adj_sysclocks;
static GtkEntry *entry_textwait_r, *entry_textwait_w;
static GtkEntry *entry_gwait2000_r, *entry_gwait2000_w;
static GtkEntry *entry_gwait80b_r, *entry_gwait80b_w;
static GtkEntry *entry_alphaclocks;

int uigtk_clockdlg_domodal( MZ2000 *_mz )
{
	int t;
	char buf[LNMAX];
	gint r;
	const gchar *s;
	GtkBuilder *builder;
	GtkDialog *dialog;

	mz = _mz;
	builder = gtk_builder_new();
	r = gtk_builder_add_from_file( builder, GLADEUIFILE, NULL );
	dialog = GTK_DIALOG( gtk_builder_get_object( builder, "clock_dialog" ) );
#ifdef EMZ2000_LIBDIR
	if (/* r || */ !dialog) {
		r = gtk_builder_add_from_file( builder, EMZ2000_LIBDIR "/" GLADEUIFILE, NULL );
		dialog = GTK_DIALOG( gtk_builder_get_object( builder, "clock_dialog" ) );
	}
#endif
	if (/* r || */ !dialog) {
		fprintf( stderr, GLADEUIFILE " and " EMZ2000_LIBDIR "/" GLADEUIFILE " not found or broken.\n" );
		g_object_unref( builder );
		return UIGTK_IDCANCEL;
	}
	gtk_builder_connect_signals( builder, NULL );
	//gtk_widget_set_parent_window( GTK_WIDGET(dialog), uigtk_window );

	adj_sysclocks = GTK_ADJUSTMENT( gtk_builder_get_object( builder, "adjustment_sysclocks" ) );
	gtk_adjustment_set_value( adj_sysclocks, mz -> nowZ80Clock / 1000 );
	entry_textwait_r = GTK_ENTRY( gtk_builder_get_object( builder, "entry_textwait_r" ) );
	snprintf( buf, LNMAX, "%d", mz -> memory_waittvramclock_r );
	gtk_entry_set_text( entry_textwait_r, buf );
	entry_textwait_w = GTK_ENTRY( gtk_builder_get_object( builder, "entry_textwait_w" ) );
	snprintf( buf, LNMAX, "%d", mz -> memory_waittvramclock_w );
	gtk_entry_set_text( entry_textwait_w, buf );
	entry_gwait2000_r = GTK_ENTRY( gtk_builder_get_object( builder, "entry_gramwait2000_r" ) );
	snprintf( buf, LNMAX, "%d", mz -> memory_waitgramclock_r );
	gtk_entry_set_text( entry_gwait2000_r, buf );
	entry_gwait2000_w = GTK_ENTRY( gtk_builder_get_object( builder, "entry_gramwait2000_w" ) );
	snprintf( buf, LNMAX, "%d", mz -> memory_waitgramclock_w );
	gtk_entry_set_text( entry_gwait2000_w, buf );
	entry_gwait80b_r = GTK_ENTRY( gtk_builder_get_object( builder, "entry_gramwait80b_r" ) );
	snprintf( buf, LNMAX, "%d", mz -> memory_waitgram80bclock_r );
	gtk_entry_set_text( entry_gwait80b_r, buf );
	entry_gwait80b_w = GTK_ENTRY( gtk_builder_get_object( builder, "entry_gramwait80b_w" ) );
	snprintf( buf, LNMAX, "%d", mz -> memory_waitgram80bclock_w );
	gtk_entry_set_text( entry_gwait80b_w, buf );
	entry_alphaclocks = GTK_ENTRY( gtk_builder_get_object( builder, "entry_alphaclocks" ) );
	snprintf( buf, LNMAX, "%d", mz -> frameclock_alpha );
	gtk_entry_set_text( entry_alphaclocks, buf );

	gtk_dialog_add_buttons( dialog,
		GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
		"DEFAULT", GTK_RESPONSE_NO, NULL );

retry:	r = gtk_dialog_run( GTK_DIALOG(dialog) );
	switch (r) {
	case GTK_RESPONSE_ACCEPT :
		t = (int)gtk_adjustment_get_value( adj_sysclocks );
		if (t < 1 || t > mz -> z80MaxClock / 1000) {
			uigtk_messagebox( "System clock value range error",
					  "Hardware Speed Settings", UIGTK_BUTTON_OKONLY | UIGTK_ICON_ERROR );
			goto retry;
		}
		mz -> nowZ80Clock = t * 1000;
		if (mz -> nowZ80Clock > mz -> z80MaxClock)
			mz -> nowZ80Clock = mz -> z80MaxClock;
		if (mz -> nowZ80Clock < 1000)
			mz -> nowZ80Clock = 1000;

		s = gtk_entry_get_text( entry_textwait_r );
		if (sscanf( s, "%d", &t ) != 1 || t < 1 || t > 1000) {
			uigtk_messagebox( "Text V-RAM read wait value range error (1-1000)",
				"Hardware Speed Settings", UIGTK_BUTTON_OKONLY | UIGTK_ICON_ERROR );
			goto retry;
		}
		mz -> memory_waittvramclock_r = t;
		s = gtk_entry_get_text( entry_textwait_w );
		if (sscanf( s, "%d", &t ) != 1 || t < 1 || t > 1000) {
			uigtk_messagebox( "Text V-RAM write wait value range error (1-1000)",
				"Hardware Speed Settings", UIGTK_BUTTON_OKONLY | UIGTK_ICON_ERROR );
			goto retry;
		}
		mz -> memory_waittvramclock_w = t;
		s = gtk_entry_get_text( entry_gwait2000_r );
		if (sscanf( s, "%d", &t ) != 1 || t < 1 || t > 1000) {
			uigtk_messagebox( "MZ-2000/2200 G-RAM read wait value range error (1-1000)",
				"Hardware Speed Settings", UIGTK_BUTTON_OKONLY | UIGTK_ICON_ERROR );
			goto retry;
		}
		mz -> memory_waitgramclock_r = t;
		s = gtk_entry_get_text( entry_gwait2000_w );
		if (sscanf( s, "%d", &t ) != 1 || t < 1 || t > 1000) {
			uigtk_messagebox( "MZ-2000/2200 G-RAM write wait value range error (1-1000)",
				"Hardware Speed Settings", UIGTK_BUTTON_OKONLY | UIGTK_ICON_ERROR );
			goto retry;
		}
		mz -> memory_waitgramclock_w = t;
		s = gtk_entry_get_text( entry_gwait80b_r );
		if (sscanf( s, "%d", &t ) != 1 || t < 1 || t > 1000) {
			uigtk_messagebox( "MZ-80B/80B2 G-RAM read wait value range error (1-1000)",
				"Hardware Speed Settings", UIGTK_BUTTON_OKONLY | UIGTK_ICON_ERROR );
			goto retry;
		}
		mz -> memory_waitgram80bclock_r = t;
		s = gtk_entry_get_text( entry_gwait80b_w );
		if (sscanf( s, "%d", &t ) != 1 || t < 1 || t > 1000) {
			uigtk_messagebox( "MZ-80B/80B2 G-RAM write wait value range error (1-1000)",
				"Hardware Speed Settings", UIGTK_BUTTON_OKONLY | UIGTK_ICON_ERROR );
			goto retry;
		}
		mz -> memory_waitgram80bclock_w = t;
		s = gtk_entry_get_text( entry_alphaclocks );
		if (sscanf( s, "%d", &t ) != 1 || t < -5000 || t > 5000) {
			uigtk_messagebox( "Frame additional clock value range error (-5000<=0<=5000)",
				"Hardware Speed Settings", UIGTK_BUTTON_OKONLY | UIGTK_ICON_ERROR );
			goto retry;
		}
		mz -> frameclock_alpha = t;
		setspeed_z80( mz -> nowZ80Clock );
		gtk_widget_destroy( GTK_WIDGET(dialog) );
		g_object_unref( builder );
		return UIGTK_IDOK;
	case GTK_RESPONSE_NO :
		gtk_adjustment_set_value( adj_sysclocks, DEFAULT_SPEED_KHZ / 1000 );
		snprintf( buf, LNMAX, "%d", MZMEMORY_WAITTVRAMCLOCK_R );
		gtk_entry_set_text( entry_textwait_r, buf );
		snprintf( buf, LNMAX, "%d", MZMEMORY_WAITTVRAMCLOCK_W );
		gtk_entry_set_text( entry_textwait_w, buf );
		snprintf( buf, LNMAX, "%d", MZMEMORY_WAITGRAMCLOCK_R );
		gtk_entry_set_text( entry_gwait2000_r, buf );
		snprintf( buf, LNMAX, "%d", MZMEMORY_WAITGRAMCLOCK_W );
		gtk_entry_set_text( entry_gwait2000_w, buf );
		snprintf( buf, LNMAX, "%d", MZMEMORY_WAITGRAM80BCLOCK_R );
		gtk_entry_set_text( entry_gwait80b_r, buf );
		snprintf( buf, LNMAX, "%d", MZMEMORY_WAITGRAM80BCLOCK_W );
		gtk_entry_set_text( entry_gwait80b_w, buf );
		snprintf( buf, LNMAX, "%d", MZCLOCK_ADJUSTVALUE );
		gtk_entry_set_text( entry_alphaclocks, buf );
		goto retry;
	case GTK_RESPONSE_CANCEL :
	default : /* close button */
		break;
	}
	gtk_widget_destroy( GTK_WIDGET(dialog) );
	g_object_unref( builder );
	return UIGTK_IDCANCEL;
}

/*
	Local Variables:
	mode:c++
	c-set-style:"k&r"
	c-basic-offset:8
	tab-width:8
	End:
*/
