/* File: uigtk_devicedlg.c

	SHARP MZ-2000/2200/80B/B2 Emulator "emz2000"
		libgtk modules for a device settings dialog

	Copyright (C) 2003-2019 FUKUI, Toshio.
	This file is part of the emz2000 software.
	See copyright notice in the COPYING file.
*/

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <gtk/gtk.h>

#include "common.h"
#include "mz2000.h"
#include "uigtk.h"

static MZ2000 *mz;
static GtkAdjustment *adj_volume1, *adj_volume2;
static GtkEntry *entry_playbuffer, *entry_delaytime;
static GtkEntry *entry_freq, *entry_bit;

int uigtk_devicedlg_domodal( MZ2000 *_mz )
{
	int t;
	char buf[LNMAX];
	gint r;
	const gchar *s;
	GtkBuilder *builder;
	GtkDialog *dialog;

	mz = _mz;
	builder = gtk_builder_new();
	r = gtk_builder_add_from_file( builder, GLADEUIFILE, NULL );
	dialog = GTK_DIALOG( gtk_builder_get_object( builder, "device_dialog" ) );
#ifdef EMZ2000_LIBDIR
	if (/* r || */ !dialog) {
		r = gtk_builder_add_from_file( builder, EMZ2000_LIBDIR "/" GLADEUIFILE, NULL );
		dialog = GTK_DIALOG( gtk_builder_get_object( builder, "device_dialog" ) );
	}
#endif
	if (/* r || */ !dialog) {
		fprintf( stderr, GLADEUIFILE " and " EMZ2000_LIBDIR "/" GLADEUIFILE " not found or broken.\n" );
		g_object_unref( builder );
		return UIGTK_IDCANCEL;
	}
	gtk_builder_connect_signals( builder, NULL );
	//gtk_widget_set_parent_window( GTK_WIDGET(dialog), uigtk_window );

	/* Sound */
	entry_playbuffer = GTK_ENTRY( gtk_builder_get_object( builder, "entry_playbuffer" ) );
	snprintf( buf, LNMAX, "%d", mz -> sound_playbuffer_ms );
	gtk_entry_set_text( entry_playbuffer, buf );
	entry_delaytime = GTK_ENTRY( gtk_builder_get_object( builder, "entry_delaytime" ) );
	snprintf( buf, LNMAX, "%d", mz -> sound_delaytime_ms );
	gtk_entry_set_text( entry_delaytime, buf );
	entry_freq = GTK_ENTRY( gtk_builder_get_object( builder, "entry_freq" ) );
	snprintf( buf, LNMAX, "%d", mz -> sound_freq );
	gtk_entry_set_text( entry_freq, buf );
	entry_bit = GTK_ENTRY( gtk_builder_get_object( builder, "entry_bit" ) );
	snprintf( buf, LNMAX, "%d", mz -> sound_bit );
	gtk_entry_set_text( entry_bit, buf );
	/* Volume */
	adj_volume1 = GTK_ADJUSTMENT( gtk_builder_get_object( builder, "adjustment_volume1" ) );
	gtk_adjustment_set_value( adj_volume1, (gdouble)mz -> sound_volume1 );
	adj_volume2 = GTK_ADJUSTMENT( gtk_builder_get_object( builder, "adjustment_volume2" ) );
	gtk_adjustment_set_value( adj_volume2, (gdouble)mz -> sound_volume2 );

	gtk_dialog_add_buttons( dialog,
		GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
		"DEFAULT", GTK_RESPONSE_NO, NULL );

retry:	r = gtk_dialog_run( GTK_DIALOG(dialog) );
	switch (r) {
	case GTK_RESPONSE_ACCEPT :
		/* Sound */
		s = gtk_entry_get_text( entry_playbuffer );
		if (sscanf( s, "%d", &t ) != 1 || t < 10 || t > 1000) {
			uigtk_messagebox( "Sound Play Buffer (Time) value range error",
				"Device Configuration", UIGTK_BUTTON_OKONLY | UIGTK_ICON_ERROR );
			goto retry;
		}
		mz -> sound_playbuffer_ms = t;
		s = gtk_entry_get_text( entry_delaytime );
		if (sscanf( s, "%d", &t ) != 1 || t < 10 || t > 1000) {
			uigtk_messagebox( "Sound Delay Time value range error",
				"Device Configuration", UIGTK_BUTTON_OKONLY | UIGTK_ICON_ERROR );
			goto retry;
		}
		mz -> sound_delaytime_ms = t;
		s = gtk_entry_get_text( entry_freq );
		if (sscanf( s, "%d", &t ) != 1 || t < 1000 || t > 192000) {
			uigtk_messagebox( "Sound Frequency value range error",
				"Device Configuration", UIGTK_BUTTON_OKONLY | UIGTK_ICON_ERROR );
			goto retry;
		}
		mz -> sound_freq = t;
		s = gtk_entry_get_text( entry_bit );
		if (sscanf( s, "%d", &t ) != 1 || (t != 8 && t != 16)) {
			uigtk_messagebox( "Sound Bit value range error (only 8 or 16)",
				"Device Configuration", UIGTK_BUTTON_OKONLY | UIGTK_ICON_ERROR );
			goto retry;
		}
		mz -> sound_bit = t;
		/* Volume */
		mz -> sound_volume1 = (char)gtk_adjustment_get_value( adj_volume1 );
		mz -> sound_volume2 = (char)gtk_adjustment_get_value( adj_volume2 );
		sound_updateparams( mz );
		gtk_widget_destroy( GTK_WIDGET(dialog) );
		g_object_unref( builder );
		return UIGTK_IDOK;
	case GTK_RESPONSE_NO :
		/* Sound */
		snprintf( buf, LNMAX, "%d", DEFAULT_SOUND_PLAYBUFFER_MS );
		gtk_entry_set_text( entry_playbuffer, buf );
		snprintf( buf, LNMAX, "%d", DEFAULT_SOUND_DELAYTIME_MS );
		gtk_entry_set_text( entry_delaytime, buf );
		snprintf( buf, LNMAX, "%d", DEFAULT_SOUND_FREQ );
		gtk_entry_set_text( entry_freq, buf );
		snprintf( buf, LNMAX, "%d", DEFAULT_SOUND_BIT );
		gtk_entry_set_text( entry_bit, buf );
		/* Volume */
		gtk_adjustment_set_value( adj_volume1, (gdouble)DEFAULT_SOUND_VOLUME1 );
		gtk_adjustment_set_value( adj_volume2, (gdouble)DEFAULT_SOUND_VOLUME2 );
		goto retry;
	case GTK_RESPONSE_CANCEL :
	default : /* close button */
		break;
	}
	gtk_widget_destroy( GTK_WIDGET(dialog) );
	g_object_unref( builder );
	return UIGTK_IDCANCEL;
}

/*
	Local Variables:
	mode:c++
	c-set-style:"k&r"
	c-basic-offset:8
	tab-width:8
	End:
*/
