/* File: uigtk_filedlg.c

	SHARP MZ-2000/2200/80B/B2 Emulator "emz2000"
		libgtk modules for a file settings dialog

	Copyright (C) 2003-2019 FUKUI, Toshio.
	This file is part of the emz2000 software.
	See copyright notice in the COPYING file.
*/

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <gtk/gtk.h>

#include "common.h"
#include "mz2000.h"
#include "uigtk.h"

int uigtk_filedlg_domodal( const char *title, char *fnbuf, int fnbuf_max, int mode, int type )
{
	gint r;
	const gchar *s;
	GtkWidget *fdialog;
	GtkFileFilter *filter;

	switch (mode) {
	case UIGTK_FILEMODE_OPEN :
		fdialog = gtk_file_chooser_dialog_new( title,
			GTK_WINDOW(uigtk_window),
			GTK_FILE_CHOOSER_ACTION_OPEN,
			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			NULL );
		break;
	case UIGTK_FILEMODE_SAVE :
		fdialog = gtk_file_chooser_dialog_new( title,
			GTK_WINDOW(uigtk_window),
			GTK_FILE_CHOOSER_ACTION_SAVE,
			GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			NULL );
		gtk_file_chooser_set_do_overwrite_confirmation( GTK_FILE_CHOOSER(fdialog), TRUE );
		break;
	default :
		ERRMSG( "uigtk_filedlg_domodal(): unknown mode found\n" );
		exit( 1 );
	}

	switch (type) {
	case UIGTK_FILETYPE_MZSTATE :
		filter = gtk_file_filter_new();
		gtk_file_filter_set_name( filter, "MZ State Files" );
		gtk_file_filter_add_pattern( filter, "*.mze" );
		gtk_file_filter_add_pattern( filter, "*.MZE" );
		gtk_file_chooser_add_filter( GTK_FILE_CHOOSER(fdialog), filter );
		break;
	case UIGTK_FILETYPE_MZTAPE :
		filter = gtk_file_filter_new();
		gtk_file_filter_set_name( filter, "Tape Image Files" );
		gtk_file_filter_add_pattern( filter, "*.mzt" );
		gtk_file_filter_add_pattern( filter, "*.MZT" );
		gtk_file_filter_add_pattern( filter, "*.mzf" );
		gtk_file_filter_add_pattern( filter, "*.MZF" );
		gtk_file_filter_add_pattern( filter, "*.mtw" );
		gtk_file_filter_add_pattern( filter, "*.MTW" );
		gtk_file_filter_add_pattern( filter, "*.mti" );
		gtk_file_filter_add_pattern( filter, "*.MTI" );
		gtk_file_filter_add_pattern( filter, "*.dat" );
		gtk_file_filter_add_pattern( filter, "*.DAT" );
		gtk_file_filter_add_pattern( filter, "*.wav" );
		gtk_file_filter_add_pattern( filter, "*.WAV" );
		gtk_file_filter_add_pattern( filter, "*.hex" );
		gtk_file_filter_add_pattern( filter, "*.HEX" );
		gtk_file_chooser_add_filter( GTK_FILE_CHOOSER(fdialog), filter );
		break;
	case UIGTK_FILETYPE_MZFD :
		filter = gtk_file_filter_new();
		gtk_file_filter_set_name( filter, "Disk Image Files" );
		gtk_file_filter_add_pattern( filter, "*.d88" );
		gtk_file_filter_add_pattern( filter, "*.D88" );
		gtk_file_filter_add_pattern( filter, "*.d20" );
		gtk_file_filter_add_pattern( filter, "*.D20" );
		gtk_file_filter_add_pattern( filter, "*.d77" );
		gtk_file_filter_add_pattern( filter, "*.D77" );
		gtk_file_filter_add_pattern( filter, "*.2d" );
		gtk_file_filter_add_pattern( filter, "*.2D" );
		gtk_file_filter_add_pattern( filter, "*.dsk" );
		gtk_file_filter_add_pattern( filter, "*.DSK" );
		gtk_file_chooser_add_filter( GTK_FILE_CHOOSER(fdialog), filter );
		break;
	case UIGTK_FILETYPE_MZQD :
		filter = gtk_file_filter_new();
		gtk_file_filter_set_name( filter, "QD Image Files" );
		gtk_file_filter_add_pattern( filter, "*.q20" );
		gtk_file_filter_add_pattern( filter, "*.Q20" );
		gtk_file_filter_add_pattern( filter, "*.mzt" );
		gtk_file_filter_add_pattern( filter, "*.MZT" );
		gtk_file_chooser_add_filter( GTK_FILE_CHOOSER(fdialog), filter );
		break;
	case UIGTK_FILETYPE_SSHOT :
		filter = gtk_file_filter_new();
		gtk_file_filter_set_name( filter, "Screen Shot Files" );
		gtk_file_filter_add_pattern( filter, "*.bmp" );
		gtk_file_filter_add_pattern( filter, "*.BMP" );
#if defined(HAVE_LIBGIF) || defined(HAVE_LIBUNGIF)
		gtk_file_filter_add_pattern( filter, "*.gif" );
		gtk_file_filter_add_pattern( filter, "*.GIF" );
#endif
		gtk_file_chooser_add_filter( GTK_FILE_CHOOSER(fdialog), filter );
		break;
	default : /* UIGTK_FILETYPE_DEFAULT */
		break;
	}
	filter = gtk_file_filter_new();
	gtk_file_filter_set_name( filter, "All Files" );
	gtk_file_filter_add_pattern( filter, "*" );
	gtk_file_chooser_add_filter( GTK_FILE_CHOOSER(fdialog), filter );

	gtk_file_chooser_set_filename( GTK_FILE_CHOOSER(fdialog), fnbuf );

	gtk_widget_show_all( fdialog );

	r = gtk_dialog_run( GTK_DIALOG(fdialog) );
	switch (r) {
	case GTK_RESPONSE_ACCEPT :
		s = gtk_file_chooser_get_filename( GTK_FILE_CHOOSER(fdialog) );
		if (fnbuf_max < PATH_MAX) {
			strncpy( fnbuf, s, fnbuf_max );
			fnbuf[fnbuf_max - 1] = '\0';
		} else {
			strncpy( fnbuf, s, PATH_MAX );
			fnbuf[PATH_MAX - 1] = '\0';
		}
		/* printf( "Filename result : %s\n", fnbuf ); */
		gtk_widget_destroy( fdialog );
		return UIGTK_IDOK;
	case GTK_RESPONSE_CANCEL :
		gtk_widget_destroy( fdialog );
		return UIGTK_IDCANCEL;
	default :
		_TRACE( "uigtk_filedlg_domdal() received the unknown response\n" );
		break;
	}
	gtk_widget_destroy( fdialog );
	return UIGTK_IDCANCEL;
}

/*
	Local Variables:
	mode:c++
	c-set-style:"k&r"
	c-basic-offset:8
	tab-width:8
	End:
*/
