/* File: uigtk_menu.c

	SHARP MZ-2000/2200/80B/B2 Emulator "emz2000"
		libgtk modules for a menu

	Copyright (C) 2002-2019 FUKUI, Toshio.
	This file is part of the emz2000 software.
	See copyright notice in the COPYING file.
*/

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <gtk/gtk.h>

#include "common.h"
#include "mz2000.h"
#include "uigtk.h"
#include "gsave.h"
#include "plugin.h"

int uigtk_menu_externalchg = FALSE;
static GtkUIManager *uigtk_ui = NULL;

static void uigtk_menu_pause( void )
{
	setplaystop( uigtk_mz, TRUE );
	uigtk_mz -> pause = !uigtk_mz -> pause;
	setplaystop( uigtk_mz, FALSE );
	return;
}

static void uigtk_menu_reset( void )
{
	uigtk_mz -> needReset = TRUE;
	stop_z80();
	setplaystop( uigtk_mz, FALSE );
	return;
}

static void uigtk_menu_iplreset( void )
{
	uigtk_mz -> needIpl = TRUE;
	stop_z80();
	setplaystop( uigtk_mz, FALSE );
	return;
}

static void uigtk_menu_iplreset2000( void )
{
	uigtk_mz -> mzmode = FALSE;
	uigtk_mz -> needIpl = TRUE;
	stop_z80();
	setplaystop( uigtk_mz, FALSE );
	return;
}

static void uigtk_menu_iplreset80b( void )
{
	uigtk_mz -> mzmode = TRUE;
	uigtk_mz -> needIpl = TRUE;
	stop_z80();
	setplaystop( uigtk_mz, FALSE );
	return;
}

static void uigtk_menu_loadstate( void )
{
	int r, flag;
	char pathbuf[PATH_MAX], *p;

	setplaystop( uigtk_mz, TRUE );
	strncpy( pathbuf, uigtk_mz -> sfilename, PATH_MAX );
	pathbuf[PATH_MAX - 1] = '\0';
	while (1) {
		r = uigtk_filedlg_domodal( "Load States Data (.mze)", pathbuf, PATH_MAX,
			UIGTK_FILEMODE_OPEN, UIGTK_FILETYPE_MZSTATE );
		if (r < 0) {
			setplaystop( uigtk_mz, FALSE );
			return;
		}
		flag = FALSE;
		p = strrchr( pathbuf, '.' );
		if (p) {
			if (!strcasecmp( p+1, "mze" ))
				flag = TRUE;
		}
		if (!flag) {
			uigtk_messagebox( "Please select the file extentions \".mze\".",
				"States", UIGTK_BUTTON_OKONLY | UIGTK_ICON_ERROR );
			continue;
		}
		break;
	}
	strncpy( uigtk_mz -> sfilename, pathbuf, PATH_MAX );
	uigtk_mz -> sfilename[PATH_MAX - 1] = '\0';
	uigtk_mz -> sfilename_havewrite = FALSE;
	r = load_state( uigtk_mz );
	if (r < 0)
		;	/* Error */
	setplaystop( uigtk_mz, FALSE );
	return;
}

static void uigtk_menu_savestate_0( int force_newname )
{
	int r, flag;
	char pathbuf[PATH_MAX], *p;

	setplaystop( uigtk_mz, TRUE );
	if (!(uigtk_mz -> sfilename_havewrite) || force_newname) {
		strncpy( pathbuf, uigtk_mz -> sfilename, PATH_MAX );
		pathbuf[PATH_MAX - 1] = '\0';
		while (1) {
			r = uigtk_filedlg_domodal( "Save States Data (.mze)", pathbuf, PATH_MAX,
				UIGTK_FILEMODE_SAVE, UIGTK_FILETYPE_MZSTATE );
			if (r < 0) {
				setplaystop( uigtk_mz, FALSE );
				return;
			}
			flag = FALSE;
			p = strrchr( pathbuf, '.' );
			if (p && p != pathbuf) {
				if (!strcasecmp( p+1, "mze" ))
					flag = TRUE;
			}
			if (!flag) {
				uigtk_messagebox( "Please use the file extentions \".mze\".",
					"States", UIGTK_BUTTON_OKONLY | UIGTK_ICON_ERROR );
				continue;
			}
			break;
		}
		strncpy( uigtk_mz -> sfilename, pathbuf, PATH_MAX );
		uigtk_mz -> sfilename[PATH_MAX - 1] = '\0';
		uigtk_mz -> sfilename_havewrite = TRUE;
	}
	r = save_state( uigtk_mz );
	if (r < 0)
		;	/* errors */
	setplaystop( uigtk_mz, FALSE );
	return;
}

void uigtk_menu_savestate( void )
{
	uigtk_menu_savestate_0( FALSE );
	return;
}

static void uigtk_menu_savestateas( void )
{
	uigtk_menu_savestate_0( TRUE );
	return;
}

static void uigtk_menu_copy( void )
{
	copypaste_copy();
	return;
}

static void uigtk_menu_paste( void )
{
	copypaste_pastestart();
	return;
}

static void uigtk_menu_pastestop( void )
{
	copypaste_pasteend( FALSE );
	return;
}

static void uigtk_menu_pastesettings( void )
{
	setplaystop( uigtk_mz, TRUE );
	uigtk_pastedlg_domodal( uigtk_mz );
	setplaystop( uigtk_mz, FALSE );
	return;
}

static void uigtk_menu_screenshot( void )
{
	int r;
	char pathbuf[PATH_MAX];

	setplaystop( uigtk_mz, TRUE );
	strncpy( pathbuf, "", PATH_MAX );
	pathbuf[PATH_MAX - 1] = '\0';
	r = uigtk_filedlg_domodal( "Select a Screen Shot File", pathbuf, PATH_MAX,
		UIGTK_FILEMODE_SAVE, UIGTK_FILETYPE_SSHOT );
	if (r < 0) {
		setplaystop( uigtk_mz, FALSE );
		return;
	}
	gsave_init();
	r = gsave_writefile( GSAVE_ATTR_NONE, pathbuf,
		MZ2000_DISPLAY_WIDTH, MZ2000_DISPLAY_HEIGHT,
		uigtk_mz -> screen -> scrbufd );
	if (!r) {
		uigtk_messagebox( "A screen shot file output error.",
			"Screen Shot", UIGTK_BUTTON_OKONLY | UIGTK_ICON_ERROR );
	}
	setplaystop( uigtk_mz, FALSE );
	return;
}

void uigtk_menu_quit( void )
{
	int i, qflag = FALSE;
	gint r;
	char buf[LNMAX];
	MZ2000 *mz;

	mz = uigtk_mz;
	setplaystop( mz, TRUE );
	r = mz2tape_getstatus( NULL, NULL, NULL, NULL, NULL, NULL );
	if (r != MZ2TAPE_RUNMODE_STOP && r != MZ2TAPE_RUNMODE_EJECT) {
		snprintf( buf, LNMAX, "Now a cassette tape deck is running. Stop it ?" );
		r = uigtk_messagebox( buf, "Warning", UIGTK_BUTTON_YESNOCANC | UIGTK_ICON_QUESTION );
		if (r == UIGTK_IDCANCEL || r == UIGTK_IDNO) {
			setplaystop( mz, FALSE );
			return;
		}
	}
	mz2tape_stop( mz );
	for (i = 0; i < MZ_DISK_DRIVE_MAX; i++) {
		if (!disk_getmodified( mz, i ))
			continue;
		snprintf( buf, LNMAX, "A drive %d image was modified. Writeback to \"%s\" ?",
			i + 1, disk_getfnptr( mz, i ) );
		r = uigtk_messagebox( buf, "Warning", UIGTK_BUTTON_YESNOCANC | UIGTK_ICON_QUESTION );
		if (r == UIGTK_IDCANCEL) {
			qflag = TRUE;
			break;
		}
		if (r == UIGTK_IDYES)
			disk_writeback( mz, i );
	}
	if (qflag) {
		setplaystop( mz, FALSE );
		return;
	}
	if (plugin_device_getinterface( PLUGIN_DEVICETYPE_QD )) {
		r = plugin_device_getrunningstatus( PLUGIN_DEVICETYPE_QD, 0 );
		if (r) {
			snprintf( buf, LNMAX, "Now a quick disk drive is running. Quit OK ?" );
			r = uigtk_messagebox( buf, "Warning", UIGTK_BUTTON_YESNOCANC | UIGTK_ICON_QUESTION );
			if (r == UIGTK_IDCANCEL || r == UIGTK_IDNO) {
				setplaystop( mz, FALSE );
				return;
			}
		}
	}
	setplaystop( mz, TRUE );
	gtk_main_quit();
	return;
}

static void uigtk_menu_tapechange( void )
{
	int r;
	char pathbuf[PATH_MAX];

	setplaystop( uigtk_mz, TRUE );
	strncpy( pathbuf, uigtk_mz->tfilename, PATH_MAX );
	pathbuf[PATH_MAX - 1] = '\0';
	r = uigtk_filedlg_domodal( "Select a Tape Image File", pathbuf, PATH_MAX,
		UIGTK_FILEMODE_OPEN, UIGTK_FILETYPE_MZTAPE );
	if (r < 0) {
		setplaystop( uigtk_mz, FALSE );
		return;
	}
	r = mz2tape_exec( uigtk_mz, pathbuf, uigtk_mz -> forcereadonly ? TRUE : FALSE );
	if (r <= MZ2TAPE_FILEMODE_ERROR) {
		if (r <= MZ2TAPE_FILEMODE_ERRNDLG) {
			uigtk_messagebox( "File open error.",
				"Tape Change or Exec.", UIGTK_BUTTON_OKONLY | UIGTK_ICON_ERROR );
		}
		setplaystop( uigtk_mz, FALSE );
		return;
	}
	strncpy( uigtk_mz -> tfilename, pathbuf, PATH_MAX );
	uigtk_mz -> tfilename[PATH_MAX - 1] = '\0';
	setplaystop( uigtk_mz, FALSE );
	return;
}

static void uigtk_menu_tapeeject( void )
{
	if (!mz2tape_getkinh())
		mz2tape_eject( uigtk_mz, TRUE );
	return;
}

static void uigtk_menu_tapestop( void )
{
	if (mz2tape_ready() && !mz2tape_getkinh())
		mz2tape_stop( uigtk_mz );
	return;
}

static void uigtk_menu_taperew( void )
{
	if (mz2tape_ready() && !mz2tape_getkinh())
		mz2tape_rew( uigtk_mz );
	return;
}

static void uigtk_menu_tapeff( void )
{
	if (mz2tape_ready() && !mz2tape_getkinh())
		mz2tape_ff( uigtk_mz );
	return;
}

static void uigtk_menu_tapetohead( void )
{
	if (mz2tape_ready() && !mz2tape_getkinh())
		mz2tape_head( uigtk_mz );
	return;
}

static void uigtk_menu_tapesettings( void )
{
	setplaystop( uigtk_mz, TRUE );
	uigtk_tapedlg_domodal( uigtk_mz );
	setplaystop( uigtk_mz, FALSE );
	return;
}

static void uigtk_menu_tapespeaker( void )
{
	if (!uigtk_menu_externalchg)
		uigtk_mz -> tapespeaker = !(uigtk_mz -> tapespeaker);
	return;
}

static void uigtk_menu_settapecounter_mz2080( void )
{
	uigtk_mz -> tapecounter = TAPE_COUNTERMODE_MZ2080;
	return;
}

static void uigtk_menu_settapecounter_mz2500( void )
{
	uigtk_mz -> tapecounter = TAPE_COUNTERMODE_MZ2500;
	return;
}

static void uigtk_menu_settapecounter_kbytes( void )
{
	uigtk_mz -> tapecounter = TAPE_COUNTERMODE_KBYTES;
	return;
}

static void uigtk_menu_settapecounter_bytes( void )
{
	uigtk_mz -> tapecounter = TAPE_COUNTERMODE_BYTES;
	return;
}

static void uigtk_menu_dropautoipltape( void )
{
	if (uigtk_menu_externalchg)
		return;
	uigtk_mz -> dropautoipltape = !(uigtk_mz -> dropautoipltape);
	return;
}

static void uigtk_menu_enablesoundtape( void )
{
	if (uigtk_menu_externalchg)
		return;
	uigtk_mz -> enable_soundtape = !(uigtk_mz -> enable_soundtape);
	sound_reset( uigtk_mz );
	return;
}

static void uigtk_menu_diskdrnchange( int drv )
{
	int r, d88flag;
	char pathbuf[PATH_MAX], *p;
	char lnbuf[LNMAX];

	setplaystop( uigtk_mz, TRUE );
	if (!disk_eject( uigtk_mz, drv, DISK_EJECT_CHECKONLY )) {
		setplaystop( uigtk_mz, FALSE );
		return;
	}
	strncpy( pathbuf, disk_getdefpath( uigtk_mz, drv ), PATH_MAX );
	pathbuf[PATH_MAX - 1] = '\0';
	snprintf( lnbuf, LNMAX, "Select a Disk Image File for Drives %d", drv + 1 );
	lnbuf[LNMAX - 1] = '\0';
	r = uigtk_filedlg_domodal( lnbuf, pathbuf, PATH_MAX,
		UIGTK_FILEMODE_OPEN, UIGTK_FILETYPE_MZFD );
	if (r < 0) {
		setplaystop( uigtk_mz, FALSE );
		return;
	}
	if (!disk_eject( uigtk_mz, drv, DISK_EJECT_FORCE )) {
		setplaystop( uigtk_mz, FALSE );
		return;
	}
	d88flag = FALSE;
	p = strrchr( pathbuf, '.' );
	if (p) {
		if (!strcasecmp( p+1 , "d88" ))
			d88flag = TRUE;
		if (!strcasecmp( p+1 , "d77" ))
			d88flag = TRUE;
		if (!strcasecmp( p+1 , "d20" ))
			d88flag = TRUE;
		if (!strcasecmp( p+1 , "dsk" ))
			d88flag = TRUE;
	}
	disk_change( uigtk_mz, drv, pathbuf, d88flag, FALSE, uigtk_mz -> forcereadonly ? TRUE : FALSE );
	setplaystop( uigtk_mz, FALSE );
	return;
}

static void uigtk_menu_diskdrneject( int drv )
{
	setplaystop( uigtk_mz, TRUE );
	disk_eject( uigtk_mz, drv, DISK_EJECT_NORMAL );
	setplaystop( uigtk_mz, FALSE );
	return;
}

static void uigtk_menu_diskdrnwb( int drv )
{
	setplaystop( uigtk_mz, TRUE );
	disk_writeback( uigtk_mz, drv );
	setplaystop( uigtk_mz, FALSE );
	return;
}

static void uigtk_menu_diskdr1change( void )
{
	uigtk_menu_diskdrnchange( 1-1 );
	return;
}

static void uigtk_menu_diskdr1eject( void )
{
	uigtk_menu_diskdrneject( 1-1 );
	return;
}

static void uigtk_menu_diskdr1wb( void )
{
	uigtk_menu_diskdrnwb( 1-1 );
	return;
}

static void uigtk_menu_diskdr2change( void )
{
	uigtk_menu_diskdrnchange( 2-1 );
	return;
}

static void uigtk_menu_diskdr2eject( void )
{
	uigtk_menu_diskdrneject( 2-1 );
	return;
}

static void uigtk_menu_diskdr2wb( void )
{
	uigtk_menu_diskdrnwb( 2-1 );
	return;
}

static void uigtk_menu_diskdr3change( void )
{
	uigtk_menu_diskdrnchange( 3-1 );
	return;
}

static void uigtk_menu_diskdr3eject( void )
{
	uigtk_menu_diskdrneject( 3-1 );
	return;
}

static void uigtk_menu_diskdr3wb( void )
{
	uigtk_menu_diskdrnwb( 3-1 );
	return;
}

static void uigtk_menu_diskdr4change( void )
{
	uigtk_menu_diskdrnchange( 4-1 );
	return;
}

static void uigtk_menu_diskdr4eject( void )
{
	uigtk_menu_diskdrneject( 4-1 );
	return;
}

static void uigtk_menu_diskdr4wb( void )
{
	uigtk_menu_diskdrnwb( 4-1 );
	return;
}

static void uigtk_menu_qdchange( void )
{
	int r, ftype, roflag;
	char pathbuf[PATH_MAX], *p;

	setplaystop( uigtk_mz, TRUE );
	if (!plugin_device_eject( PLUGIN_DEVICETYPE_QD, 0, PLUGIN_DEVICEEJECT_CHECKONLY )) {
		setplaystop( uigtk_mz, FALSE );
		return;
	}
	strncpy( pathbuf, (char *)uigtk_mz->qfilename, PATH_MAX );
	pathbuf[PATH_MAX - 1] = '\0';
	r = uigtk_filedlg_domodal( "Select a Image File for QD", pathbuf, PATH_MAX,
		UIGTK_FILEMODE_OPEN, UIGTK_FILETYPE_MZQD );
	if (r < 0) {
		setplaystop( uigtk_mz, FALSE );
		return;
	}
	if (!plugin_device_eject( PLUGIN_DEVICETYPE_QD, 0, PLUGIN_DEVICEEJECT_FORCE )) {
		setplaystop( uigtk_mz, FALSE );
		return;
	}
	ftype = -1;
	p = strrchr( pathbuf, '.' );
	if (p) {
		if (!strcasecmp( p+1 , "q20" ))
			ftype = 0;
		else if (!strcasecmp( p+1 , "mzt" ))
			ftype = 1;
	}
	if (ftype < 0) {
		uigtk_messagebox( "A unknown type extentions filename was selected.",
			"QD Changes", UIGTK_BUTTON_OKONLY | UIGTK_ICON_ERROR );
		setplaystop( uigtk_mz, FALSE );
		return;
	}
	strncpy( (char *)uigtk_mz -> qfilename, pathbuf, PATH_MAX );
	uigtk_mz -> qfilename[PATH_MAX - 1] = '\0';
	roflag = uigtk_mz -> forcereadonly ? TRUE : FALSE;
	plugin_device_change( PLUGIN_DEVICETYPE_QD, 0, pathbuf, ftype, FALSE, roflag );
	setplaystop( uigtk_mz, FALSE );
	return;
}

static void uigtk_menu_qdeject( void )
{
	setplaystop( uigtk_mz, TRUE );
	plugin_device_eject( PLUGIN_DEVICETYPE_QD, 0, DISK_EJECT_NORMAL );
	setplaystop( uigtk_mz, FALSE );
	return;
}

static void uigtk_menu_enablefddif( void )
{
	if (!uigtk_menu_externalchg)
		uigtk_mz -> enable_fddif = !(uigtk_mz -> enable_fddif);
	return;
}

static void uigtk_menu_enablefastrunning( void )
{
	if (!uigtk_menu_externalchg) {
		uigtk_mz -> fd_exec_fast = !uigtk_mz -> fd_exec_fast;
		disk_setparam( uigtk_mz -> fd_exec_fast, -1, -1 );
	}
	return;
}

static void uigtk_menu_enableignorereadcrcchk( void )
{
	if (!uigtk_menu_externalchg) {
		uigtk_mz -> fd_ignore_crc = !uigtk_mz -> fd_ignore_crc;
		disk_setparam( -1, uigtk_mz -> fd_ignore_crc, -1 );
	}
	return;
}

static void uigtk_menu_enableerase17sctcrc( void )
{
	if (!uigtk_menu_externalchg) {
		uigtk_mz -> fd_erase_sct17crc = !uigtk_mz -> fd_erase_sct17crc;
		disk_setparam( -1, -1, uigtk_mz -> fd_erase_sct17crc );
	}
	return;
}

static void uigtk_menu_droptofd2( void )
{
	if (uigtk_menu_externalchg)
		return;
	uigtk_mz -> droptofd2 = !(uigtk_mz -> droptofd2);
	return;
}

static void uigtk_menu_dropautoiplfd( void )
{
	if (uigtk_menu_externalchg)
		return;
	uigtk_mz -> dropautoipl = !(uigtk_mz -> dropautoipl);
	return;
}

static void uigtk_menu_enablesoundfd( void )
{
	if (uigtk_menu_externalchg)
		return;
	uigtk_mz -> enable_soundfdd = !(uigtk_mz -> enable_soundfdd);
	sound_reset( uigtk_mz );
	return;
}

static void uigtk_menu_pluginsettings( void )
{
	setplaystop( uigtk_mz, TRUE );
	uigtk_plugindlg_domodal( uigtk_mz );
	setplaystop( uigtk_mz, FALSE );
	return;
}

static void uigtk_menu_setkeyscursor( int mode )
{
	if (uigtk_menu_externalchg)
		return;
	switch (mode) {
	case 1 :
		if (uigtk_mz -> keyboard_mode & 0x01)
			uigtk_mz -> keyboard_mode &= ~0x03U;
		else {
			uigtk_mz -> keyboard_mode &= ~0x03U;
			uigtk_mz -> keyboard_mode |= 0x01U;
		}
		break;
	case 2 :
		if (uigtk_mz -> keyboard_mode & 0x02)
			uigtk_mz -> keyboard_mode &= ~0x03U;
		else {
			uigtk_mz -> keyboard_mode &= ~0x03U;
			uigtk_mz -> keyboard_mode |= 0x02U;
		}
		break;
	default :
		uigtk_mz -> keyboard_mode &= ~0x03U;
		break;
	}
	keyboard_setkeymode( uigtk_mz -> keyboard_mode );
	return;
}

static void uigtk_menu_setkeyscursor_1( void )
{
	return uigtk_menu_setkeyscursor( 1 );
}

static void uigtk_menu_setkeyscursor_2( void )
{
	return uigtk_menu_setkeyscursor( 2 );
}

static void uigtk_menu_setkeysten( void )
{
	if (uigtk_menu_externalchg)
		return;
	if (!(uigtk_mz -> keyboard_mode & 0x04))
		uigtk_mz -> keyboard_mode |= 0x04U;
	else
		uigtk_mz -> keyboard_mode &= ~0x04U;
	keyboard_setkeymode( uigtk_mz -> keyboard_mode );
	return;
}

static void uigtk_menu_setkeyspctab( void )
{
	if (uigtk_menu_externalchg)
		return;
	if (!(uigtk_mz -> keyboard_mode & 0x08)) {
		uigtk_mz -> keyboard_mode &= ~0x10U;
		uigtk_mz -> keyboard_mode |= 0x08U;
	} else
		uigtk_mz -> keyboard_mode &= ~0x08U;
	keyboard_setkeymode( uigtk_mz -> keyboard_mode );
	return;
}

static void uigtk_menu_setkeygrphtab( void )
{
	if (uigtk_menu_externalchg)
		return;
	if (!(uigtk_mz -> keyboard_mode & 0x10)) {
		uigtk_mz -> keyboard_mode &= ~0x08U;
		uigtk_mz -> keyboard_mode |= 0x10U;
	} else
		uigtk_mz -> keyboard_mode &= ~0x10U;
	keyboard_setkeymode( uigtk_mz -> keyboard_mode );
	return;
}

static void uigtk_menu_setkey123to000( void )
{
	if (uigtk_menu_externalchg)
		return;
	if (!(uigtk_mz -> keyboard_mode & 0x20))
		uigtk_mz -> keyboard_mode |= 0x20U;
	else
		uigtk_mz -> keyboard_mode &= ~0x20U;
	keyboard_setkeymode( uigtk_mz -> keyboard_mode );
	return;
}

static void uigtk_menu_green( void )
{
	if (uigtk_menu_externalchg)
		return;
	uigtk_mz -> displayMode = FALSE;
	screen_setmonitor( uigtk_mz, FALSE );
	return;
}

static void uigtk_menu_color( void )
{
	if (uigtk_menu_externalchg)
		return;
	uigtk_mz -> displayMode = TRUE;
	screen_setmonitor( uigtk_mz, TRUE );
	return;
}

static void uigtk_menu_sethighreso_enable( void )
{
	if (uigtk_menu_externalchg)
		return;
	screen_sethighreso( uigtk_mz, TRUE );
	return;
}

static void uigtk_menu_sethighreso_disable( void )
{
	if (uigtk_menu_externalchg)
		return;
	screen_sethighreso( uigtk_mz, FALSE );
	return;
}

static void uigtk_menu_setfontlang_english( void )
{
	if (uigtk_menu_externalchg)
		return;
	screen_setfontlang( uigtk_mz, TRUE );
	return;
}

static void uigtk_menu_setfontlang_japanese( void )
{
	if (uigtk_menu_externalchg)
		return;
	screen_setfontlang( uigtk_mz, FALSE );
	return;
}

static void uigtk_menu_setspeed_n( int speed )
{
	if (uigtk_menu_externalchg)
		return;
	uigtk_mz -> nowZ80Clock = speed;
	setspeed_z80( uigtk_mz -> nowZ80Clock );
	return;
}

static void uigtk_menu_setspeed_4MHz( void )
{
	uigtk_menu_setspeed_n( 4000 );
	return;
}

static void uigtk_menu_setspeed_6MHz( void )
{
	uigtk_menu_setspeed_n( 6000 );
	return;
}

static void uigtk_menu_setspeed_8MHz( void )
{
	uigtk_menu_setspeed_n( 8000 );
	return;
}

static void uigtk_menu_setspeed_400MHz( void )
{
	uigtk_menu_setspeed_n( 400000 );
	return;
}

static void uigtk_menu_speedsettings( void )
{
	setplaystop( uigtk_mz, TRUE );
	uigtk_clockdlg_domodal( uigtk_mz );
	setplaystop( uigtk_mz, FALSE );
	return;
}

static void uigtk_menu_enablesound( void )
{
	if (uigtk_menu_externalchg)
		return;
	uigtk_mz -> soundmode = !(uigtk_mz -> soundmode);
	sound_reset( uigtk_mz );
	return;
}

static void uigtk_menu_consrblankcol( void )
{
	if (uigtk_menu_externalchg)
		return;
	uigtk_mz -> consoler_cblank = !(uigtk_mz -> consoler_cblank);
	screen_setreverse( uigtk_mz, -1 );
	return;
}

static void uigtk_menu_vblankspecial( void )
{
	if (uigtk_menu_externalchg)
		return;
	uigtk_mz -> vblankspecial = !(uigtk_mz -> vblankspecial);
	return;
}

static void uigtk_menu_forcereadonly( void )
{
	if (uigtk_menu_externalchg)
		return;
	uigtk_mz -> forcereadonly = !(uigtk_mz -> forcereadonly);
	return;
}

static void uigtk_menu_enableinitreset( void )
{
	if (uigtk_menu_externalchg)
		return;
	uigtk_mz -> enable_initreset = !(uigtk_mz -> enable_initreset);
	return;
}

static void uigtk_menu_iplmemclr( void )
{
	if (uigtk_menu_externalchg)
		return;
	uigtk_mz -> iplmemclr = !(uigtk_mz -> iplmemclr);
	return;
}

static void uigtk_menu_iplmemzclr( void )
{
	if (uigtk_menu_externalchg)
		return;
	uigtk_mz -> iplmemzclr = !(uigtk_mz -> iplmemzclr);
	return;
}

static void uigtk_menu_deviceconfigs( void )
{
	setplaystop( uigtk_mz, TRUE );
	uigtk_devicedlg_domodal( uigtk_mz );
	setplaystop( uigtk_mz, FALSE );
	return;
}

static void uigtk_menu_sounditpolations( void )
{
	int r;

	if (uigtk_menu_externalchg)
		return;
	r = getspeed_z80();
	uigtk_mz -> enable_soundtminterpolations = !(uigtk_mz -> enable_soundtminterpolations);
	if (r > 0)
		setspeed_z80( r );
	sound_reset( uigtk_mz );
	return;
}

static void uigtk_menu_setwinsize( int mode )
{
	GtkWidget *screen;

	if (uigtk_menu_externalchg)
		return;
	screen = uigtk_mz -> screen -> screen_widget;
	switch (mode) {
	case 10 :
		//if (!gtk_window_get_resizable(GTK_WINDOW(uigtk_window)))
		//	break;
		if (uigtk_mz -> screen_window & GDK_WINDOW_STATE_ICONIFIED)
			break;
		if (uigtk_mz -> screen_window & GDK_WINDOW_STATE_MAXIMIZED)
			break;
		if (uigtk_mz -> screen_window & GDK_WINDOW_STATE_FULLSCREEN)
			break;
		gtk_window_resize( GTK_WINDOW(uigtk_window),
			MZ2000_DISPLAY_WIDTH  + uigtk_winadd_x,
			MZ2000_DISPLAY_HEIGHT + uigtk_winadd_y );
		break;
	case 15 :
		//if (!gtk_window_get_resizable(GTK_WINDOW(uigtk_window)))
		//	break;
		if (uigtk_mz -> screen_window & GDK_WINDOW_STATE_ICONIFIED)
			break;
		if (uigtk_mz -> screen_window & GDK_WINDOW_STATE_MAXIMIZED)
			break;
		if (uigtk_mz -> screen_window & GDK_WINDOW_STATE_FULLSCREEN)
			break;
		gtk_window_resize( GTK_WINDOW(uigtk_window),
			MZ2000_DISPLAY_WIDTH  * 3/2 + uigtk_winadd_x,
			MZ2000_DISPLAY_HEIGHT * 3/2 + uigtk_winadd_y );
		break;
	case 20 :
		//if (!gtk_window_get_resizable(GTK_WINDOW(uigtk_window)))
		//	break;
		if (uigtk_mz -> screen_window & GDK_WINDOW_STATE_ICONIFIED)
			break;
		if (uigtk_mz -> screen_window & GDK_WINDOW_STATE_MAXIMIZED)
			break;
		if (uigtk_mz -> screen_window & GDK_WINDOW_STATE_FULLSCREEN)
			break;
		gtk_window_resize( GTK_WINDOW(uigtk_window),
			MZ2000_DISPLAY_WIDTH  * 2 + uigtk_winadd_x,
			MZ2000_DISPLAY_HEIGHT * 2 + uigtk_winadd_y );
		break;
	case 99 :	/* Full Screen */
		if (uigtk_mz -> screen_window & GDK_WINDOW_STATE_ICONIFIED)
			break;
		if (uigtk_mz -> screen_window & GDK_WINDOW_STATE_MAXIMIZED)
			break;
		if (uigtk_mz -> screen_window & GDK_WINDOW_STATE_FULLSCREEN)
			gtk_window_unfullscreen( GTK_WINDOW(uigtk_window) );
		else {
			if (!gtk_window_get_resizable(GTK_WINDOW(uigtk_window)))
				break;
			gtk_window_fullscreen( GTK_WINDOW(uigtk_window) );
		}
		break;
	default :
		//gtk_window_unfullscreen( GTK_WINDOW(uigtk_window) );
		//gtk_window_unmaximize( GTK_WINDOW(uigtk_window) );
		break;
	}
	gtk_widget_show_all( uigtk_window );
	return;
}

static void uigtk_menu_setwinsize_x1( void )
{
	uigtk_menu_setwinsize( 10 );
	return;
}

static void uigtk_menu_setwinsize_x1_5( void )
{
	uigtk_menu_setwinsize( 15 );
	return;
}

static void uigtk_menu_setwinsize_x2( void )
{
	uigtk_menu_setwinsize( 20 );
	return;
}

static void uigtk_menu_setwinsize_fullscreen( void )
{
	uigtk_menu_setwinsize( 99 );
	return;
}

#ifdef TRACE
static void uigtk_menu_trace( GtkAction *action, GtkRadioAction *p )
{
	const gchar *s = gtk_action_get_name( GTK_ACTION(p) );

	if (!strcmp( s, "Traces On" ))
		trace_on();
	if (!strcmp( s, "Traces Off" ))
		trace_off();
	return;
}
#endif

static void uigtk_menu_about( void )
{
	static const gchar *authors[] = {
		"FUKUI, Toshio.",
		"KASUKAWA, Takeya. (emz2000 ver.0.7)",
		"YAMAMOTO, Michiaki. (EmuZ-2000 ver.0.400RC)",
		"TSUNEOKA, Shinji. (Plugins Supports, INI, etc.)",
		NULL
	};

	//setplaystop( uigtk_mz, TRUE );
	gtk_show_about_dialog( GTK_WINDOW(uigtk_window),
		"program-name",	"emz2000 TF-EDITION",
		"version",	"ver." EMZ2000_VERSION,
		"authors",	authors,
		"comments",	"SHARP MZ-2000/2200/80B/80B2 Emulator",
		"copyright",
			"Copyright (C) " EMZ2000_COPYRIGHT " FUKUI, Toshio.\n"
			"Copyright (C) 1999-2000 KASUKAWA, Takeya.\n"
			"Copyright (C) 2002 YAMAMOTO, Michiaki.\n"
			"Copyright (C) 2010 TSUNEOKA, Shinji.\n",
		"website",	"http://fukui.s17.xrea.com/comp/progs/mz20emu.html",
		"website-label","EmuZ-2000 / emz2000 TF-Edition",
		"license",	"This is a free software.\nNO LIMITs, NO WARRENTALies.",
		"wrap-license",	FALSE,
		NULL );
	//setplaystop( uigtk_mz, FALSE );
	return;
}

#if 0
static void uigtk_menu_nosupport( void )
{
	setplaystop( uigtk_mz, TRUE );
	uigtk_messagebox(
		"Sorry. Not implemented yet ...",
		"Information", UIGTK_BUTTON_OKONLY | UIGTK_ICON_INFO );
	setplaystop( uigtk_mz, FALSE );
	return;
}
#endif

static void uigtk_menu_setcheck( const char *path, int state )
{
	GtkWidget *w;
	char buf[LNMAX];

	state = state ? TRUE : FALSE;
	strcpy( buf, "/root_menu_bar" );
	strncat( buf, path, LNMAX - strlen(buf) - 1 );
	w = gtk_ui_manager_get_widget( uigtk_ui, buf );
	if (!w) {
		fprintf( stderr, "menu path:\"%s\" is not found.\n", buf);
		return;
	}
	if (gtk_check_menu_item_get_active( GTK_CHECK_MENU_ITEM(w) ) != state) {
		uigtk_menu_externalchg = TRUE;
		gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(w), state );
		uigtk_menu_externalchg = FALSE;
	}
	return;
}

static void uigtk_menu_setenable( const char *path, int enable )
{
	GtkWidget *w;
	char buf[LNMAX];

	enable = enable ? TRUE : FALSE;
	strcpy( buf, "/root_menu_bar" );
	strncat( buf, path, LNMAX - strlen(buf) - 1 );
	w = gtk_ui_manager_get_widget( uigtk_ui, buf );
	if (!w) {
		fprintf( stderr, "menu path:\"%s\" is not found.\n", buf);
		return;
	}
	gtk_widget_set_sensitive( GTK_WIDGET(w), enable );
	return;
}

int uigtk_menu_updateall( void )
{
	int i, flag;
	MZ2000 *mz = uigtk_mz;

	if (!mz)
		return -1;

	/* File */
	uigtk_menu_setcheck( "/File/Pause", mz -> pause );
	uigtk_menu_setcheck( "/File/MZ-2000,2200", !(mz -> mzmode) );
	uigtk_menu_setcheck( "/File/MZ-80B,80B2",  mz -> mzmode );
	uigtk_menu_setenable( "/File/States Save", !(mz -> howipl > 0) );
	uigtk_menu_setenable( "/File/States Save As", !(mz -> howipl > 0) );
	uigtk_menu_setenable( "/File/Paste",
		!copypaste_havepaste() || copypaste_runningpaste() ? FALSE : TRUE );
	uigtk_menu_setenable( "/File/Paste Stop",
		!copypaste_runningpaste() ? FALSE : TRUE );

	/* Cassette Tape Deck */
	uigtk_menu_setenable( "/CasTape/Eject Cas",
		!mz2tape_getkinh() );
	uigtk_menu_setenable( "/CasTape/Stop",
		mz2tape_ready() && !mz2tape_getkinh() );
	uigtk_menu_setenable( "/CasTape/Rew",
		mz2tape_ready() && !mz2tape_getkinh() );
	uigtk_menu_setenable( "/CasTape/FF",
		mz2tape_ready() && !mz2tape_getkinh() );
	uigtk_menu_setenable( "/CasTape/Rew to Head",
		mz2tape_ready() && !mz2tape_getkinh() );
	uigtk_menu_setcheck( "/CasTape/En. SpeakerSlow", mz -> tapespeaker );
	uigtk_menu_setcheck( "/CasTape/Counter Types/MZ-2000,2200,80B",
		mz -> tapecounter == TAPE_COUNTERMODE_MZ2080 );
	uigtk_menu_setcheck( "/CasTape/Counter Types/MZ-2500",
		mz -> tapecounter == TAPE_COUNTERMODE_MZ2500 );
	uigtk_menu_setcheck( "/CasTape/Counter Types/Kbytes",
		mz -> tapecounter == TAPE_COUNTERMODE_KBYTES );
	uigtk_menu_setcheck( "/CasTape/Counter Types/bytes",
		mz -> tapecounter == TAPE_COUNTERMODE_BYTES );
	uigtk_menu_setcheck( "/CasTape/Drop Auto IPL Cas", mz -> dropautoipltape );
	uigtk_menu_setcheck( "/CasTape/Enable Sound Cas", mz -> enable_soundtape );

	/* FD */
	{
		const char *update_menu_path[] = {
			"/FD/Drive 1/Writeback1",
			"/FD/Drive 2/Writeback2",
			"/FD/Drive 3/Writeback3",
			"/FD/Drive 4/Writeback4"
		};
		const char *eject_menu_path[] = {
			"/FD/Drive 1/Eject1",
			"/FD/Drive 2/Eject2",
			"/FD/Drive 3/Eject3",
			"/FD/Drive 4/Eject4"
		};
		for (i = 0; i < MZ_DISK_DRIVE_MAX; i++) {
			uigtk_menu_setenable( update_menu_path[i],
				disk_getmodified( mz, i ) );
			uigtk_menu_setenable( eject_menu_path[i],
				disk_getexist( mz, i ) );
		}
	}
	uigtk_menu_setcheck( "/FD/Enable IF", mz -> enable_fddif );
	uigtk_menu_setcheck( "/FD/Fast Running mode", mz -> fd_exec_fast );
	uigtk_menu_setcheck( "/FD/Ignore Read CRC checks", mz -> fd_ignore_crc );
	uigtk_menu_setcheck( "/FD/In format, erase 17sct CRC", mz -> fd_erase_sct17crc );

	uigtk_menu_setcheck( "/FD/Drop to FD2 mode", mz -> droptofd2 );
	uigtk_menu_setcheck( "/FD/Drop Auto IPL FD", mz -> dropautoipl );
	uigtk_menu_setcheck( "/FD/Enable Sound FD", mz -> enable_soundfdd );

	/* QD */
	if (!plugin_device_getinterface( PLUGIN_DEVICETYPE_QD )) {
		uigtk_menu_setenable( "/QD/Change QD", FALSE );
		uigtk_menu_setenable( "/QD/Eject QD", FALSE );
	} else {
		uigtk_menu_setenable( "/QD/Change QD", TRUE );
		uigtk_menu_setenable( "/QD/Eject QD",
			plugin_device_getexist( PLUGIN_DEVICETYPE_QD, 0 ) );
	}

	/* Tools */
	uigtk_menu_setcheck( "/Tools/Keydef Selects/Cursor Keys=2468",	  mz -> keyboard_mode & 0x01U );
	uigtk_menu_setcheck( "/Tools/Keydef Selects/Cursor Keys=[00]135", mz -> keyboard_mode & 0x02U );
	uigtk_menu_setcheck( "/Tools/Keydef Selects/Ten Keys 0=[00]",     mz -> keyboard_mode & 0x04U );
	uigtk_menu_setcheck( "/Tools/Keydef Selects/ZX <-> TABSPACE",     mz -> keyboard_mode & 0x08U );
	uigtk_menu_setcheck( "/Tools/Keydef Selects/GRPH <-> TAB",        mz -> keyboard_mode & 0x10U );
	uigtk_menu_setcheck( "/Tools/Keydef Selects/Ten Keys 1 2 3 = 0 [00] .", mz -> keyboard_mode & 0x20U );
	uigtk_menu_setcheck( "/Tools/Display Colors/Green",	!(mz -> displayMode) );
	uigtk_menu_setcheck( "/Tools/Display Colors/Color",	mz -> displayMode );
	uigtk_menu_setcheck( "/Tools/Display Resolutions/15KHz", !(mz -> screen_highreso) );
	uigtk_menu_setcheck( "/Tools/Display Resolutions/24KHz", mz -> screen_highreso );
	uigtk_menu_setcheck( "/Tools/Select Fonts/Japanese",	!(mz -> font_english) );
	uigtk_menu_setcheck( "/Tools/Select Fonts/English",	mz -> font_english );
	uigtk_menu_setcheck( "/Tools/System Clock/4MHz",	mz -> nowZ80Clock == 4000 );
	uigtk_menu_setcheck( "/Tools/System Clock/6MHz",	mz -> nowZ80Clock == 6000 );
	uigtk_menu_setcheck( "/Tools/System Clock/8MHz",	mz -> nowZ80Clock == 8000 );
	uigtk_menu_setcheck( "/Tools/System Clock/400MHz",	mz -> nowZ80Clock == 400000 );
	uigtk_menu_setcheck( "/Tools/Cons-R Blank Col.",mz -> consoler_cblank );
	uigtk_menu_setcheck( "/Tools/V-blank Special mode",	mz -> vblankspecial );
	uigtk_menu_setcheck( "/Tools/Force Read Only mode",	mz -> forcereadonly );
	uigtk_menu_setcheck( "/Tools/Auto Init. IO on Reset",	mz -> enable_initreset );
	uigtk_menu_setcheck( "/Tools/IPL Memory Initialize",	mz -> iplmemclr );
	uigtk_menu_setcheck( "/Tools/Memory Init. is Zero Clear", mz -> iplmemzclr );
	uigtk_menu_setcheck( "/Tools/Enable Sounds mode",	mz -> soundmode );
	uigtk_menu_setcheck( "/Tools/Sounds Time Interpolations", mz -> enable_soundtminterpolations );

	flag = TRUE;
	if (uigtk_mz -> screen_window & GDK_WINDOW_STATE_ICONIFIED)
		flag = FALSE;
	if (uigtk_mz -> screen_window & GDK_WINDOW_STATE_MAXIMIZED)
		flag = FALSE;
	if (uigtk_mz -> screen_window & GDK_WINDOW_STATE_FULLSCREEN)
		flag = FALSE;
	uigtk_menu_setenable( "/View/Window Size x 1", flag );
	uigtk_menu_setcheck( "/View/Window Size x 1",
		uigtk_winx == MZ2000_DISPLAY_WIDTH && uigtk_winy == MZ2000_DISPLAY_HEIGHT );
	uigtk_menu_setenable( "/View/Window Size x 1.5", flag );
	uigtk_menu_setcheck( "/View/Window Size x 1.5",
		uigtk_winx == MZ2000_DISPLAY_WIDTH *2/3 && uigtk_winy == MZ2000_DISPLAY_HEIGHT *2/3 );
	uigtk_menu_setenable( "/View/Window Size x 2", flag );
	uigtk_menu_setcheck( "/View/Window Size x 2",
		uigtk_winx == MZ2000_DISPLAY_WIDTH *2 && uigtk_winy == MZ2000_DISPLAY_HEIGHT *2 );

	flag = TRUE;
	if (uigtk_mz -> screen_window & GDK_WINDOW_STATE_ICONIFIED)
		flag = FALSE;
	if (uigtk_mz -> screen_window & GDK_WINDOW_STATE_MAXIMIZED)
		flag = FALSE;
	uigtk_menu_setenable( "/View/Full Screen mode", flag );
	uigtk_menu_setcheck( "/View/Full Screen mode", mz -> screen_window & GDK_WINDOW_STATE_FULLSCREEN );
	return 0;
}

static const gchar *menu_items = "<ui><menubar name='root_menu_bar'>"

"<menu name='File' action='File'>"
"	<menuitem name='Pause' action='Pause'/>"
"	<separator/>"
"	<menuitem name='Reset' action='Reset'/>"
"	<menuitem name='IPL Reset' action='IPL Reset'/>"
"	<menuitem name='MZ-2000,2200' action='MZ-2000,2200'/>"
"	<menuitem name='MZ-80B,80B2' action='MZ-80B,80B2'/>"
"	<separator/>"
"	<menuitem name='States Load' action='States Load'/>"
"	<menuitem name='States Save' action='States Save'/>"
"	<menuitem name='States Save As' action='States Save As'/>"
"	<separator/>"
"	<menuitem name='Copy to Clipboard' action='Copy to Clipboard'/>"
"	<menuitem name='Paste' action='Paste'/>"
"	<menuitem name='Paste Stop' action='Paste Stop'/>"
"	<menuitem name='Paste Settings' action='Paste Settings'/>"
"	<separator/>"
"	<menuitem name='Screen Shots' action='Screen Shots'/>"
"	<separator/>"
"	<menuitem name='Exit' action='Exit'/>"
"</menu>"

"<menu name='CasTape' action='CasTape'>"
"	<menuitem name='Change or Exec' action='Change or Exec'/>"
"	<menuitem name='Eject Cas' action='Eject Cas'/>"
"	<menuitem name='Stop' action='Stop'/>"
"	<menuitem name='Rew' action='Rew'/>"
"	<menuitem name='FF' action='FF'/>"
"	<menuitem name='Rew to Head' action='Rew to Head'/>"
"	<separator/>"
"	<menuitem name='Settings' action='Settings'/>"
"	<menuitem name='En. SpeakerSlow' action='En. SpeakerSlow'/>"
"	<menu name='Counter Types' action='Counter Types'>"
"		<menuitem name='MZ-2000,2200,80B' action='MZ-2000,2200,80B'/>"
"		<menuitem name='MZ-2500' action='MZ-2500'/>"
"		<menuitem name='Kbytes' action='Kbytes'/>"
"		<menuitem name='bytes' action='bytes'/>"
"	</menu>"
"	<separator/>"
"	<menuitem name='Drop Auto IPL Cas' action='Drop Auto IPL Cas'/>"
"	<menuitem name='Enable Sound Cas' action='Enable Sound Cas'/>"
"</menu>"

"<menu name='FD' action='FD'>"
"	<menu name='Drive 1' action='Drive 1'>"
"		<menuitem name='Change1' action='Change1'/>"
"		<menuitem name='Eject1' action='Eject1'/>"
"		<menuitem name='Writeback1' action='Writeback1'/>"
"	</menu>"
"	<menu name='Drive 2' action='Drive 2'>"
"		<menuitem name='Change2' action='Change2'/>"
"		<menuitem name='Eject2' action='Eject2'/>"
"		<menuitem name='Writeback2' action='Writeback2'/>"
"	</menu>"
"	<menu name='Drive 3' action='Drive 3'>"
"		<menuitem name='Change3' action='Change3'/>"
"		<menuitem name='Eject3' action='Eject3'/>"
"		<menuitem name='Writeback3' action='Writeback3'/>"
"	</menu>"
"	<menu name='Drive 4' action='Drive 4'>"
"		<menuitem name='Change4' action='Change4'/>"
"		<menuitem name='Eject4' action='Eject4'/>"
"		<menuitem name='Writeback4' action='Writeback4'/>"
"	</menu>"
"	<separator/>"
"	<menuitem name='Enable IF' action='Enable IF'/>"
"	<menuitem name='Fast Running mode' action='Fast Running mode'/>"
"	<menuitem name='Ignore Read CRC checks' action='Ignore Read CRC checks'/>"
"	<menuitem name='In format, erase 17sct CRC' action='In format, erase 17sct CRC'/>"
"	<separator/>"
"	<menuitem name='Drop to FD2 mode' action='Drop to FD2 mode'/>"
"	<menuitem name='Drop Auto IPL FD' action='Drop Auto IPL FD'/>"
"	<menuitem name='Enable Sound FD' action='Enable Sound FD'/>"
"</menu>"

"<menu name='QD' action='QD'>"
"	<menuitem name='Change QD' action='Change QD'/>"
"	<menuitem name='Eject QD' action='Eject QD'/>"
"</menu>"

"<menu name='Tools' action='Tools'>"
"	<menuitem name='Plugins Support Settings' action='Plugins Support Settings'/>"
"	<separator/>"
"	<menu name='Keydef Selects' action='Keydef Selects'>"
"		<menuitem name='Cursor Keys=2468' action='Cursor Keys=2468'/>"
"		<menuitem name='Cursor Keys=[00]135' action='Cursor Keys=[00]135'/>"
"		<menuitem name='Ten Keys 0=[00]' action='Ten Keys 0=[00]'/>"
"		<menuitem name='ZX <-> TABSPACE' action='ZX <-> TABSPACE'/>"
"		<menuitem name='GRPH <-> TAB' action='GRPH <-> TAB'/>"
"		<menuitem name='Ten Keys 1 2 3 = 0 [00] .' action='Ten Keys 1 2 3 = 0 [00] .'/>"
"	</menu>"
"	<menu name='Display Colors' action='Display Colors'>"
"		<menuitem name='Green' action='Green'/>"
"		<menuitem name='Color' action='Color'/>"
"	</menu>"
"	<menu name='Display Resolutions' action='Display Resolutions'>"
"		<menuitem name='15KHz' action='15KHz'/>"
"		<menuitem name='24KHz' action='24KHz'/>"
"	</menu>"
"	<menu name='Select Fonts' action='Select Fonts'>"
"		<menuitem name='Japanese' action='Japanese'/>"
"		<menuitem name='English' action='English'/>"
"	</menu>"
"	<menu name='System Clock' action='System Clock'>"
"		<menuitem name='4MHz' action='4MHz'/>"
"		<menuitem name='6MHz' action='6MHz'/>"
"		<menuitem name='8MHz' action='8MHz'/>"
"		<menuitem name='400MHz' action='400MHz'/>"
"	</menu>"
"	<menuitem name='Hardware Speed Settings' action='Hardware Speed Settings'/>"
"	<menuitem name='Cons-R Blank Col.' action='Cons-R Blank Col.'/>"
"	<menuitem name='V-blank Special mode' action='V-blank Special mode'/>"
"	<menuitem name='Force Read Only mode' action='Force Read Only mode'/>"
"	<menuitem name='Auto Init. IO on Reset' action='Auto Init. IO on Reset'/>"
"	<menuitem name='IPL Memory Initialize' action='IPL Memory Initialize'/>"
"	<menuitem name='Memory Init. is Zero Clear' action='Memory Init. is Zero Clear'/>"
"	<separator/>"
"	<menuitem name='Device Configurations' action='Device Configurations'/>"
"	<menuitem name='Enable Sounds mode' action='Enable Sounds mode'/>"
"	<menuitem name='Sounds Time Interpolations' action='Sounds Time Interpolations'/>"
"</menu>"
"<menu name='View' action='View'>"
"	<menuitem name='Window Size x 1' action='Window Size x 1'/>"
"	<menuitem name='Window Size x 1.5' action='Window Size x 1.5'/>"
"	<menuitem name='Window Size x 2' action='Window Size x 2'/>"
"	<menuitem name='Full Screen mode' action='Full Screen mode'/>"
"</menu>"

#ifdef TRACE
"<menu name='Debugs' action='Debugs'>"
"	<menuitem name='Print Regs.' action='Print Regs.'/>"
"	<menuitem name='Traces Off' action='Traces Off'/>"
"	<menuitem name='Traces On' action='Traces On'/>"
"</menu>"
#endif

"<menu name='Help' action='Help'>"
"	<menuitem name='About' action='About'/>"
"</menu>"

"</menubar></ui>";

static GtkActionEntry menu_entries[] = {
	{"File", NULL, "_File",	NULL,			NULL, NULL },
	{"Reset", NULL, "_Reset", "<Alt>F11",		"Reset", uigtk_menu_reset },
	{"IPL Reset", NULL, "_IPL Reset", "<Alt>F12",	"IPL Reset", uigtk_menu_iplreset },
	{"States Load", NULL, "States _Load...", NULL,	"State Load", uigtk_menu_loadstate },
	{"States Save", NULL, "States _Save", "<Alt>F8", "States Save", uigtk_menu_savestate },
	{"States Save As", NULL, "States Save _As...", NULL,	"States Save As", uigtk_menu_savestateas },
	{"Copy to Clipboard", NULL, "_Copy to Clipboad", NULL,	"Screen Copy to Clipboard", uigtk_menu_copy },
	{"Paste", NULL, "Pas_te", NULL,	"Paste from Clipboard", uigtk_menu_paste },
	{"Paste Stop", NULL, "Paste St_op", NULL,	"Stop the paste typing", uigtk_menu_pastestop },
	{"Paste Settings", NULL, "Paste S_ettings...", NULL,	"Open Paste Settings Dialog", uigtk_menu_pastesettings },
	{"Screen Shots", NULL, "_Screen Shots...", NULL,	"Output the Screen Shots", uigtk_menu_screenshot },
	{"Exit", NULL, "E_xit",	NULL,			"Exit", uigtk_menu_quit },

	{"CasTape", NULL, "Cas_Tape", NULL,		NULL, NULL },
	{"Change or Exec", NULL, "_Change or Exec...", NULL, "Change Tape or Execute it", uigtk_menu_tapechange },
	{"Eject Cas", NULL, "_Eject\t\tAlt+__", NULL,	"Eject the Casette Tape", uigtk_menu_tapeeject },
	{"Stop", NULL,  "_Stop([])\tAlt+?", NULL,	"Cassette Tape to Stop", uigtk_menu_tapestop },
	{"Rew", NULL, "_Rew(<<)\tAlt+,", NULL,		"Cassette Tape to Rew", uigtk_menu_taperew },
	{"FF", NULL, "_FF(>>)\t\tAlt+.", NULL,		"Cassette Tape to FF", uigtk_menu_tapeff },
	{"Rew to Head", NULL, "Rew to _Head(|<<)", NULL, "Rewind to Head", uigtk_menu_tapetohead },
	{"Settings", NULL, "Se_ttings...", NULL,	"Open the Tape Settings Dialog", uigtk_menu_tapesettings },
	{"Counter Types", NULL, "Co_unter Types",	NULL,	NULL, NULL },

	{"FD", NULL, "F_D", NULL,			NULL, NULL },
	{"Drive 1", NULL, "Drive _1", NULL,		NULL, NULL },
	{"Change1", NULL, "_Change...", NULL,		"Change drive 1", uigtk_menu_diskdr1change },
	{"Eject1", NULL, "_Eject", NULL,		"Eject drive 1", uigtk_menu_diskdr1eject },
	{"Writeback1", NULL, "_Writeback", NULL,	"Writeback to drive 1", uigtk_menu_diskdr1wb },
	{"Drive 2", NULL, "Drive _2", NULL,		NULL, NULL },
	{"Change2", NULL, "_Change...", NULL,		"Change drive 2", uigtk_menu_diskdr2change },
	{"Eject2", NULL, "_Eject", NULL,		"Eject drive 2", uigtk_menu_diskdr2eject },
	{"Writeback2", NULL, "_Writeback", NULL,	"Writeback to drive 2", uigtk_menu_diskdr2wb },
	{"Drive 3", NULL, "Drive _3", NULL,		NULL, NULL },
	{"Change3", NULL, "_Change...", NULL,		"Change drive 3", uigtk_menu_diskdr3change },
	{"Eject3", NULL, "_Eject", NULL,		"Eject drive 3", uigtk_menu_diskdr3eject },
	{"Writeback3", NULL, "_Writeback", NULL,	"Writeback to drive 3", uigtk_menu_diskdr3wb },
	{"Drive 4", NULL, "Drive _4", NULL,		NULL, NULL },
	{"Change4", NULL, "_Change...", NULL,		"Change drive 4", uigtk_menu_diskdr4change },
	{"Eject4", NULL, "_Eject", NULL,		"Eject drive 4", uigtk_menu_diskdr4eject },
	{"Writeback4", NULL, "_Writeback", NULL,	"Writeback to drive 4", uigtk_menu_diskdr4wb },

	{"QD", NULL, "_QD", NULL,			NULL, NULL },
	{"Change QD", NULL, "_Change...", NULL,		"Change QD", uigtk_menu_qdchange },
	{"Eject QD", NULL, "_Eject", NULL,		"Eject QD", uigtk_menu_qdeject },

	{"Tools", NULL, "Too_ls", NULL,			NULL, NULL },
	{"Plugins Support Settings", NULL, "_Plugins Support Settings...", NULL, "Open the Plugins Support Settings Dialog", uigtk_menu_pluginsettings },
	{"Keydef Selects", NULL, "_Keydef Selects", NULL,	NULL, NULL },
	{"Display Colors", NULL, "_Display Colors", NULL,	NULL, NULL },
	{"Display Resolutions", NULL, "Display _Resolutions", NULL,  NULL, NULL },
	{"Select Fonts", NULL, "_Select Fonts", NULL,		NULL, NULL },
	{"System Clock", NULL, "System _Clock",	NULL,		NULL, NULL },
	{"Hardware Speed Settings", NULL, "_Hardware Speed Settings...", NULL, "Open the Hardware Speed Settings Dialog", uigtk_menu_speedsettings },
	{"Device Configurations", NULL, "D_evice Configurations...", NULL, "Open the Device Configurations Dialog", uigtk_menu_deviceconfigs },
	{"View", NULL, "_View",	NULL,			NULL, NULL },

#ifdef TRACE
	{"Debugs", NULL, "Deb_ugs", NULL,		NULL, NULL },
	{"Print Regs.", NULL, "_Print Regs.", "<Alt>R",	"Output the Registers to Console", print_registers },
#endif

	{"Help", NULL, "_Help",	NULL,			NULL, NULL },
	{"About", NULL, "_About...",	NULL,		"Open the Application Information Dialog", uigtk_menu_about }
};

static GtkToggleActionEntry menu_entries_toggle[] = {
	{"Pause", NULL, "_Pause", "<Alt>P",		"Pause", uigtk_menu_pause, FALSE },
	{"MZ-2000,2200", NULL, "MZ-_2000,2200", NULL,	"Change to MZ-2000/2200 mode", uigtk_menu_iplreset2000, FALSE },
	{"MZ-80B,80B2", NULL, "MZ-_80B,80B2", NULL,	"Change to MZ-80B/80B2 mode", uigtk_menu_iplreset80b, FALSE },

	{"En. SpeakerSlow", NULL, "En. Spe_aker&Slow",	NULL, "Enable to Tape x1 mode and speaker", uigtk_menu_tapespeaker, FALSE },
	{"MZ-2000,2200,80B", NULL, "MZ-_2000,2200,80B",NULL,  "Change Tape Counter to MZ-2000/80B", uigtk_menu_settapecounter_mz2080, FALSE },
	{"MZ-2500", NULL, "MZ-2_500", NULL,		"Change Tape Counter to MZ-2500", uigtk_menu_settapecounter_mz2500, FALSE },
	{"Kbytes", NULL, "_Kbytes", NULL,		"Change Tape Counter to KBytes", uigtk_menu_settapecounter_kbytes, FALSE },
	{"bytes", NULL, "_bytes", NULL,			"Change Tape Counter to Bytes", uigtk_menu_settapecounter_bytes, FALSE },
	{"Drop Auto IPL Cas", NULL, "Drop _Auto IPL mode", NULL, "Enable Auto IPL after Tape File Drops", uigtk_menu_dropautoipltape, FALSE },
	{"Enable Sound Cas", NULL, "Enable S_ound mode", "<Alt>S", "Enable to Casette Tape Sounds", uigtk_menu_enablesoundtape, FALSE },

	{"Enable IF", NULL, "Enable _IF", NULL,		"Enable to a FDD Interface", uigtk_menu_enablefddif, FALSE },
	{"Fast Running mode", NULL, "Fast _Running mode", NULL,	"Fast Running FDD Access mode", uigtk_menu_enablefastrunning, FALSE },
	{"Ignore Read CRC checks", NULL, "I_gnore Read CRC checks", NULL,
	 						"Ignore FDD Read CRC Check", uigtk_menu_enableignorereadcrcchk, FALSE },
	{"In format, erase 17sct CRC", NULL, "In format, _erase 17sct CRC", NULL,
	 						"In FDD format, erase 17sct CRC", uigtk_menu_enableerase17sctcrc, FALSE },
	{"Drop to FD2 mode", NULL, "Drop to _FD2 mode", NULL,	"Change the Drop Drive to Drive 2 from Drive 1", uigtk_menu_droptofd2, FALSE },
	{"Drop Auto IPL FD", NULL, "Drop _Auto IPL mode", NULL, "Enable to Auto IPL after FD File Drops", uigtk_menu_dropautoiplfd, FALSE },
	{"Enable Sound FD", NULL, "Enable _Sound mode", NULL, "Enable to FDD Sounds", uigtk_menu_enablesoundfd, FALSE },

	{"Cursor Keys=2468", NULL, "Cursor Keys=_2468", "<Alt>1", "Set the Keyboard mode to Cursor Keys=2468", uigtk_menu_setkeyscursor_1, FALSE },
	{"Cursor Keys=[00]135", NULL, "Cursor Keys=[00]_135", "<Alt>2", "Set the Keyboard mode to Cursor Keys=[00]135", uigtk_menu_setkeyscursor_2, FALSE },
	{"Ten Keys 0=[00]", NULL, "Ten Keys _0=[00]", "<Alt>3", "Set the Keyboard mode to Ten Key 0=[00]", uigtk_menu_setkeysten, FALSE },
	{"ZX <-> TABSPACE", NULL, "_ZX <-> TABSPACE", "<Alt>4", "Change the Keyboard mode to ZX <-> TABSPACE", uigtk_menu_setkeyspctab, FALSE },
	{"GRPH <-> TAB", NULL, "_GRPH <-> TAB", "<Alt>5", "Change the Keyboard mode to GRPH <-> TAB", uigtk_menu_setkeygrphtab, FALSE },
	{"Ten Keys 1 2 3 = 0 [00] .", NULL, "Ten Keys 1 2 3 = 0 [00] .", "<Alt>6", "Set the Keyboard mode to Ten Keys 1 2 3 = 0 [00] .", uigtk_menu_setkey123to000, FALSE },

	{"Green", NULL, "_Green", "<Alt>G",		"Set the Display mode to Green", uigtk_menu_green, FALSE },
	{"Color", NULL, "_Color", "<Alt>C",		"Set the Display mode to Color", uigtk_menu_color, FALSE },

	{"15KHz", NULL, "_15KHz", "<Alt>N",		"Set the Display mode to 15KHz", uigtk_menu_sethighreso_disable, FALSE },
	{"24KHz", NULL, "_24KHz", "<Alt>B",		"Set the Display mode to 24KHz", uigtk_menu_sethighreso_enable, FALSE },

	{"Japanese", NULL, "_Japanese", NULL,		"Select the Fonts to Japanese", uigtk_menu_setfontlang_japanese, FALSE },
	{"English", NULL, "_English", NULL,		"Select the Fonts to English", uigtk_menu_setfontlang_english, FALSE },

	{"4MHz", NULL, "_4MHz\tAlt+]", NULL,		"Set the System Clock to 4MHz", uigtk_menu_setspeed_4MHz, FALSE },
	{"6MHz", NULL, "_6MHz",	NULL,			"Set the System Clock to 6MHz", uigtk_menu_setspeed_6MHz, FALSE },
	{"8MHz", NULL, "_8MHz",	NULL,			"Set the System Clock to 8MHz", uigtk_menu_setspeed_8MHz, FALSE },
	{"400MHz", NULL, "4_00MHz", NULL,		"Set the System Clock to 400MHz", uigtk_menu_setspeed_400MHz, FALSE },

	{"Cons-R Blank Col.", NULL, "Co_ns-R Blank Col.", NULL,		"Enable the Console Reverse Blank on Color Display", uigtk_menu_consrblankcol, FALSE },
	{"V-blank Special mode", NULL, "_V-blank Special mode", NULL,	"Enable the V-blank Special mode", uigtk_menu_vblankspecial, FALSE },
	{"Force Read Only mode", NULL, "Force Read _Only mode", NULL,	"Enable the Force Read Only mode", uigtk_menu_forcereadonly, FALSE },
	{"Auto Init. IO on Reset", NULL, "_Auto Init. IO on Reset", NULL, "Enable the Auto Initialize IO on Reset", uigtk_menu_enableinitreset, FALSE },
	{"IPL Memory Initialize", NULL, "_IPL Memory Initialize", NULL,	"Enable the Memory Initialize on IPL Reset", uigtk_menu_iplmemclr, FALSE },
	{"Memory Init. is Zero Clear", NULL, "Memory Init. is _Zero Clear", NULL, "Enable to Memory Initialize is Zero Clear on IPL Reset", uigtk_menu_iplmemzclr, FALSE },
	{"Enable Sounds mode", NULL, "Enable _Sounds mode",	"<Alt>S", "Enable the All Sounds mode", uigtk_menu_enablesound, FALSE },
	{"Sounds Time Interpolations", NULL, "Sounds Time Interpolations(_0)",	NULL, "Enable the Sounds Time Interpolations", uigtk_menu_sounditpolations, FALSE },

	{"Window Size x 1", NULL, "Window Size x _1", NULL,	"Set the Window Size to x1", uigtk_menu_setwinsize_x1 },
	{"Window Size x 1.5", NULL, "Window Size x 1._5", NULL,	"Set the Window Size to x1.5", uigtk_menu_setwinsize_x1_5 },
	{"Window Size x 2", NULL, "Window Size x _2", NULL,	"Set the Window Size to x2", uigtk_menu_setwinsize_x2 },
	{"Full Screen mode", NULL, "_Full Screen mode", "<Alt>A", "Set to Fullscreen mode", uigtk_menu_setwinsize_fullscreen, FALSE },
};

#ifdef TRACE
static GtkRadioActionEntry menu_entries_trace[] = {
	{"Traces Off", NULL, "Traces O_ff", NULL,	"Disable the Trace Outputs", FALSE },
	{"Traces On", NULL, "_Traces On", NULL,		"Enable the Trace Outputs", TRUE },
};
#endif

void uigtk_create_menu( GtkWidget *vbox )
{
	GtkWidget *menubar;

	/* menu bar */
	uigtk_ui = gtk_ui_manager_new();
	uigtk_actions = gtk_action_group_new( "rootmenu" );
	gtk_action_group_add_actions( uigtk_actions,
		menu_entries, sizeof(menu_entries)/sizeof(menu_entries[0]), NULL );
	gtk_action_group_add_toggle_actions( uigtk_actions,
		menu_entries_toggle, sizeof(menu_entries_toggle)/sizeof(menu_entries_toggle[0]),
		NULL );
#ifdef TRACE
	gtk_action_group_add_radio_actions( uigtk_actions,
		menu_entries_trace, sizeof(menu_entries_trace)/sizeof(menu_entries_trace[0]),
		FALSE, G_CALLBACK(uigtk_menu_trace), NULL );
#endif
	gtk_ui_manager_insert_action_group( uigtk_ui, uigtk_actions, 0 );
	gtk_ui_manager_set_add_tearoffs( uigtk_ui, TRUE );
	gtk_ui_manager_add_ui_from_string( uigtk_ui, menu_items, -1, NULL );
	gtk_window_add_accel_group( GTK_WINDOW(uigtk_window), gtk_ui_manager_get_accel_group(uigtk_ui) );
	g_object_set_data( G_OBJECT(uigtk_window), "ui", (gpointer)uigtk_ui );
	menubar = gtk_ui_manager_get_widget( uigtk_ui, "/root_menu_bar" );
	if (!menubar) {
		fprintf( stderr, "/root_menu_bar open error: val=%08lx\n", (unsigned long)menubar );
		exit(1);
	}
	gtk_box_pack_start( GTK_BOX(vbox), menubar, FALSE, TRUE, 0 );
	gtk_widget_show( menubar );

	uigtk_menu_externalchg = FALSE;
	return;
}

/*
	Local Variables:
	mode:c++
	c-set-style:"k&r"
	c-basic-offset:8
	tab-width:8
	End:
*/
