/* File: uigtk_pastedlg.c

	SHARP MZ-2000/2200/80B/B2 Emulator "emz2000"
		libgtk modules for a paste settings dialog

	Copyright (C) 2003-2019 FUKUI, Toshio.
	This file is part of the emz2000 software.
	See copyright notice in the COPYING file.
*/

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <gtk/gtk.h>

#include "common.h"
#include "mz2000.h"
#include "uigtk.h"

static MZ2000 *mz;
static GtkRadioButton *radiobuttons[PASTE_MONITOR_MAXNUM];
static GtkEntry *entry_normalwait, *entry_crwait;

int uigtk_pastedlg_domodal( MZ2000 *_mz )
{
	int i, t;
	char buf[LNMAX];
	gint r;
	const gchar *s;
	GtkBuilder *builder;
	GtkDialog *dialog;

	mz = _mz;
	builder = gtk_builder_new();
	r = gtk_builder_add_from_file( builder, GLADEUIFILE, NULL );
	dialog = GTK_DIALOG( gtk_builder_get_object( builder, "paste_dialog" ) );
#ifdef EMZ2000_LIBDIR
	if (/* r || */ !dialog) {
		r = gtk_builder_add_from_file( builder, EMZ2000_LIBDIR "/" GLADEUIFILE, NULL );
		dialog = GTK_DIALOG( gtk_builder_get_object( builder, "paste_dialog" ) );
	}
#endif
	if (/* r || */ !dialog) {
		fprintf( stderr, GLADEUIFILE " and " EMZ2000_LIBDIR "/" GLADEUIFILE " not found or broken.\n" );
		g_object_unref( builder );
		return UIGTK_IDCANCEL;
	}
	gtk_builder_connect_signals( builder, NULL );
	//gtk_widget_set_parent_window( GTK_WIDGET(dialog), uigtk_window );

	/* Environment */
	radiobuttons[0] = GTK_RADIO_BUTTON( gtk_builder_get_object( builder, "radiobutton_normal" ) );
	radiobuttons[1] = GTK_RADIO_BUTTON( gtk_builder_get_object( builder, "radiobutton_pupmon" ) );
	radiobuttons[2] = GTK_RADIO_BUTTON( gtk_builder_get_object( builder, "radiobutton_tstf" ) );
	radiobuttons[3] = GTK_RADIO_BUTTON( gtk_builder_get_object( builder, "radiobutton_hu" ) );
	radiobuttons[4] = GTK_RADIO_BUTTON( gtk_builder_get_object( builder, "radiobutton_db" ) );
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(radiobuttons[paste_mode]), TRUE );
	/* Wait Time */
	entry_normalwait = GTK_ENTRY( gtk_builder_get_object( builder, "entry_normalwait" ) );
	snprintf( buf, LNMAX, "%d", paste_wait );
	gtk_entry_set_text( entry_normalwait, buf );
	entry_crwait = GTK_ENTRY( gtk_builder_get_object( builder, "entry_crwait" ) );
	snprintf( buf, LNMAX, "%d", paste_waitcr );
	gtk_entry_set_text( entry_crwait, buf );

	gtk_dialog_add_buttons( dialog,
		GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
		"DEFAULT", GTK_RESPONSE_NO, NULL );

retry:	r = gtk_dialog_run( GTK_DIALOG(dialog) );
	switch (r) {
	case GTK_RESPONSE_ACCEPT :
		/* Environment */
		for (i = 0; i < PASTE_MONITOR_MAXNUM; i++) {
			if (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(radiobuttons[i]) )) {
				paste_mode = i;
				break;
			}
		}
		/* Wait Time */
		s = gtk_entry_get_text( entry_normalwait );
		if (sscanf( s, "%d", &t ) != 1 || t < 1 || t > 100) {
			uigtk_messagebox( "Wait time value range error",
				"Paste Settings", UIGTK_BUTTON_OKONLY | UIGTK_ICON_ERROR );
			goto retry;
		}
		paste_wait = t;
		/* Wait Time for CR */
		s = gtk_entry_get_text( entry_crwait );
		if (sscanf( s, "%d", &t ) != 1 || t < 1 || t > 100) {
			uigtk_messagebox( "Wait time for CR value range error",
				"Paste Settings", UIGTK_BUTTON_OKONLY | UIGTK_ICON_ERROR );
			goto retry;
		}
		paste_waitcr = t;
		gtk_widget_destroy( GTK_WIDGET(dialog) );
		g_object_unref( builder );
		return UIGTK_IDOK;
	case GTK_RESPONSE_NO :
		/* Environment */
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(radiobuttons[PASTE_MONITOR_DEFAULT]), TRUE );
		/* Wait Time */
		snprintf( buf, LNMAX, "%d", PASTE_WAIT_DEFAULT );
		gtk_entry_set_text( entry_normalwait, buf );
		snprintf( buf, LNMAX, "%d", PASTE_WAITCR_DEFAULT );
		gtk_entry_set_text( entry_crwait, buf );
		goto retry;
	case GTK_RESPONSE_CANCEL :
	default : /* close button */
		break;
	}
	gtk_widget_destroy( GTK_WIDGET(dialog) );
	g_object_unref( builder );
	return UIGTK_IDCANCEL;
}

/*
	Local Variables:
	mode:c++
	c-set-style:"k&r"
	c-basic-offset:8
	tab-width:8
	End:
*/
