/* File: uigtk_plugindlg.c

	SHARP MZ-2000/2200/80B/B2 Emulator "emz2000"
		libgtk modules for plugins supports dialog

 	Created by FUKUI, Toshio and TSUNEOKA, Shinji.
	This file is part of the emz2000 software.
	See copyright notice in the COPYING file.
*/

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <gtk/gtk.h>

#include "common.h"
#include "mz2000.h"
#include "uigtk.h"
#include "plugin.h"

static MZ2000 *mz;
static GtkButton *button_custom;
static GtkRadioButton *slotbuttons[PLUGIN_BOARD_MAXNUM];
static GtkLabel *label_plugininfo;
static int current_select = -1;

static char *_encode_amp( const char *cp, char *wbuf )
{
	strncpy( wbuf, cp, LNMAX );
	wbuf[LNMAX - 1] = '\0';
	return wbuf;
}

static gint uigtk_plugindlg_select_event( GtkWidget *widget, GdkEventAny *event, void *user_ptr )
{
	int i;
	char buf[LNMAX];
	char tbuf1[LNMAX], tbuf2[LNMAX], tbuf3[LNMAX], tbuf4[LNMAX];
	PEMUZ2000EBIH pebih  = NULL;

	/* get current select */
	for (i = 0; i < PLUGIN_BOARD_MAXNUM; i++) {
		if (gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(slotbuttons[i]) ))
			break;
	}
	if (i >= PLUGIN_BOARD_MAXNUM)
		return TRUE;
	if (current_select == i)
		return TRUE;
	current_select = i;
	//_TRACE( "select: %d\n", i );

	if ((pebih = plugin_get_boardInformation(current_select)) == NULL) {
		gtk_label_set_text( label_plugininfo, "No Board Information" );
		gtk_widget_set_sensitive( GTK_WIDGET(button_custom), FALSE );
		return TRUE;
	}
	if (pebih -> flgs.val & USE_INFODIALOG)
		gtk_widget_set_sensitive( GTK_WIDGET(button_custom), TRUE );
	else
		gtk_widget_set_sensitive( GTK_WIDGET(button_custom), FALSE );

	switch (pebih->ioMapper.nCount) {
	case 0:
		snprintf( buf, sizeof(buf), "%s\n\n%s\n\n%s\n%s\n",
			_encode_amp(pebih->szBoardName, tbuf1),
			_encode_amp(pebih->szDllCopyright, tbuf2),
			_encode_amp(pebih->szDllMakerMail, tbuf3),
			_encode_amp(pebih->szDllMakerUrl, tbuf4) );
		break;
	case 1:
		snprintf( buf, sizeof(buf), "%s\nrequired ports : $%02X\n\n%s\n\n%s\n%s\n",
			_encode_amp(pebih->szBoardName, tbuf1),
			pebih->ioMapper.ucStartPort,
			_encode_amp(pebih->szDllCopyright, tbuf2),
			_encode_amp(pebih->szDllMakerMail, tbuf3),
			_encode_amp(pebih->szDllMakerUrl, tbuf4) );
		break;
	default:
		snprintf( buf, sizeof(buf), "%s\nrequired ports : $%02X - $%02X\n\n%s\n\n%s\n%s\n",
			_encode_amp(pebih->szBoardName, tbuf1),
			pebih->ioMapper.ucStartPort, pebih->ioMapper.ucStartPort + pebih->ioMapper.nCount - 1,
			_encode_amp(pebih->szDllCopyright, tbuf2),
			_encode_amp(pebih->szDllMakerMail, tbuf3),
			_encode_amp(pebih->szDllMakerUrl, tbuf4) );
		break;
	}
	gtk_label_set_text( label_plugininfo, buf );
	return TRUE;
}

static gint uigtk_plugindlg_custom_event( GtkWidget *widget, GdkEventAny *event, void *user_ptr )
{
	plugin_show_informationDialog( (void *)uigtk_window, current_select );
	return TRUE;
}

int uigtk_plugindlg_domodal( MZ2000 *_mz )
{
	int i, items, st;
	gint r;
	GtkBuilder *builder;
	GtkDialog *dialog;

	mz = _mz;
	builder = gtk_builder_new();
	r = gtk_builder_add_from_file( builder, GLADEUIFILE, NULL );
	dialog = GTK_DIALOG( gtk_builder_get_object( builder, "plugin_dialog" ) );
#ifdef EMZ2000_LIBDIR
	if (/* r || */ !dialog) {
		r = gtk_builder_add_from_file( builder, EMZ2000_LIBDIR "/" GLADEUIFILE, NULL );
		dialog = GTK_DIALOG( gtk_builder_get_object( builder, "plugin_dialog" ) );
	}
#endif
	if (/* r || */ !dialog) {
		fprintf( stderr, GLADEUIFILE " and " EMZ2000_LIBDIR "/" GLADEUIFILE " not found or broken.\n" );
		g_object_unref( builder );
		return UIGTK_IDCANCEL;
	}
	gtk_builder_connect_signals( builder, NULL );
	//gtk_widget_set_parent_window( GTK_WIDGET(dialog), uigtk_window );

	button_custom = GTK_BUTTON( gtk_builder_get_object( builder, "button_custom" ) );
	label_plugininfo = GTK_LABEL( gtk_builder_get_object( builder, "label_plugininfo" ) );

	current_select = -1;
	st = -1;
	items = 0;
	for (i = 0; i < PLUGIN_BOARD_MAXNUM; i++) {
		char buf[LNMAX];
		PEMUZ2000EBIH pebih;

		snprintf( buf, LNMAX, "radiobutton_slot%d", i );
		slotbuttons[i] = GTK_RADIO_BUTTON( gtk_builder_get_object( builder, buf ) );
		g_signal_connect( slotbuttons[i], "clicked",
			G_CALLBACK(uigtk_plugindlg_select_event), NULL );
		pebih = plugin_get_boardInformation( i );
		if (!pebih) {
			gtk_widget_set_sensitive( GTK_WIDGET(slotbuttons[i]), FALSE );
			continue;
		}
		if (st < 0)
			st = i;
		items++;
	}
	if (st < 0)
		st = 0;
	if (!items) {
		gtk_widget_set_sensitive( GTK_WIDGET(button_custom), FALSE );
		gtk_label_set_text( label_plugininfo, "Plugins are not installed." );
	} else {
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(slotbuttons[st]), TRUE );
		uigtk_plugindlg_select_event( NULL, NULL, NULL );
	}
	g_signal_connect( button_custom, "clicked",
		G_CALLBACK(uigtk_plugindlg_custom_event), NULL );

	gtk_dialog_add_buttons( dialog, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL );

	r = gtk_dialog_run( GTK_DIALOG(dialog) );
	switch (r) {
	case GTK_RESPONSE_ACCEPT :
		gtk_widget_destroy( GTK_WIDGET(dialog) );
		g_object_unref( builder );
		return UIGTK_IDOK;
	default : /* close button */
		break;
	}
	gtk_widget_destroy( GTK_WIDGET(dialog) );
	g_object_unref( builder );
	return UIGTK_IDCANCEL;
}

/*
	Local Variables:
	mode:c++
	c-set-style:"k&r"
	c-basic-offset:8
	tab-width:8
	End:
*/
