/* File: uigtk_tapedlg.c

	SHARP MZ-2000/2200/80B/B2 Emulator "emz2000"
		libgtk modules for a cassette tape deck settings dialog

	Copyright (C) 2003-2019 FUKUI, Toshio.
	This file is part of the emz2000 software.
	See copyright notice in the COPYING file.
*/

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <gtk/gtk.h>

#include "common.h"
#include "mz2000.h"
#include "uigtk.h"

static MZ2000 *mz;
static GtkAdjustment *adj_pitch, *adj_threshold, *adj_amplitude, *adj_center;
static GtkCheckButton *chk_reverse, *chk_ignoreff, *chk_writelinear;

int uigtk_tapedlg_domodal( MZ2000 *_mz )
{
	gint r;
	GtkBuilder *builder;
	GtkDialog *dialog;

	mz = _mz;
	builder = gtk_builder_new();
	r = gtk_builder_add_from_file( builder, GLADEUIFILE, NULL );
	dialog = GTK_DIALOG( gtk_builder_get_object( builder, "tape_dialog" ) );
#ifdef EMZ2000_LIBDIR
	if (/* r || */ !dialog) {
		r = gtk_builder_add_from_file( builder, EMZ2000_LIBDIR "/" GLADEUIFILE, NULL );
		dialog = GTK_DIALOG( gtk_builder_get_object( builder, "tape_dialog" ) );
	}
#endif
	if (/* r || */ !dialog) {
		fprintf( stderr, GLADEUIFILE " and " EMZ2000_LIBDIR "/" GLADEUIFILE " not found or broken.\n" );
		g_object_unref( builder );
		return UIGTK_IDCANCEL;
	}
	gtk_builder_connect_signals( builder, NULL );
	//gtk_widget_set_parent_window( GTK_WIDGET(dialog), uigtk_window );

	adj_pitch = GTK_ADJUSTMENT( gtk_builder_get_object( builder, "adjustment_pitch" ) );
	gtk_adjustment_set_value( adj_pitch, (gdouble)mz -> tapePitch );
	adj_threshold = GTK_ADJUSTMENT( gtk_builder_get_object( builder, "adjustment_threshold" ) );
	gtk_adjustment_set_value( adj_threshold, (gdouble)mz -> tapeThreshold );
	adj_amplitude = GTK_ADJUSTMENT( gtk_builder_get_object( builder, "adjustment_amplitude" ) );
	gtk_adjustment_set_value( adj_amplitude, (gdouble)mz -> amplitudeRange );
	adj_center = GTK_ADJUSTMENT( gtk_builder_get_object( builder, "adjustment_center" ) );
	gtk_adjustment_set_value( adj_center, (gdouble)mz -> writeThreshold );
	chk_reverse = GTK_CHECK_BUTTON( gtk_builder_get_object( builder, "check_reverse" ) );
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(chk_reverse), mz -> tapemode_reverse );
	chk_ignoreff = GTK_CHECK_BUTTON( gtk_builder_get_object( builder, "check_ignoreff" ) );
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(chk_ignoreff), mz -> tapeff_ignore );
	chk_writelinear = GTK_CHECK_BUTTON( gtk_builder_get_object( builder, "check_writelinear" ) );
	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(chk_writelinear), mz -> tapewrite_interpolation );

	gtk_dialog_add_buttons( dialog,
		GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
		"DEFAULT", GTK_RESPONSE_NO, NULL );

retry:	r = gtk_dialog_run( GTK_DIALOG(dialog) );
	switch (r) {
	case GTK_RESPONSE_ACCEPT :
		mz -> tapePitch = (int)gtk_adjustment_get_value( adj_pitch );
		mz -> tapeThreshold = (int)gtk_adjustment_get_value( adj_threshold );
		mz -> amplitudeRange = (int)gtk_adjustment_get_value( adj_amplitude );
		mz -> writeThreshold = (int)gtk_adjustment_get_value( adj_center );
		mz -> tapemode_reverse = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(chk_reverse) );
		mz -> tapeff_ignore = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(chk_ignoreff) );
		mz -> tapewrite_interpolation = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(chk_writelinear) );
		gtk_widget_destroy( GTK_WIDGET(dialog) );
		g_object_unref( builder );
		return UIGTK_IDOK;
	case GTK_RESPONSE_NO :
		gtk_adjustment_set_value( adj_pitch, (gdouble)DEFAULT_TAPE_PITCH );
		gtk_adjustment_set_value( adj_threshold, (gdouble)DEFAULT_TAPE_THRESHOLD );
		gtk_adjustment_set_value( adj_amplitude, (gdouble)DEFAULT_TAPE_AMPLITUDERANGE );
		gtk_adjustment_set_value( adj_center, (gdouble)DEFAULT_TAPE_WRITETHRESHOLD );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(chk_reverse), FALSE );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(chk_ignoreff), FALSE );
		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(chk_writelinear), FALSE );
		goto retry;
	case GTK_RESPONSE_CANCEL :
	default : /* close button */
		break;
	}
	gtk_widget_destroy( GTK_WIDGET(dialog) );
	g_object_unref( builder );
	return UIGTK_IDCANCEL;
}

/*
	Local Variables:
	mode:c++
	c-set-style:"k&r"
	c-basic-offset:8
	tab-width:8
	End:
*/
