#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "tape.h"

int error(char *argv);

int main(int argc, char **argv)
{
	char *infile;
	char *outfile;
	char *filename = NULL;
	int staddr = 0;
	int jpaddr = 0;
	int size;
	int i;
	FILE *infp;
	TAPE *intp, *outtp;
	unsigned char body[65536+2+0x8a];
	unsigned char *header = body;
	unsigned char *inf = body + 0x08;
	unsigned char *dat = body + 0x8a;
	int sum;
	int format;
	char *formatstr;

	formatstr = argv[0] + strlen(argv[0]) - 4;

	if (!strcmp(formatstr, "2mzf")) {
		format = TAPE_FILE2000B;
	} else if (!strcmp(formatstr, "2rlf")) {
		format = TAPE_RUNLENGTH;
	} else if (!strcmp(formatstr, "2bin")) {
		format = TAPE_DATA2000B;
	} else {
		fprintf(stderr, "%s: invalid program name\n", argv[0]);
		exit(1);
	}

	for (i = 1; i < argc; i++) {
		if (argv[i][0] != '-') break;
		if (i == argc - 1) break;
		switch(argv[i][1]) {
		case 'n':
			filename = argv[++i];
			break;
		case 's':
			staddr = atoi(argv[++i]);
			break;
		case 'j':
			jpaddr = atoi(argv[++i]);
			break;
		default:
			error(argv[0]);
			break;
		}
	}
	if (i + 1 >= argc) error(argv[0]);
	infile = argv[i];
	outfile = argv[i+1];
	if (filename == NULL) filename = outfile;

	if ((infp = fopen(infile, "r")) == NULL) {
		fprintf(stderr, "%s: Can't read file\n", infile);
		exit(1);
	}

	for (size = 0; size < 65536; size++) {
		int c = fgetc(infp);
		if (c == EOF) break;
		dat[size] = c;
	}
	fclose(infp);

	memset(header, 0, 8);
	memset(inf, 0, 0x82);
	memcpy(header, "mz20", 4);
	header[5] = 0x02;
	inf[0] = 0x01;
	memset(inf+1, 0x20, 16);
	strcpy((char *)(inf+1), filename);
	inf[17] = 0x0d;
	inf[18] = size & 0xff;
	inf[19] = (size >> 8) & 0xff;
	inf[20] = staddr & 0xff;
	inf[21] = (staddr >> 8) & 0xff;
	inf[22] = jpaddr & 0xff;
	inf[23] = (jpaddr >> 8) & 0xff;
    
	sum = 0;
	for (i = 0; i < 0x80; i++) {
		int j;
		for (j = 1; j < 0x100; j = j * 2) {
			if (inf[i] & j) sum++;
		}
	}

	inf[0x80] = (sum >> 8) & 0xff;
	inf[0x81] = sum & 0xff;

	sum = 0;
	for (i = 0; i < size; i++) {
		int j;
		for (j = 1; j < 0x100; j = j * 2) {
			if (dat[i] & j) sum++;
		}
	}

	dat[size] = (sum >> 8) & 0xff;
	dat[size+1] = sum & 0xff;

	if ((intp = tropen_mem(body, size+2+0x8a)) == NULL) {
		fprintf(stderr, "%s: Unexpected error\n", outfile);
		exit(1);
	}

	if ((outtp = twopen(outfile, format)) == NULL) {
		fprintf(stderr, "%s: Can't write file\n", outfile);
		exit(1);
	}

	tcopy(intp, outtp, 90);
	tclose(intp);
	tclose(outtp);

	return 0;
}

int error(char *argv)
{
	fprintf(stderr, "%s [-n name] [-s addr] [-j addr] infile outfile\n", argv);
	exit(1);
}
